--MIT License
--
--Copyright (c) Stat Weights Classic
--
--Permission is hereby granted, free of charge, to any person obtaining a copy
--of this software and associated documentation files (the "Software"), to deal
--in the Software without restriction, including without limitation the rights
--to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
--copies of the Software, and to permit persons to whom the Software is
--furnished to do so, subject to the following conditions:
--
--The above copyright notice and this permission notice shall be included in all
--copies or substantial portions of the Software.
--
--THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
--IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
--FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
--AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
--LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
--OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
--SOFTWARE.

local _, class = UnitClass("player");
local _, race = UnitRace("player");

local function spell_name_to_ids()
    if class == "MAGE" then
        return {
            ["Frostbolt"]               = 116,
            ["Frost Nova"]              = 122,
            ["Cone of Cold"]            = 120,
            ["Blizzard"]                = 10,
            ["Fireball"]                = 133,
            ["Fire Blast"]              = 2136,
            ["Scorch"]                  = 2948,
            ["Pyroblast"]               = 11366,
            ["Blast Wave"]              = 11113,
            ["Flamestrike"]             = 2120,
            ["Arcane Missiles"]         = 5143,
            ["Arcane Explosion"]        = 1449,
            ["Arcane Blast"]            = 30451,
            ["Mana Shield"]             = 1463,
            ["Arcane Barrage"]          = 44425,
            ["Dragon's Breath"]         = 31661,
            ["Frostfire Bolt"]          = 44614,
            ["Living Bomb"]             = 44457,
            ["Ice Barrier"]             = 11426,
            ["Ice Lance"]               = 30455,
            ["Deep Freeze"]             = 44572,
        };
    elseif class == "DRUID" then
        return {
            ["Healing Touch"]           = 5185,
            ["Rejuvenation"]            = 774,
            ["Tranquility"]             = 740,
            ["Regrowth"]                = 8936,
            ["Moonfire"]                = 8921,
            ["Wrath"]                   = 5176,
            ["Starfire"]                = 2912,
            ["Insect Swarm"]            = 5570,
            ["Hurricane"]               = 16914,
            ["Entangling Roots"]        = 339,
            ["Mark of the Wild"]        = 1126,
            ["Gift of the Wild"]        = 21849,
            ["Lifebloom"]               = 33763,
            ["Wild Growth"]             = 48438,
            ["Nourish"]                 = 50464,
            ["Force of Nature"]         = 33831,
            ["Typhoon"]                 = 50516,
            ["Starfall"]                = 48505,
            ["Force of Nature"]         = 33831,
        };
    elseif class == "PRIEST" then
        return {
            ["Lesser Heal"]             = 2050,
            ["Heal"]                    = 2054,
            ["Greater Heal"]            = 2060,
            ["Flash Heal"]              = 2061,
            ["Prayer of Healing"]       = 596,
            ["Renew"]                   = 139,
            ["Power Word: Shield"]      = 17,
            ["Holy Nova"]               = 15237,
            ["Smite"]                   = 585,
            ["Holy Fire"]               = 14914,
            ["Mind Blast"]              = 8092,
            ["Shadow Word: Pain"]       = 589,
            ["Mind Flay"]               = 15407,
            ["Devouring Plague"]        = 2944,
            ["Divine Spirit"]           = 14752,
            ["Prayer of Spirit"]        = 27681,
            ["Binding Heal"]            = 32546,
            ["Circle of Healing"]       = 34861,
            ["Penance"]                 = 47540,
            ["Prayer of Mending"]       = 33076,
            ["Vampiric Touch"]          = 34914,
            ["Mind Sear"]               = 48045,
            ["Shadow Word: Death"]      = 32379,
            ["Lightwell"]               = 724,
            ["Shadowfiend"]             = 34433,
            ["Desperate Prayer"]        = 19236,
            ["Shadowform"]              = 15473,
            ["Divine Hymn"]             = 64843,
        };
    elseif class == "SHAMAN" then
        return {
            ["Healing Stream Totem"]    = 5394,
            ["Lesser Healing Wave"]     = 8004,
            ["Healing Wave"]            = 331,
            ["Chain Heal"]              = 1064,
            ["Lightning Bolt"]          = 403,
            ["Chain Lightning"]         = 421,
            ["Lightning Shield"]        = 324,
            ["Earth Shock"]             = 8042,
            ["Magma Totem"]             = 8190,
            ["Flame Shock"]             = 8050,
            ["Frost Shock"]             = 8056,
            ["Fire Nova"]               = 1535,
            ["Searing Totem"]           = 3599,
            ["Lava Burst"]              = 51505,
            ["Riptide"]                 = 61295,
            ["Earth Shield"]            = 974,
            ["Thunderstorm"]            = 51490,
        };
    elseif class == "PALADIN" then
        return {
            ["Flash of Light"]          = 19750,
            ["Holy Light"]              = 635,
            ["Holy Shock"]              = 20473,
            ["Sacred Shield"]           = 53601,
        };
    elseif class == "WARLOCK" then
        return {
            ["Curse of Agony"]          = 980,
            ["Death Coil"]              = 6789,
            ["Corruption"]              = 172,
            ["Drain Life"]              = 689,
            ["Drain Soul"]              = 1120,
            ["Shadow Bolt"]             = 686,
            ["Searing Pain"]            = 5676,
            ["Soul Fire"]               = 6353,
            ["Hellfire"]                = 1949,
            ["Rain of Fire"]            = 5740,
            ["Immolate"]                = 348,
            ["Shadowburn"]              = 17877,
            ["Curse of Doom"]           = 603,
            ["Unstable Affliction"]     = 30108,
            ["Seed of Corruption"]      = 27243,
            ["Shadowflame"]             = 47897,
            ["Haunt"]                   = 48181,
            ["Incinerate"]              = 29722,
            ["Inferno"]                 = 1122,
            ["Shadowfury"]              = 30283,
            ["Chaos Bolt"]              = 50796,
            ["Health Funnel"]           = 755,
            ["Shadow Ward"]             = 6229,
            ["Immolation Aura"]         = 50589,
            ["Shadow Cleave"]           = 50581,
        };
    else
        return {};
    end
end

local spell_name_to_id = spell_name_to_ids();

local localized_spell_names_to_id = {};
for _, v in pairs(spell_name_to_id) do
    local lname = GetSpellInfo(v);

    localized_spell_names_to_id[lname] = v;
end


local magic_school = {
     physical = 1,
     holy     = 2,
     fire     = 3,
     nature   = 4,
     frost    = 5,
     shadow   = 6,
     arcane   = 7
};

local spell_flags = {
    aoe                     = bit.lshift(1,1),
    snare                   = bit.lshift(1,2),
    heal                    = bit.lshift(1,3),
    absorb                  = bit.lshift(1,4),
    over_time_crit          = bit.lshift(1,5),
    cd                      = bit.lshift(1,6),
    over_time_range         = bit.lshift(1,7),
    channel_missable        = bit.lshift(1,8), -- e.g. missing mid flay only loses you one gcd instead of the whole cast
    curse                   = bit.lshift(1,9),
    over_time_lvl_scaling   = bit.lshift(1,10),
};


local function create_spells()
    
    if class == "MAGE" then
        return  {
            --frostbolts
            [116] = {
                base_min            = 18,
                base_max            = 20,
                over_time           = 0,
                over_time_tick_freq = 0,
                over_time_duration  = 0,
                cast_time           = 3.0,
                rank                = 1,
                lvl_req             = 4,
                lvl_max             = 8,
                lvl_outdated        = 7,
                cost_base_percent   = 0.11,
                flags               = spell_flags.snare,
                school              = magic_school.frost,
                coef                = 0.8571,
                over_time_coef      = 0.0,
                lvl_scaling         = 0.5,
            },
            [205] = {
                base_min            = 31,
                base_max            = 35,
                over_time           = 0,
                rank                = 2,
                lvl_req             = 8,
                lvl_max             = 12,
                lvl_outdated        = 13,
                lvl_scaling         = 0.7,
            },
            [837] = {
                base_min            = 51,
                base_max            = 57,
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 14,
                lvl_max             = 18,
                lvl_outdated        = 19,
                lvl_scaling         = 0.9,
            },
            [7322] = {
                base_min            = 74,
                base_max            = 82,
                over_time           = 0,
                rank                = 4,
                lvl_req             = 20,
                lvl_max             = 24,
                lvl_outdated        = 25,
                lvl_scaling         = 1.1,
            },
            [8406] = {
                base_min            = 126,
                base_max            = 138,
                over_time           = 0,
                rank                = 5,
                lvl_req             = 26,
                lvl_max             = 30,
                lvl_outdated        = 31,
                lvl_scaling         = 1.5,
            },
            [8407] = {
                base_min            = 174,
                base_max            = 190,
                over_time           = 0,
                rank                = 6,
                lvl_req             = 32,
                lvl_max             = 36,
                lvl_outdated        = 37,
                lvl_scaling         = 1.7,
            },
            [8408] = {
                base_min            = 227,
                base_max            = 247,
                over_time           = 0,
                rank                = 7,
                lvl_req             = 38,
                lvl_max             = 42,
                lvl_outdated        = 43,
                lvl_scaling         = 2,
            },
            [10179] = {
                base_min            = 292,
                base_max            = 316,
                over_time           = 0,
                rank                = 8,
                lvl_req             = 44,
                lvl_max             = 48,
                lvl_outdated        = 49,
                lvl_scaling         = 2.3,
            },
            [10180] = {
                base_min            = 353,
                base_max            = 383,
                over_time           = 0,
                rank                = 9,
                lvl_req             = 50,
                lvl_max             = 54,
                lvl_outdated        = 55,
                lvl_scaling         = 2.6,
            },
            [10181] = {
                base_min            = 429,
                base_max            = 463,
                over_time           = 0,
                rank                = 10,
                lvl_req             = 56,
                lvl_max             = 60,
                lvl_outdated        = 59,
                lvl_scaling         = 2.9,
            },
            [25304] = {
                base_min            = 515,
                base_max            = 555,
                over_time           = 0,
                rank                = 11,
                lvl_req             = 60,
                lvl_max             = 64,
                lvl_outdated        = 62,
                lvl_scaling         = 3.2,
            },
            [27071] = {
                base_min            = 536,
                base_max            = 578,
                over_time           = 0,
                rank                = 12,
                lvl_req             = 63,
                lvl_max             = 67,
                lvl_outdated        = 68,
                lvl_scaling         = 3.2,
            },
            [27072] = {
                base_min            = 597,
                base_max            = 643,
                over_time           = 0,
                rank                = 13,
                lvl_req             = 69,
                lvl_max             = 73,
                lvl_outdated        = 69,
                lvl_scaling         = 3.5,
            },
            [38697] = {
                base_min            = 630,
                base_max            = 680,
                over_time           = 0,
                rank                = 14,
                lvl_req             = 70,
                lvl_max             = 74,
                lvl_outdated        = 74,
                lvl_scaling         = 3.8,
            },
            [42841] = {
                base_min            = 702,
                base_max            = 758,
                over_time           = 0,
                rank                = 15,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 78,
                lvl_scaling         = 4.2,
            },
            [42842] = {
                base_min            = 799,
                base_max            = 861,
                over_time           = 0,
                rank                = 16,
                lvl_req             = 79,
                lvl_max             = 83,
                lvl_outdated        = 80,
                lvl_scaling         = 4.8,
            },
            -- frost nova
            -- wowhead coef used
            [122] = {
                base_min            = 19,
                base_max            = 21,
                over_time           = 0,
                over_time_tick_freq = 0,
                over_time_duration  = 0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 10,
                lvl_max             = 15,
                lvl_outdated        = 25,
                cost_base_percent   = 0.07,
                flags               = bit.bor(spell_flags.snare, spell_flags.aoe, spell_flags.cd),
                school              = magic_school.frost,
                coef                = 0.043,
                over_time_coef      = 0.0,
                lvl_scaling         = 0.5,
            },
            [865] = {
                base_min            = 33,
                base_max            = 37,
                over_time           = 0,
                rank                = 2,
                lvl_req             = 26,
                lvl_max             = 31,
                lvl_outdated        = 39,
                lvl_scaling         = 0.5,
            },
            [6131] = {
                base_min            = 52,
                base_max            = 58,
                over_time           = 0,
                rank                = 1,
                lvl_req             = 40,
                lvl_max             = 45,
                lvl_outdated        = 53,
                lvl_scaling         = 0.5,
            },
            [10230] = {
                base_min            = 70,
                base_max            = 80,
                over_time           = 0,
                rank                = 4,
                lvl_req             = 54,
                lvl_max             = 59,
                lvl_outdated        = 66,
                lvl_scaling         = 0.5,
            },
            [27088] = {
                base_min            = 230,
                base_max            = 260,
                over_time           = 0,
                rank                = 5,
                lvl_req             = 67,
                lvl_max             = 74,
                lvl_outdated        = 74,
                lvl_scaling         = 0.5,
            },
            [42917] = {
                base_min            = 365,
                base_max            = 415,
                over_time           = 0,
                rank                = 6,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 80,
                lvl_scaling         = 0.8,
            },
            --cone of cold
            [120] = {
                base_min            = 98,
                base_max            = 108,
                over_time           = 0,
                over_time_tick_freq = 0,
                over_time_duration  = 0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 26,
                lvl_max             = 31,
                lvl_outdated        = 33,
                cost_base_percent   = 0.25,
                flags               = bit.bor(spell_flags.snare, spell_flags.aoe, spell_flags.cd),
                school              = magic_school.frost,
                coef                = 0.214,
                over_time_coef      = 0.0,
                lvl_scaling         = 0.8,
            },
            [8492] = {
                base_min            = 146,
                base_max            = 160,
                over_time           = 0,
                rank                = 2,
                lvl_req             = 34,
                lvl_max             = 39,
                lvl_outdated        = 41,
                lvl_scaling         = 1,
            },
            [10159] = {
                base_min            = 203,
                base_max            = 223,
                over_time           = 0,
                rank                = 3,
                lvl_req             = 42,
                lvl_max             = 47,
                lvl_outdated        = 49,
                lvl_scaling         = 1.2,
            },
            [10160] = {
                base_min            = 264,
                base_max            = 290,
                over_time           = 0,
                rank                = 4,
                lvl_req             = 50,
                lvl_max             = 55,
                lvl_outdated        = 57,
                lvl_scaling         = 1.3,
            },
            [10161] = {
                base_min            = 335,
                base_max            = 365,
                over_time           = 0,
                rank                = 5,
                lvl_req             = 58,
                lvl_max             = 63,
                lvl_outdated        = 64,
                lvl_scaling         = 1.5,
            },
            [27087] = {
                base_min            = 410,
                base_max            = 448,
                over_time           = 0,
                rank                = 6,
                lvl_req             = 65,
                lvl_max             = 70,
                lvl_outdated        = 71,
                lvl_scaling         = 1.7,
            },
            [42930] = {
                base_min            = 559,
                base_max            = 611,
                over_time           = 0,
                rank                = 7,
                lvl_req             = 72,
                lvl_max             = 76,
                lvl_outdated        = 78,
                lvl_scaling         = 2.3,
            },
            [42931] = {
                base_min            = 707,
                base_max            = 773,
                over_time           = 0,
                rank                = 8,
                lvl_req             = 79,
                lvl_max             = 83,
                lvl_outdated        = 80,
                lvl_scaling         = 2.9,
            },
            -- blizzard
            [10] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 36,
                over_time_tick_freq = 1,
                over_time_duration  = 8,
                cast_time           = 8.0,
                rank                = 1,
                lvl_req             = 20,
                lvl_max             = 25,
                lvl_outdated        = 27,
                cost_base_percent   = 0.74,
                flags               = bit.bor(spell_flags.aoe, spell_flags.over_time_crit, spell_flags.over_time_lvl_scaling),
                school              = magic_school.frost,
                coef                = 0.0,
                over_time_coef      = 0.1429,
				lvl_scaling			= 0.1,
            },
            [6141] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 63,
                rank                = 2,
                lvl_req             = 28,
                lvl_max             = 33,
                lvl_outdated        = 35,
				lvl_scaling			= 0.2,
            },
            [8427] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 92,
                rank                = 3,
                lvl_req             = 36,
                lvl_max             = 41,
                lvl_outdated        = 43,
				lvl_scaling			= 0.2,
            },
            [10185] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 128,
                rank                = 4,
                lvl_req             = 44,
                lvl_max             = 49,
                lvl_outdated        = 51,
				lvl_scaling			= 0.3,
            },
            [10186] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 166,
                rank                = 5,
                lvl_req             = 52,
                lvl_max             = 57,
                lvl_outdated        = 59,
				lvl_scaling			= 0.3,
            },
            [10187] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 212,
                rank                = 6,
                lvl_req             = 60,
                lvl_max             = 65,
                lvl_outdated        = 67,
				lvl_scaling			= 0.4,
            },
            [27085] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 273,
                rank                = 7,
                lvl_req             = 68,
                lvl_max             = 73,
                lvl_outdated        = 73,
				lvl_scaling			= 0.4,
            },
            [42939] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 350,
                rank                = 8,
                lvl_req             = 74,
                lvl_max             = 78,
                lvl_outdated        = 79,
				lvl_scaling			= 0.5,
            },
            [42940] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 426,
                rank                = 9,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 0.6,
            },
            -- fireball
            [133] = {
                base_min            = 14,
                base_max            = 22,
                over_time           = 1,
                over_time_tick_freq = 2,
                over_time_duration  = 8,
                cast_time           = 3.5,
                rank                = 1,
                lvl_req             = 1,
                lvl_max             = 5,
                lvl_outdated        = 5,
                cost_base_percent   = 0.19,
                flags               = 0,
                school              = magic_school.fire,
                coef                = 1.0,
                over_time_coef      = 0.0,
				lvl_scaling			= 0.6,
            },
            [143] = {
                base_min            = 31,
                base_max            = 45,
                over_time           = 1,
                rank                = 2,
                lvl_req             = 6,
                lvl_max             = 10,
                lvl_outdated        = 11,
				lvl_scaling			= 0.8,
            },
            [145] = {
                base_min            = 53,
                base_max            = 73,
                over_time           = 2,
                rank                = 3,
                lvl_req             = 12,
                lvl_max             = 16,
                lvl_outdated        = 17,
				lvl_scaling			= 1,
            },
            [3140] = {
                base_min            = 84,
                base_max            = 116,
                over_time           = 3,
                rank                = 4,
                lvl_req             = 18,
                lvl_max             = 22,
                lvl_outdated        = 23,
				lvl_scaling			= 1.3,
            },
            [8400] = {
                base_min            = 139,
                base_max            = 187,
                over_time           = 5,
                rank                = 5,
                lvl_req             = 24,
                lvl_max             = 28,
                lvl_outdated        = 29,
				lvl_scaling			= 1.8,
            },
            [8401] = {
                base_min            = 199,
                base_max            = 265,
                over_time           = 7,
                rank                = 6,
                lvl_req             = 30,
                lvl_max             = 34,
                lvl_outdated        = 35,
				lvl_scaling			= 2.1,
            },
            [8402] = {
                base_min            = 255,
                base_max            = 335,
                over_time           = 8,
                rank                = 7,
                lvl_req             = 36,
                lvl_max             = 40,
                lvl_outdated        = 41,
				lvl_scaling			= 2.4,
            },
            [10148] = {
                base_min            = 318,
                base_max            = 414,
                over_time           = 10,
                rank                = 8,
                lvl_req             = 42,
                lvl_max             = 46,
                lvl_outdated        = 47,
				lvl_scaling			= 2.7,
            },
            [10149] = {
                base_min            = 392,
                base_max            = 506,
                over_time           = 13,
                rank                = 9,
                lvl_req             = 48,
                lvl_max             = 52,
                lvl_outdated        = 53,
				lvl_scaling			= 3,
            },
            [10150] = {
                base_min            = 475,
                base_max            = 609,
                over_time           = 15,
                rank                = 10,
                lvl_req             = 54,
                lvl_max             = 58,
                lvl_outdated        = 59,
				lvl_scaling			= 3.4,
            },
            [10151] = {
                base_min            = 561,
                base_max            = 715,
                over_time           = 18,
                rank                = 11,
                lvl_req             = 60,
                lvl_max             = 64,
                lvl_outdated        = 60,
				lvl_scaling			= 3.7,
            },
            [25306] = {
                base_min            = 596,
                base_max            = 760,
                over_time           = 19,
                rank                = 12,
                lvl_req             = 60,
                lvl_max             = 64,
                lvl_outdated        = 65,
				lvl_scaling			= 3.8,
            },
            [27070] = {
                base_min            = 633,
                base_max            = 805,
                over_time           = 21,
                rank                = 13,
                lvl_req             = 66,
                lvl_max             = 70,
                lvl_outdated        = 69,
				lvl_scaling			= 4,
            },
            [38692] = {
                base_min            = 717,
                base_max            = 913,
                over_time           = 23,
                rank                = 14,
                lvl_req             = 70,
                lvl_max             = 74,
                lvl_outdated        = 73,
				lvl_scaling			= 4.2,
            },
            [42832] = {
                base_min            = 783,
                base_max            = 997,
                over_time           = 25,
                rank                = 15,
                lvl_req             = 74,
                lvl_max             = 78,
                lvl_outdated        = 77,
				lvl_scaling			= 4.6,
            },
            [42833] = {
                base_min            = 888,
                base_max            = 1132,
                over_time           = 29,
                rank                = 16,
                lvl_req             = 78,
                lvl_max             = 82,
                lvl_outdated        = 80,
				lvl_scaling			= 5.2,
            },
            -- fire blast
            [2136] = {
                base_min            = 24,
                base_max            = 32,
                over_time           = 0,
                over_time_tick_freq = 0,
                over_time_duration  = 0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 6,
                lvl_max             = 11,
                lvl_outdated        = 13,
                cost_base_percent   = 0.21,
                flags               = spell_flags.cd,
                school              = magic_school.fire,
                coef                = 0.4286,
                over_time_coef      = 0.0,
				lvl_scaling			= 0.6,
            },
            [2137] = {
                base_min            = 57,
                base_max            = 71,
                over_time           = 0,
                rank                = 2,
                lvl_req             = 14,
                lvl_max             = 19,
                lvl_outdated        = 21,
				lvl_scaling			= 1,
            },
            [2138] = {
                base_min            = 103,
                base_max            = 127,
                over_time           = 0,
                rank                = 3,
                lvl_req             = 22,
                lvl_max             = 27,
                lvl_outdated        = 29,
				lvl_scaling			= 1.4,
            },
            [8412] = {
                base_min            = 168,
                base_max            = 202,
                over_time           = 0,
                rank                = 4,
                lvl_req             = 30,
                lvl_max             = 35,
                lvl_outdated        = 37,
				lvl_scaling			= 1.8,
            },
            [8413] = {
                base_min            = 242,
                base_max            = 290,
                over_time           = 0,
                rank                = 5,
                lvl_req             = 38,
                lvl_max             = 43,
                lvl_outdated        = 45,
				lvl_scaling			= 2.2,
            },
            [10197] = {
                base_min            = 332,
                base_max            = 394,
                over_time           = 0,
                rank                = 6,
                lvl_req             = 46,
                lvl_max             = 51,
                lvl_outdated        = 53,
				lvl_scaling			= 2.6,
            },
            [10199] = {
                base_min            = 431,
                base_max            = 509,
                over_time           = 0,
                rank                = 7,
                lvl_req             = 54,
                lvl_max             = 59,
                lvl_outdated        = 60,
				lvl_scaling			= 3,
            },
            [27078] = {
                base_min            = 539,
                base_max            = 637,
                over_time           = 0,
                rank                = 8,
                lvl_req             = 61,
                lvl_max             = 66,
                lvl_outdated        = 69,
				lvl_scaling			= 3.3,
            },
            [27079] = {
                base_min            = 664,
                base_max            = 786,
                over_time           = 0,
                rank                = 9,
                lvl_req             = 70,
                lvl_max             = 75,
                lvl_outdated        = 73,
				lvl_scaling			= 3.7,
            },
            [42872] = {
                base_min            = 760,
                base_max            = 900,
                over_time           = 0,
                rank                = 10,
                lvl_req             = 74,
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 4,
            },
            [42873] = {
                base_min            = 925,
                base_max            = 1095,
                over_time           = 0,
                rank                = 11,
                lvl_req             = 80,
                lvl_max             = 85,
                lvl_outdated        = 80,
				lvl_scaling			= 4.9,
            },
            -- scorch
            [2948] = {
                base_min            = 53,
                base_max            = 65,
                over_time           = 0,
                over_time_tick_freq = 0,
                over_time_duration  = 0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 22, 
                lvl_max             = 26,
                lvl_outdated        = 27,
                cost_base_percent   = 0.08,
                flags               = 0,
                school              = magic_school.fire,
                coef                = 0.4286,
                over_time_coef      = 0.0,
				lvl_scaling			= 0.9,
            },
            [8444] = {
                base_min            = 77,
                base_max            = 93,
                over_time           = 0,
                rank                = 2,
                lvl_req             = 28,
                lvl_max             = 32,
                lvl_outdated        = 33,
				lvl_scaling			= 1.1,
            },
            [8445] = {
                base_min            = 100,
                base_max            = 120,
                over_time           = 0,
                rank                = 3,
                lvl_req             = 34,
                lvl_max             = 38,
                lvl_outdated        = 39,
				lvl_scaling			= 1.3,
            },
            [8446] = {
                base_min            = 133,
                base_max            = 159,
                over_time           = 0,
                rank                = 4,
                lvl_req             = 40,
                lvl_max             = 44,
                lvl_outdated        = 45,
				lvl_scaling			= 1.5,
            },
            [10205] = {
                base_min            = 162,
                base_max            = 192,
                over_time           = 0,
                rank                = 5,
                lvl_req             = 46,
                lvl_max             = 50,
                lvl_outdated        = 51,
				lvl_scaling			= 1.7,
            },
            [10206] = {
                base_min            = 200,
                base_max            = 239,
                over_time           = 0,
                rank                = 6,
                lvl_req             = 52,
                lvl_max             = 56,
                lvl_outdated        = 57,
				lvl_scaling			= 1.9,
            },
            [10207] = {
                base_min            = 233,
                base_max            = 275,
                over_time           = 0,
                rank                = 7,
                lvl_req             = 58,
                lvl_max             = 62,
                lvl_outdated        = 64,
				lvl_scaling			= 2.1,
            },
            [27073] = {
                base_min            = 269,
                base_max            = 317,
                over_time           = 0,
                rank                = 8,
                lvl_req             = 65,
                lvl_max             = 69,
                lvl_outdated        = 69,
				lvl_scaling			= 2.3,
            },
            [27074] = {
                base_min            = 305,
                base_max            = 361,
                over_time           = 0,
                rank                = 9,
                lvl_req             = 70,
                lvl_max             = 74,
                lvl_outdated        = 72,
				lvl_scaling			= 2.5,
            },
            [42858] = {
                base_min            = 321,
                base_max            = 379,
                over_time           = 0,
                rank                = 10,
                lvl_req             = 73,
                lvl_max             = 77,
                lvl_outdated        = 77,
				lvl_scaling			= 2.6,
            },
            [42859] = {
                base_min            = 376,
                base_max            = 444,
                over_time           = 0,
                rank                = 11,
                lvl_req             = 78,
                lvl_max             = 82,
                lvl_outdated        = 80,
				lvl_scaling			= 3.1,
            },
            -- pyroblast
            [11366] = {
                base_min            = 141,
                base_max            = 187,
                over_time           = 14,
                over_time_tick_freq = 3,
                over_time_duration  = 12,
                cast_time           = 5.0,
                rank                = 1,
                lvl_req             = 20,
                lvl_max             = 24,
                lvl_outdated        = 23,
                cost_base_percent   = 0.22,
                flags               = 0,
                school              = magic_school.fire,
                coef                = 1.15,
                over_time_coef      = 0.05,
				lvl_scaling			= 1.9,
            },
            [12505] = {
                base_min            = 180,
                base_max            = 236,
                over_time           = 18,
                rank                = 2,
                lvl_req             = 24,
                lvl_max             = 30,
                lvl_outdated        = 29,
				lvl_scaling			= 2.2,
            },
            [12522] = {
                base_min            = 255,
                base_max            = 327,
                over_time           = 24,
                rank                = 3,
                lvl_req             = 30,
                lvl_max             = 36,
                lvl_outdated        = 35,
				lvl_scaling			= 2.6,
            },
            [12523] = {
                base_min            = 329,
                base_max            = 419,
                over_time           = 31,
                rank                = 4,
                lvl_req             = 36,
                lvl_max             = 42,
                lvl_outdated        = 41,
				lvl_scaling			= 3,
            },
            [12524] = {
                base_min            = 407,
                base_max            = 515,
                over_time           = 39,
                rank                = 5,
                lvl_req             = 42,
                lvl_max             = 48,
                lvl_outdated        = 47,
				lvl_scaling			= 3.4,
            },
            [12525] = {
                base_min            = 503,
                base_max            = 631,
                over_time           = 47,
                rank                = 6,
                lvl_req             = 48,
                lvl_max             = 54,
                lvl_outdated        = 53,
				lvl_scaling			= 3.8,
            },
            [12526] = {
                base_min            = 600,
                base_max            = 750,
                over_time           = 57,
                rank                = 7,
                lvl_req             = 54,
                lvl_max             = 60,
                lvl_outdated        = 59,
				lvl_scaling			= 4.2,
            },
            [18809] = {
                base_min            = 708,
                base_max            = 898,
                over_time           = 67,
                rank                = 8,
                lvl_req             = 60,
                lvl_max             = 66,
                lvl_outdated        = 65,
				lvl_scaling			= 4.6,
            },
            [27132] = {
                base_min            = 846,
                base_max            = 1074,
                over_time           = 78,
                rank                = 9,
                lvl_req             = 66,
                lvl_max             = 72,
                lvl_outdated        = 69,
				lvl_scaling			= 5,
            },
            [33938] = {
                base_min            = 939,
                base_max            = 1191,
                over_time           = 89,
                rank                = 10,
                lvl_req             = 70,
                lvl_max             = 76,
                lvl_outdated        = 72,
				lvl_scaling			= 5.4,
            },
            [42890] = {
                base_min            = 1014,
                base_max            = 1286,
                over_time           = 96,
                rank                = 11,
                lvl_req             = 73,
                lvl_max             = 77,
                lvl_outdated        = 76,
				lvl_scaling			= 5.8,
            },
            [42891] = {
                base_min            = 1190,
                base_max            = 1510,
                over_time           = 113,
                rank                = 12,
                lvl_req             = 77,
                lvl_max             = 81,
                lvl_outdated        = 80,
				lvl_scaling			= 6.8,
            },
            -- blast wave
            [11113] = {
                base_min            = 154,
                base_max            = 186,
                over_time           = 0,
                over_time_tick_freq = 0,
                over_time_duration  = 0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 30, 
                lvl_max             = 36,
                lvl_outdated        = 35,
                cost_base_percent   = 0.07,
                flags               = bit.bor(spell_flags.snare, spell_flags.aoe, spell_flags.cd),
                school              = magic_school.fire,
                coef                = 0.1936,
                over_time_coef      = 0.0,
				lvl_scaling			= 1,
            },
            [13018] = {
                base_min            = 201,
                base_max            = 241,
                over_time           = 0,
                rank                = 2,
                lvl_req             = 36,
                lvl_max             = 42,
                lvl_outdated        = 43,
				lvl_scaling			= 1.2,
            },
            [13019] = {
                base_min            = 277,
                base_max            = 329,
                over_time           = 0,
                rank                = 3,
                lvl_req             = 44,
                lvl_max             = 50,
                lvl_outdated        = 51,
				lvl_scaling			= 1.4,
            },
            [13020] = {
                base_min            = 365,
                base_max            = 433,
                over_time           = 0,
                rank                = 4,
                lvl_req             = 52,
                lvl_max             = 58,
                lvl_outdated        = 59,
				lvl_scaling			= 1.6,
            },
            [13021] = {
                base_min            = 462,
                base_max            = 544,
                over_time           = 0,
                rank                = 5,
                lvl_req             = 60,
                lvl_max             = 66,
                lvl_outdated        = 64,
				lvl_scaling			= 1.9,
            },
            [27133] = {
                base_min            = 533,
                base_max            = 627,
                over_time           = 0,
                rank                = 6,
                lvl_req             = 65,
                lvl_max             = 71,
                lvl_outdated        = 69,
				lvl_scaling			= 2.1,
            },
            [33933] = {
                base_min            = 616,
                base_max            = 724,
                over_time           = 0,
                rank                = 7,
                lvl_req             = 70,
                lvl_max             = 76,
                lvl_outdated        = 74,
				lvl_scaling			= 2.3,
            },
            [42944] = {
                base_min            = 882,
                base_max            = 1038,
                over_time           = 0,
                rank                = 8,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 3.3,
            },
            [42945] = {
                base_min            = 1047,
                base_max            = 1233,
                over_time           = 0,
                rank                = 9,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 3.9,
            },
            -- flamestrike
            [2120] = {
                base_min            = 52,
                base_max            = 68,
                over_time           = 12,
                over_time_tick_freq = 2,
                over_time_duration  = 8,
                cast_time           = 2.0,
                rank                = 1,
                lvl_req             = 16, 
                lvl_max             = 21,
                lvl_outdated        = 23, 
                cost_base_percent   = 0.35,
                flags               = spell_flags.aoe,
                school              = magic_school.fire,
                coef                = 0.243,
                over_time_coef      = 0.122,
				lvl_scaling			= 0.6,
            },
            [2121] = {
                base_min            = 96,
                base_max            = 122,
                over_time           = 22,
                rank                = 2,
                lvl_req             = 24, 
                lvl_max             = 29,
                lvl_outdated        = 31, 
				lvl_scaling			= 0.8,
            },
            [8422] = {
                base_min            = 154,
                base_max            = 192,
                over_time           = 35,
                rank                = 3,
                lvl_req             = 32, 
                lvl_max             = 37,
                lvl_outdated        = 39, 
				lvl_scaling			= 1,
            },
            [8423] = {
                base_min            = 220,
                base_max            = 272,
                over_time           = 49,
                rank                = 4,
                lvl_req             = 40, 
                lvl_max             = 45,
                lvl_outdated        = 47, 
				lvl_scaling			= 1.3,
            },
            [10215] = {
                base_min            = 291,
                base_max            = 359,
                over_time           = 66,
                rank                = 5,
                lvl_req             = 48, 
                lvl_max             = 53,
                lvl_outdated        = 55, 
				lvl_scaling			= 1.5,
            },
            [10216] = {
                base_min            = 375,
                base_max            = 459,
                over_time           = 85,
                rank                = 6,
                lvl_req             = 56, 
                lvl_max             = 61,
                lvl_outdated        = 63, 
				lvl_scaling			= 1.7,
            },
            [27086] = {
                base_min            = 471,
                base_max            = 575,
                over_time           = 106,
                rank                = 7,
                lvl_req             = 64, 
                lvl_max             = 69,
                lvl_outdated        = 71, 
				lvl_scaling			= 1.9,
            },
            [42925] = {
                base_min            = 688,
                base_max            = 842,
                over_time           = 155,
                rank                = 8,
                lvl_req             = 72, 
                lvl_max             = 76,
                lvl_outdated        = 78, 
				lvl_scaling			= 2.8,
            },
            [42926] = {
                base_min            = 873,
                base_max            = 1067,
                over_time           = 195,
                rank                = 9,
                lvl_req             = 79, 
                lvl_max             = 83,
                lvl_outdated        = 80, 
				lvl_scaling			= 3.5,
            },
            -- arcane missiles
            [5143] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 24,
                over_time_tick_freq = 1,
                over_time_duration  = 5.0,
                cast_time           = 5.0,
                rank                = 1,
                lvl_req             = 8, 
                lvl_max             = 12,
                lvl_outdated        = 15, 
                cost_base_percent   = 0.31,
                flags               = bit.bor(spell_flags.over_time_crit, spell_flags.over_time_lvl_scaling),
                school              = magic_school.arcane,
                coef                = 0.0,
                over_time_coef      = 0.286,
				lvl_scaling			= 0.3,
            },
            [5144] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 36,
                rank                = 2,
                lvl_req             = 16, 
                lvl_max             = 20,
                lvl_outdated        = 23, 
				lvl_scaling			= 0.4,
            },
            [5145] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 56,
                rank                = 3,
                lvl_req             = 24, 
                lvl_max             = 28,
                lvl_outdated        = 31, 
				lvl_scaling			= 0.5,
            },
            [8416] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 83,
                rank                = 4,
                lvl_req             = 32, 
                lvl_max             = 36,
                lvl_outdated        = 39, 
				lvl_scaling			= 0.6,
            },
            [8417] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 115,
                rank                = 5,
                lvl_req             = 40, 
                lvl_max             = 44,
                lvl_outdated        = 47, 
				lvl_scaling			= 0.7,
            },
            [10211] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 151,
                rank                = 6,
                lvl_req             = 48, 
                lvl_max             = 52,
                lvl_outdated        = 55, 
				lvl_scaling			= 0.8,
            },
            [10212] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 192,
                rank                = 7,
                lvl_req             = 56, 
                lvl_max             = 60,
                lvl_outdated        = 59, 
				lvl_scaling			= 0.9,
            },
            [25345] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 230,
                rank                = 8,
                lvl_req             = 60, 
                lvl_max             = 64,
                lvl_outdated        = 62,
				lvl_scaling			= 1,
            },
            [27075] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 240,
                rank                = 9,
                lvl_req             = 63, 
                lvl_max             = 67,
                lvl_outdated        = 68, 
				lvl_scaling			= 1.1,
            },
            [38699] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 260,
                rank                = 10,
                lvl_req             = 69, 
                lvl_max             = 73,
                lvl_outdated        = 69, 
				lvl_scaling			= 1.2,
            },
            [38704] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 280,
                rank                = 11,
                lvl_req             = 70, 
                lvl_max             = 74,
                lvl_outdated        = 74, 
				lvl_scaling			= 1.3,
            },
            [42843] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 320,
                rank                = 12,
                lvl_req             = 75, 
                lvl_max             = 79,
                lvl_outdated        = 79, 
				lvl_scaling			= 1.5,
            },
            [42846] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 360,
                rank                = 13,
                lvl_req             = 80, 
                lvl_max             = 83,
                lvl_outdated        = 80, 
				lvl_scaling			= 1.7,
            },
            -- arcane explosion
            [1449] = {
                base_min            = 32,
                base_max            = 36,
                over_time           = 0,
                over_time_tick_freq = 0,
                over_time_duration  = 0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 14, 
                lvl_max             = 19,
                lvl_outdated        = 21,
                cost_base_percent   = 0.22,
                flags               = spell_flags.aoe,
                school              = magic_school.arcane,
                coef                = 0.2143,
                over_time_coef      = 0.0,
				lvl_scaling			= 0.4,
            },
            [8437] = {
                base_min            = 57,
                base_max            = 63,
                over_time           = 0,
                rank                = 2,
                lvl_req             = 22,
                lvl_max             = 27,
                lvl_outdated        = 29,
				lvl_scaling			= 0.6,
            },
            [8438] = {
                base_min            = 97,
                base_max            = 105,
                over_time           = 0,
                rank                = 3,
                lvl_req             = 30,
                lvl_max             = 35,
                lvl_outdated        = 37,
				lvl_scaling			= 0.9,
            },
            [8439] = {
                base_min            = 139,
                base_max            = 151,
                over_time           = 0,
                rank                = 4,
                lvl_req             = 38,
                lvl_max             = 43,
                lvl_outdated        = 45,
				lvl_scaling			= 0.9,
            },
            [10201] = {
                base_min            = 186,
                base_max            = 202,
                over_time           = 0,
                rank                = 5,
                lvl_req             = 46,
                lvl_max             = 51,
                lvl_outdated        = 53,
				lvl_scaling			= 1.1,
            },
            [10202] = {
                base_min            = 243,
                base_max            = 263,
                over_time           = 0,
                rank                = 6,
                lvl_req             = 54,
                lvl_max             = 59,
                lvl_outdated        = 61,
				lvl_scaling			= 1.3,
            },
            [27080] = {
                base_min            = 306,
                base_max            = 330,
                over_time           = 0,
                rank                = 7,
                lvl_req             = 62,
                lvl_max             = 67,
                lvl_outdated        = 69,
				lvl_scaling			= 1.5,
            },
            [27082] = {
                base_min            = 377,
                base_max            = 407,
                over_time           = 0,
                rank                = 8,
                lvl_req             = 70,
                lvl_max             = 75,
                lvl_outdated        = 75,
				lvl_scaling			= 1.6,
            },
            [42920] = {
                base_min            = 481,
                base_max            = 519,
                over_time           = 0,
                rank                = 9,
                lvl_req             = 76,
                lvl_max             = 80,
                lvl_outdated        = 79,
				lvl_scaling			= 2,
            },
            [42921] = {
                base_min            = 538,
                base_max            = 582,
                over_time           = 0,
                rank                = 10,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 2.3,
            },
            -- arcane blast
            [30451] = {
                base_min            = 842,
                base_max            = 978,
                over_time           = 0,
                over_time_tick_freq = 0,
                over_time_duration  = 0,
                cast_time           = 2.5,
                rank                = 1,
                lvl_req             = 64, 
                lvl_max             = 68,
                lvl_outdated        = 70,
                cost_base_percent   = 0.07,
                flags               = 0,
                school              = magic_school.arcane,
                coef                = 0.7143,
                over_time_coef      = 0.0,
				lvl_scaling			= 5,
            },
            [42894] = {
                base_min            = 897,
                base_max            = 1041,
                over_time           = 0,
                rank                = 2,
                lvl_req             = 71, 
                lvl_max             = 75,
                lvl_outdated        = 75,
				lvl_scaling			= 5.3,
            },
            [42896] = {
                base_min            = 1047,
                base_max            = 1215,
                over_time           = 0,
                rank                = 3,
                lvl_req             = 76, 
                lvl_max             = 80,
                lvl_outdated        = 79,
				lvl_scaling			= 6.2,
            },
            [42897] = {
                base_min            = 1185,
                base_max            = 1377,
                over_time           = 0,
                rank                = 4,
                lvl_req             = 80, 
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 7,
            },
            -- mana shield
            [1463] = {
                base_min            = 120,
                base_max            = 120,
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 20,
                lvl_max             = 27,
                lvl_outdated        = 27,
                cost_base_percent   = 0.07,
                flags               = spell_flags.absorb,
                school              = magic_school.arcane,
                coef                = 0.8053,
                over_time_coef      = 0.0,
				lvl_scaling			= 0.0,
            },
            [8494] = {
                base_min            = 210,
                base_max            = 210,
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 28,
                lvl_max             = 35,
                lvl_outdated        = 35,
				lvl_scaling			= 0.0,
            },
            [8495] = {
                base_min            = 300,
                base_max            = 300,
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 36,
                lvl_max             = 43,
                lvl_outdated        = 43,
				lvl_scaling			= 0.0,
            },
            [10191] = {
                base_min            = 390,
                base_max            = 390,
                over_time           = 0.0,
                rank                = 4,
                lvl_req             = 44,
                lvl_max             = 51,
                lvl_outdated        = 51,
				lvl_scaling			= 0.0,
            },
            [10192] = {
                base_min            = 480,
                base_max            = 480,
                over_time           = 0.0,
                rank                = 5,
                lvl_req             = 52,
                lvl_max             = 59,
                lvl_outdated        = 59,
				lvl_scaling			= 0.0,
            },
            [10193] = {
                base_min            = 570,
                base_max            = 570,
                over_time           = 0.0,
                rank                = 6,
                lvl_req             = 60,
                lvl_max             = 67,
                lvl_outdated        = 67,
				lvl_scaling			= 0.0,
            },
            [27131] = {
                base_min            = 715,
                base_max            = 715,
                over_time           = 0.0,
                rank                = 7,
                lvl_req             = 68,
                lvl_max             = 75,
                lvl_outdated        = 72,
				lvl_scaling			= 0.0,
            },
            [43019] = {
                base_min            = 1080,
                base_max            = 1080,
                over_time           = 0.0,
                rank                = 8,
                lvl_req             = 73,
                lvl_max             = 77,
                lvl_outdated        = 78,
				lvl_scaling			= 0.0,
            },
            [43020] = {
                base_min            = 1330,
                base_max            = 1330,
                over_time           = 0.0,
                rank                = 9,
                lvl_req             = 79,
                lvl_max             = 83,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            -- arcane barrage
            [44425] = {
                base_min            = 386,
                base_max            = 470,
                over_time           = 0,
                over_time_tick_freq = 0,
                over_time_duration  = 0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 60, 
                lvl_max             = 66,
                lvl_outdated        = 69,
                cost_base_percent   = 0.18,
                flags               = bit.bor(spell_flags.aoe, spell_flags.cd),
                school              = magic_school.arcane,
                coef                = 0.7143,
                over_time_coef      = 0.0,
				lvl_scaling			= 2.5,
            },
            [44780] = {
                base_min            = 709,
                base_max            = 865,
                over_time           = 0,
                rank                = 2,
                lvl_req             = 70, 
                lvl_max             = 76,
                lvl_outdated        = 79, 
				lvl_scaling			= 15.5,
            },
            [44781] = {
                base_min            = 936,
                base_max            = 1144,
                over_time           = 0,
                rank                = 3,
                lvl_req             = 80, 
                lvl_max             = 86,
                lvl_outdated        = 80, 
				lvl_scaling			= 6.1,
            },
            -- dragons breath
            [31661] = {
                base_min            = 370,
                base_max            = 430,
                over_time           = 0,
                over_time_tick_freq = 0,
                over_time_duration  = 0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 50, 
                lvl_max             = 58,
                lvl_outdated        = 55,
                cost_base_percent   = 0.07,
                flags               = bit.bor(spell_flags.snare, spell_flags.aoe, spell_flags.cd),
                school              = magic_school.fire,
                coef                = 0.1936,
                over_time_coef      = 0.0,
				lvl_scaling			= 1.5,
            },
            [33041] = {
                base_min            = 454,
                base_max            = 526,
                over_time           = 0,
                rank                = 2,
                lvl_req             = 56, 
                lvl_max             = 62,
                lvl_outdated        = 63,
				lvl_scaling			= 1.6,
            },
            [33042] = {
                base_min            = 574,
                base_max            = 666,
                over_time           = 0,
                rank                = 3,
                lvl_req             = 64, 
                lvl_max             = 70,
                lvl_outdated        = 69,
				lvl_scaling			= 1.8,
            },
            [33043] = {
                base_min            = 680,
                base_max            = 790,
                over_time           = 0,
                rank                = 4,
                lvl_req             = 70, 
                lvl_max             = 76,
                lvl_outdated        = 74,
				lvl_scaling			= 2,
            },
            [42949] = {
                base_min            = 935,
                base_max            = 1085,
                over_time           = 0,
                rank                = 5,
                lvl_req             = 75, 
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 2.7,
            },
            [42950] = {
                base_min            = 1101,
                base_max            = 1279,
                over_time           = 0,
                rank                = 6,
                lvl_req             = 80, 
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 3.2,
            },
            --frostfire bolt
            [44614] = {
                base_min            = 629,
                base_max            = 731,
                over_time           = 20,
                over_time_tick_freq = 3,
                over_time_duration  = 9,
                cast_time           = 3.0,
                rank                = 1,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 79,
                cost_base_percent   = 0.14,
                flags               = spell_flags.snare,
                school              = magic_school.frost,
                coef                = 0.8571,
                over_time_coef      = 0.0,
                lvl_scaling         = 3.9,
            },
            [47610] = {
                base_min            = 722,
                base_max            = 838,
                over_time           = 30,
                rank                = 2,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
                lvl_scaling         = 4.5,
            },
            --living bomb
            [44457] = {
                base_min            = 306.0,
                base_max            = 306.0, 
                over_time           = 153,
                over_time_tick_freq = 3,
                over_time_duration  = 12,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 60,
                lvl_max             = 66,
                lvl_outdated        = 69,
                cost_base_percent   = 0.22,
                flags               = 0,
                school              = magic_school.fire,
                coef                = 0.4,
                over_time_coef      = 0.2,
				lvl_scaling			= 0.0,
            },
            [55359] = {
                base_min            = 512.0,
                base_max            = 512.0, 
                over_time           = 256,
                rank                = 2,
                lvl_req             = 70,
                lvl_max             = 76,
                lvl_outdated        = 79,
				lvl_scaling			= 0.0,
            },
            [55360] = {
                base_min            = 690.0,
                base_max            = 690.0, 
                over_time           = 345,
                rank                = 3,
                lvl_req             = 80,
                lvl_max             = 86,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            -- ice barrier
            [11426] = {
                base_min            = 438,
                base_max            = 438,
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 40,
                lvl_max             = 46,
                lvl_outdated        = 45,
                cost_base_percent   = 0.21,
                flags               = spell_flags.absorb,
                school              = magic_school.frost,
                coef                = 0.8053,
                over_time_coef      = 0.0,
				lvl_scaling			= 2.8,
            },
            [13031] = {
                base_min            = 549,
                base_max            = 549,
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 46,
                lvl_max             = 52,
                lvl_outdated        = 51,
				lvl_scaling			= 3.2,
            },
            [13032] = {
                base_min            = 678,
                base_max            = 678,
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 52,
                lvl_max             = 58,
                lvl_outdated        = 57,
				lvl_scaling			= 3.6,
            },
            [13033] = {
                base_min            = 818,
                base_max            = 818,
                over_time           = 0.0,
                rank                = 4,
                lvl_req             = 58,
                lvl_max             = 64,
                lvl_outdated        = 63,
				lvl_scaling			= 4,
            },
            [27134] = {
                base_min            = 925,
                base_max            = 925,
                over_time           = 0.0,
                rank                = 5,
                lvl_req             = 64,
                lvl_max             = 70,
                lvl_outdated        = 69,
				lvl_scaling			= 4.4,
            },
            [33405] = {
                base_min            = 1075,
                base_max            = 1075,
                over_time           = 0.0,
                rank                = 6,
                lvl_req             = 70,
                lvl_max             = 76,
                lvl_outdated        = 74,
				lvl_scaling			= 4.8,
            },
            [43038] = {
                base_min            = 2800,
                base_max            = 2800,
                over_time           = 0.0,
                rank                = 7,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 15,
            },
            [43039] = {
                base_min            = 3300,
                base_max            = 3300,
                over_time           = 0.0,
                rank                = 8,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 15,
            },
            -- ice lance
            [30455] = {
                base_min            = 161,
                base_max            = 187,
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 66,
                lvl_max             = 70,
                lvl_outdated        = 71,
                cost_base_percent   = 0.06,
                flags               = 0,
                school              = magic_school.frost,
                coef                = 0.1429,
                over_time_coef      = 0.0,
                lvl_scaling         = 3.2,
            },
            [42913] = {
                base_min            = 182,
                base_max            = 210,
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 72,
                lvl_max             = 76,
                lvl_outdated        = 77,
                lvl_scaling         = 1.3,
            },
            [42914] = {
                base_min            = 221,
                base_max            = 255,
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 78,
                lvl_max             = 82,
                lvl_outdated        = 80,
                lvl_scaling         = 1.3,
            },
            -- deep freeze
            -- TODO: are there no more ranks of this??
            -- -- or 71757 id
            [44572] = {
                base_min            = 1469.0,
                base_max            = 1741.0, 
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 60,
                lvl_max             = 0,
                lvl_outdated        = 80,
                cost_base_percent   = 0.09,
                flags               = spell_flags.cd,
                school              = magic_school.frost,
                coef                = 2.143,
                over_time_coef      = 0.0,
                lvl_scaling         = 45.0,
            },
        };

    elseif class == "DRUID" then
        return {
            --  healing touch
            [5185] = {
                base_min            = 37,
                base_max            = 51,
                over_time           = 0,
                over_time_tick_freq = 0,
                over_time_duration  = 0,
                cast_time           = 3.0,
                rank                = 1,
                lvl_req             = 1,
                lvl_max             = 5,
                lvl_outdated        = 7,
                cost_base_percent   = 0.33,
                flags               = spell_flags.heal,
                school              = magic_school.nature,
                coef                = 1.6114,
                over_time_coef      = 0,
				lvl_scaling			= 0.8,
            },
            [5186] = {
                base_min            = 88,
                base_max            = 112,
                over_time           = 0,
                rank                = 2,
                lvl_req             = 8,
                lvl_max             = 13,
                lvl_outdated        = 13,
				lvl_scaling			= 1.3,
            },
            [5187] = {
                base_min            = 195,
                base_max            = 243,
                over_time           = 0,
                rank                = 3,
                lvl_req             = 14,
                lvl_max             = 19,
                lvl_outdated        = 19,
				lvl_scaling			= 1.9,
            },
            [5188] = {
                base_min            = 363,
                base_max            = 445,
                over_time           = 0,
                rank                = 4,
                lvl_req             = 20,
                lvl_max             = 25,
                lvl_outdated        = 25,
				lvl_scaling			= 2.7,
            },
            [5189] = {
                base_min            = 490,
                base_max            = 594,
                over_time           = 0,
                rank                = 5,
                lvl_req             = 26,
                lvl_max             = 31,
                lvl_outdated        = 31,
				lvl_scaling			= 3,
            },
            [6778] = {
                base_min            = 636,
                base_max            = 766,
                over_time           = 0,
                rank                = 6,
                lvl_req             = 32,
                lvl_max             = 37,
                lvl_outdated        = 37,
				lvl_scaling			= 3.4,
            },
            [8903] = {
                base_min            = 802,
                base_max            = 960,
                over_time           = 0,
                rank                = 7,
                lvl_req             = 38,
                lvl_max             = 43,
                lvl_outdated        = 43,
				lvl_scaling			= 3.9,
            },
            [9758] = {
                base_min            = 1199,
                base_max            = 1427,
                over_time           = 0,
                rank                = 8,
                lvl_req             = 44,
                lvl_max             = 49,
                lvl_outdated        = 49,
				lvl_scaling			= 4.5,
            },
            [9888] = {
                base_min            = 1299,
                base_max            = 1539,
                over_time           = 0,
                rank                = 9,
                lvl_req             = 50,
                lvl_max             = 55,
                lvl_outdated        = 55,
				lvl_scaling			= 5.1,
            },
            [9889] = {
                base_min            = 1620,
                base_max            = 1912,
                over_time           = 0,
                rank                = 10,
                lvl_req             = 56,
                lvl_max             = 61,
                lvl_outdated        = 59,
				lvl_scaling			= 5.7,
            },
            [25297] = {
                base_min            = 1944,
                base_max            = 2294,
                over_time           = 0,
                rank                = 11,
                lvl_req             = 60,
                lvl_max             = 65,
                lvl_outdated        = 61,
				lvl_scaling			= 6.2,
            },
            [26978] = {
                base_min            = 2026,
                base_max            = 2392,
                over_time           = 0,
                rank                = 12,
                lvl_req             = 62,
                lvl_max             = 67,
                lvl_outdated        = 68,
				lvl_scaling			= 6.3,
            },
            [26979] = {
                base_min            = 2321,
                base_max            = 2739,
                over_time           = 0,
                rank                = 13,
                lvl_req             = 69,
                lvl_max             = 73,
                lvl_outdated        = 73,
				lvl_scaling			= 7,
            },
            [48377] = {
                base_min            = 3223,
                base_max            = 3805,
                over_time           = 0,
                rank                = 14,
                lvl_req             = 74,
                lvl_max             = 78,
                lvl_outdated        = 78,
				lvl_scaling			= 9.8,
            },
            [48378] = {
                base_min            = 3750,
                base_max            = 4428,
                over_time           = 0,
                rank                = 15,
                lvl_req             = 79,
                lvl_max             = 83,
                lvl_outdated        = 80,
				lvl_scaling			= 11.4,
            },
            -- rejuvenation
            [774] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 8,
                over_time_tick_freq = 3,
                over_time_duration  = 15,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 4,
                lvl_max             = 9,
                lvl_outdated        = 9,
                cost_base_percent   = 0.18,
                flags               = spell_flags.heal,
                school              = magic_school.nature,
                coef                = 0.0,
                over_time_coef      = 0.376,
				lvl_scaling			= 0.0,
            },
            [1058] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 14,
                rank                = 2,
                lvl_req             = 10,
                lvl_max             = 15,
                lvl_outdated        = 15,
				lvl_scaling			= 0.0,
            },
            [1430] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 29,
                rank                = 3,
                lvl_req             = 16,
                lvl_max             = 21,
                lvl_outdated        = 21,
				lvl_scaling			= 0.0,
            },
            [2090] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 45,
                rank                = 4,
                lvl_req             = 22,
                lvl_max             = 27,
                lvl_outdated        = 27,
				lvl_scaling			= 0.0,
            },
            [2091] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 61,
                rank                = 5,
                lvl_req             = 28,
                lvl_max             = 33,
                lvl_outdated        = 33,
				lvl_scaling			= 0.0,
            },
            [3627] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 76,
                rank                = 6,
                lvl_req             = 34,
                lvl_max             = 39,
                lvl_outdated        = 39,
				lvl_scaling			= 0.0,
            },
            [8910] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 97,
                rank                = 7,
                lvl_req             = 40,
                lvl_max             = 45,
                lvl_outdated        = 45,
				lvl_scaling			= 0.0,
            },
            [9839] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 122,
                rank                = 8,
                lvl_req             = 46,
                lvl_max             = 51,
                lvl_outdated        = 51,
				lvl_scaling			= 0.0,
            },
            [9840] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 152,
                rank                = 9,
                lvl_req             = 52,
                lvl_max             = 57,
                lvl_outdated        = 57,
				lvl_scaling			= 0.0,
            },
            [9841] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 189,
                rank                = 10,
                lvl_req             = 58,
                lvl_max             = 63,
                lvl_outdated        = 59,
				lvl_scaling			= 0.0,
            },
            [25299] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 222,
                rank                = 11,
                lvl_req             = 60,
                lvl_max             = 65,
                lvl_outdated        = 62,
				lvl_scaling			= 0.0,
            },
            [26981] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 233,
                rank                = 12,
                lvl_req             = 63,
                lvl_max             = 68,
                lvl_outdated        = 68,
				lvl_scaling			= 0.0,
            },
            [26982] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 265,
                rank                = 13,
                lvl_req             = 69,
                lvl_max             = 73,
                lvl_outdated        = 74,
				lvl_scaling			= 0.0,
            },
            [48440] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 298,
                rank                = 14,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 0.0,
            },
            [48441] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 338,
                rank                = 15,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },

            -- tranquility
            [740] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 351,
                over_time_tick_freq = 2,
                over_time_duration  = 8,
                cast_time           = 8,
                rank                = 1,
                lvl_req             = 30,
                lvl_max             = 36,
                lvl_outdated        = 39,
                cost_base_percent   = 0.7,
                flags               = bit.bor(spell_flags.heal, spell_flags.aoe, spell_flags.cd),
                school              = magic_school.nature,
                coef                = 0.0,
                over_time_coef      = 0.538,
				lvl_scaling			= 2.2,
            },
            [8918] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 515,
                rank                = 2,
                lvl_req             = 40,
                lvl_max             = 46,
                lvl_outdated        = 49,
				lvl_scaling			= 2.6,
            },
            [9862] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 765,
                rank                = 3,
                lvl_req             = 50,
                lvl_max             = 56,
                lvl_outdated        = 59,
				lvl_scaling			= 3.4,
            },
            [9863] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 1097,
                rank                = 4,
                lvl_req             = 60,
                lvl_max             = 66,
                lvl_outdated        = 69,
				lvl_scaling			= 3.7,
            },
            [26983] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 1518,
                rank                = 5,
                lvl_req             = 70,
                lvl_max             = 74,
                lvl_outdated        = 74,
				lvl_scaling			= 4.5,
            },
            [48446] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 2598,
                rank                = 6,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 4.5,
            },
            [48447] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 3035,
                rank                = 7,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 4.5,
            },
            -- regrowth
            [8936] = {
                base_min            = 84,
                base_max            = 98,
                over_time           = 14,
                over_time_tick_freq = 3,
                over_time_duration  = 21,
                cast_time           = 2.0,
                rank                = 1,
                lvl_req             = 12,
                lvl_max             = 17,
                lvl_outdated        = 17,
                cost_base_percent   = 0.29,
                flags               = spell_flags.heal,
                school              = magic_school.nature,
                coef                = 0.6455,
                over_time_coef      = 0.1878,
				lvl_scaling			= 1.8,
            },
            [8938] = {
                base_min            = 164,
                base_max            = 188,
                over_time           = 25,
                rank                = 2,
                lvl_req             = 18,
                lvl_max             = 23,
                lvl_outdated        = 23,
				lvl_scaling			= 2.5,
            },
            [8939] = {
                base_min            = 240,
                base_max            = 274,
                over_time           = 37,
                rank                = 3,
                lvl_req             = 24,
                lvl_max             = 29,
                lvl_outdated        = 29,
				lvl_scaling			= 3.1,
            },
            [8940] = {
                base_min            = 318,
                base_max            = 360,
                over_time           = 49,
                rank                = 4,
                lvl_req             = 30,
                lvl_max             = 35,
                lvl_outdated        = 35,
				lvl_scaling			= 3.6,
            },
            [8941] = {
                base_min            = 405,
                base_max            = 457,
                over_time           = 61,
                rank                = 5,
                lvl_req             = 36,
                lvl_max             = 41,
                lvl_outdated        = 41,
				lvl_scaling			= 4.1,
            },
            [9750] = {
                base_min            = 511,
                base_max            = 575,
                over_time           = 78,
                rank                = 6,
                lvl_req             = 42,
                lvl_max             = 47,
                lvl_outdated        = 47,
				lvl_scaling			= 4.7,
            },
            [9856] = {
                base_min            = 646,
                base_max            = 724,
                over_time           = 98,
                rank                = 7,
                lvl_req             = 48,
                lvl_max             = 53,
                lvl_outdated        = 53,
				lvl_scaling			= 5.3,
            },
            [9857] = {
                base_min            = 809,
                base_max            = 905,
                over_time           = 123,
                rank                = 8,
                lvl_req             = 54,
                lvl_max             = 59,
                lvl_outdated        = 59,
				lvl_scaling			= 6,
            },
            [9858] = {
                base_min            = 1003,
                base_max            = 1119,
                over_time           = 152,
                rank                = 9,
                lvl_req             = 60,
                lvl_max             = 65,
                lvl_outdated        = 64,
				lvl_scaling			= 6.8,
            },
            [26980] = {
                base_min            = 1215,
                base_max            = 1355,
                over_time           = 182,
                rank                = 10,
                lvl_req             = 65,
                lvl_max             = 70,
                lvl_outdated        = 70,
				lvl_scaling			= 7.7,
            },
            [48442] = {
                base_min            = 1710,
                base_max            = 1908,
                over_time           = 256,
                rank                = 11,
                lvl_req             = 71,
                lvl_max             = 76,
                lvl_outdated        = 76,
				lvl_scaling			= 10.8,
            },
            [48443] = {
                base_min            = 2234,
                base_max            = 2494,
                over_time           = 335,
                rank                = 12,
                lvl_req             = 77,
                lvl_max             = 81,
                lvl_outdated        = 80,
				lvl_scaling			= 7.7,
            },
            -- moonfire
            [8921] = {
                base_min            = 7,
                base_max            = 9,
                over_time           = 4,
                over_time_tick_freq = 3,
                over_time_duration  = 12,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 4,
                lvl_max             = 9,
                lvl_outdated        = 9,
                cost_base_percent   = 0.21,
                flags               = 0,
                school              = magic_school.arcane,
                coef                = 0.15,
                over_time_coef      = 0.1302,
				lvl_scaling			= 0.5,
            },
            [8924] = {
                base_min            = 13,
                base_max            = 17,
                over_time           = 8,
                rank                = 2,
                lvl_req             = 10,
                lvl_max             = 15,
                lvl_outdated        = 15,
				lvl_scaling			= 0.8,
            },
            [8925] = {
                base_min            = 25,
                base_max            = 31,
                over_time           = 13,
                rank                = 3,
                lvl_req             = 16,
                lvl_max             = 21,
                lvl_outdated        = 21,
				lvl_scaling			= 1.1,
            },
            [8926] = {
                base_min            = 40,
                base_max            = 48,
                over_time           = 20,
                rank                = 4,
                lvl_req             = 22,
                lvl_max             = 27,
                lvl_outdated        = 27,
				lvl_scaling			= 1.4,
            },
            [8927] = {
                base_min            = 61,
                base_max            = 73,
                over_time           = 31,
                rank                = 5,
                lvl_req             = 28,
                lvl_max             = 33,
                lvl_outdated        = 33,
				lvl_scaling			= 1.8,
            },
            [8928] = {
                base_min            = 81,
                base_max            = 97,
                over_time           = 41,
                rank                = 6,
                lvl_req             = 34,
                lvl_max             = 39,
                lvl_outdated        = 39,
				lvl_scaling			= 2.1,
            },
            [8929] = {
                base_min            = 105,
                base_max            = 125,
                over_time           = 53,
                rank                = 7,
                lvl_req             = 40,
                lvl_max             = 45,
                lvl_outdated        = 45,
				lvl_scaling			= 2.4,
            },
            [9833] = {
                base_min            = 130,
                base_max            = 154,
                over_time           = 66,
                rank                = 8,
                lvl_req             = 46,
                lvl_max             = 51,
                lvl_outdated        = 51,
				lvl_scaling			= 2.7,
            },
            [9834] = {
                base_min            = 157,
                base_max            = 185,
                over_time           = 80,
                rank                = 9,
                lvl_req             = 52,
                lvl_max             = 57,
                lvl_outdated        = 57,
				lvl_scaling			= 3,
            },
            [9835] = {
                base_min            = 189,
                base_max            = 221,
                over_time           = 96,
                rank                = 10,
                lvl_req             = 58,
                lvl_max             = 63,
                lvl_outdated        = 62,
				lvl_scaling			= 3.3,
            },
            [26987] = {
                base_min            = 220,
                base_max            = 258,
                over_time           = 111,
                rank                = 11,
                lvl_req             = 63,
                lvl_max             = 68,
                lvl_outdated        = 69,
				lvl_scaling			= 3.6,
            },
            [26988] = {
                base_min            = 305,
                base_max            = 357,
                over_time           = 150,
                rank                = 12,
                lvl_req             = 70,
                lvl_max             = 74,
                lvl_outdated        = 74,
				lvl_scaling			= 3.9,
            },
            [48462] = {
                base_min            = 347,
                base_max            = 407,
                over_time           = 171,
                rank                = 13,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 5,
            },
            [48463] = {
                base_min            = 406,
                base_max            = 476,
                over_time           = 200,
                rank                = 14,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 6,
            },
            -- wrath
            [5176] = {
                base_min            = 17,
                base_max            = 19,
                over_time           = 0.0,
                over_time_tick_freq = 0.0,
                over_time_duration  = 0.0,
                cast_time           = 2.0,
                rank                = 1,
                lvl_req             = 1,
                lvl_max             = 5,
                lvl_outdated        = 5,
                cost_base_percent   = 0.11,
                flags               = 0,
                school              = magic_school.nature,
                coef                = 0.5714,
                over_time_coef      = 0.0,
				lvl_scaling			= 0.4,
            },
            [5177] = {
                base_min            = 32,
                base_max            = 36,
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 6,
                lvl_max             = 12,
                lvl_outdated        = 13,
				lvl_scaling			= 0.6,
            },
            [5178] = {
                base_min            = 49,
                base_max            = 57,
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 14,
                lvl_max             = 20,
                lvl_outdated        = 21,
				lvl_scaling			= 0.8,
            },
            [5179] = {
                base_min            = 68,
                base_max            = 78,
                over_time           = 0.0,
                rank                = 4,
                lvl_req             = 22,
                lvl_max             = 28,
                lvl_outdated        = 29,
				lvl_scaling			= 1,
            },
            [5180] = {
                base_min            = 130,
                base_max            = 148,
                over_time           = 0.0,
                rank                = 5,
                lvl_req             = 30,
                lvl_max             = 36,
                lvl_outdated        = 37,
				lvl_scaling			= 1.3,
            },
            [6780] = {
                base_min            = 169,
                base_max            = 191,
                over_time           = 0.0,
                rank                = 6,
                lvl_req             = 38,
                lvl_max             = 44,
                lvl_outdated        = 45,
				lvl_scaling			= 1.6,
            },
            [8905] = {
                base_min            = 215,
                base_max            = 241,
                over_time           = 0.0,
                rank                = 7,
                lvl_req             = 46,
                lvl_max             = 52,
                lvl_outdated        = 53,
				lvl_scaling			= 1.8,
            },
            [9912] = {
                base_min            = 306,
                base_max            = 344,
                over_time           = 0.0,
                rank                = 8,
                lvl_req             = 54,
                lvl_max             = 60,
                lvl_outdated        = 60,
				lvl_scaling			= 2.1,
            },
            [26984] = {
                base_min            = 397,
                base_max            = 447,
                over_time           = 0.0,
                rank                = 9,
                lvl_req             = 61,
                lvl_max             = 67,
                lvl_outdated        = 68,
				lvl_scaling			= 2.4,
            },
            [26985] = {
                base_min            = 431,
                base_max            = 485,
                over_time           = 0.0,
                rank                = 10,
                lvl_req             = 69,
                lvl_max             = 73,
                lvl_outdated        = 73,
				lvl_scaling			= 2.7,
            },
            [48459] = {
                base_min            = 504,
                base_max            = 568,
                over_time           = 0.0,
                rank                = 11,
                lvl_req             = 74,
                lvl_max             = 78,
                lvl_outdated        = 78,
				lvl_scaling			= 3,
            },
            [48461] = {
                base_min            = 553,
                base_max            = 623,
                over_time           = 0.0,
                rank                = 12,
                lvl_req             = 79,
                lvl_max             = 83,
                lvl_outdated        = 80,
				lvl_scaling			= 4,
            },
            -- starfire
            [2912] = {
                base_min            = 121,
                base_max            = 149,
                over_time           = 0.0,
                over_time_tick_freq = 0.0,
                over_time_duration  = 0.0,
                cast_time           = 3.5,
                rank                = 1,
                lvl_req             = 20,
                lvl_max             = 25,
                lvl_outdated        = 25,
                cost_base_percent   = 0.16,
                flags               = 0,
                school              = magic_school.arcane,
                coef                = 1.0,
                over_time_coef      = 0.0,
				lvl_scaling			= 1.2,
            },
            [8949] = {
                base_min            = 189,
                base_max            = 231,
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 26,
                lvl_max             = 32,
                lvl_outdated        = 33,
				lvl_scaling			= 1.6,
            },
            [8950] = {
                base_min            = 272,
                base_max            = 328,
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 34,
                lvl_max             = 40,
                lvl_outdated        = 41,
				lvl_scaling			= 1.9,
            },
            [8951] = {
                base_min            = 370,
                base_max            = 442,
                over_time           = 0.0,
                rank                = 4,
                lvl_req             = 42,
                lvl_max             = 48,
                lvl_outdated        = 49,
				lvl_scaling			= 2.3,
            },
            [9875] = {
                base_min            = 485,
                base_max            = 575,
                over_time           = 0.0,
                rank                = 5,
                lvl_req             = 50,
                lvl_max             = 56,
                lvl_outdated        = 57,
				lvl_scaling			= 2.7,
            },
            [9876] = {
                base_min            = 615,
                base_max            = 725,
                over_time           = 0.0,
                rank                = 6,
                lvl_req             = 58,
                lvl_max             = 64,
                lvl_outdated        = 59,
				lvl_scaling			= 3,
            },
            [25298] = {
                base_min            = 693,
                base_max            = 817,
                over_time           = 0.0,
                rank                = 7,
                lvl_req             = 60,
                lvl_max             = 66,
                lvl_outdated        = 66,
				lvl_scaling			= 3.1,
            },
            [26986] = {
                base_min            = 818,
                base_max            = 964,
                over_time           = 0.0,
                rank                = 8,
                lvl_req             = 67,
                lvl_max             = 71,
                lvl_outdated        = 71,
				lvl_scaling			= 3.4,
            },
            [48464] = {
                base_min            = 854,
                base_max            = 1006,
                over_time           = 0.0,
                rank                = 9,
                lvl_req             = 72,
                lvl_max             = 76,
                lvl_outdated        = 77,
				lvl_scaling			= 4,
            },
            [48465] = {
                base_min            = 1028,
                base_max            = 1212,
                over_time           = 0.0,
                rank                = 10,
                lvl_req             = 78,
                lvl_max             = 82,
                lvl_outdated        = 80,
				lvl_scaling			= 5,
            },
            -- insect swarm
            [5570] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 24,
                over_time_tick_freq = 2.0,
                over_time_duration  = 12.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 20,
                lvl_max             = 29,
                lvl_outdated        = 29,
                cost_base_percent   = 0.08,
                flags               = spell_flags.snare,
                school              = magic_school.nature,
                coef                = 0.0,
                over_time_coef      = 0.2,
				lvl_scaling			= 0.0,
            },
            [24974] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 39,
                rank                = 2,
                lvl_req             = 30,
                lvl_max             = 39,
                lvl_outdated        = 39,
				lvl_scaling			= 0.0,
            },
            [24975] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 62,
                rank                = 3,
                lvl_req             = 40,
                lvl_max             = 49,
                lvl_outdated        = 49,
				lvl_scaling			= 0.0,
            },
            [24976] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 90,
                rank                = 4,
                lvl_req             = 50,
                lvl_max             = 59,
                lvl_outdated        = 59,
				lvl_scaling			= 0.0,
            },
            [24977] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 124,
                rank                = 5,
                lvl_req             = 60,
                lvl_max             = 69,
                lvl_outdated        = 69,
				lvl_scaling			= 0.0,
            },
            [27013] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 172,
                rank                = 6,
                lvl_req             = 70,
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 0.0,
            },
            [48468] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 215,
                rank                = 7,
                lvl_req             = 80,
                lvl_max             = 89,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            -- hurricane
            [16914] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 100,
                over_time_tick_freq = 1.0,
                over_time_duration  = 10.0,
                cast_time           = 10,
                rank                = 1,
                lvl_req             = 40,
                lvl_max             = 46,
                lvl_outdated        = 49,
                cost_base_percent   = 0.81,
                flags               = bit.bor(spell_flags.snare, spell_flags.aoe, spell_flags.over_time_crit),
                school              = magic_school.nature,
                coef                = 0.0,
                over_time_coef      = 0.12898,
				lvl_scaling			= 0.2,
            },
            [17401] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 142,
                rank                = 2,
                lvl_req             = 50,
                lvl_max             = 56,
                lvl_outdated        = 59,
				lvl_scaling			= 0.2,
            },
            [17402] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 190,
                rank                = 3,
                lvl_req             = 60,
                lvl_max             = 66,
                lvl_outdated        = 69,
				lvl_scaling			= 0.3,
            },
            [27012] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 303,
                rank                = 4,
                lvl_req             = 70,
                lvl_max             = 76,
                lvl_outdated        = 79,
				lvl_scaling			= 0.4,
            },
            [48467] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 451,
                rank                = 5,
                lvl_req             = 80,
                lvl_max             = 86,
                lvl_outdated        = 80,
				lvl_scaling			= 0.5,
            },
            -- entangling roots
            [339] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 5,
                over_time_tick_freq = 3.0,
                over_time_duration  = 27.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 8,
                lvl_max             = 0,
                lvl_outdated        = 17,
                cost_base_percent   = 0.07,
                flags               = spell_flags.snare,
                school              = magic_school.nature,
                coef                = 0.0,
                over_time_coef      = 0.1,
				lvl_scaling			= 0.0,
            },
            [1062] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 10,
                rank                = 2,
                lvl_req             = 18,
                lvl_max             = 0,
                lvl_outdated        = 27,
				lvl_scaling			= 0.0,
            },
            [5195] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 15,
                rank                = 3,
                lvl_req             = 28,
                lvl_max             = 0,
                lvl_outdated        = 37,
				lvl_scaling			= 0.0,
            },
            [5196] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 20,
                rank                = 4,
                lvl_req             = 38,
                lvl_max             = 0,
                lvl_outdated        = 47,
				lvl_scaling			= 0.0,
            },
            [9852] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 25,
                rank                = 5,
                lvl_req             = 48,
                lvl_max             = 0,
                lvl_outdated        = 57,
				lvl_scaling			= 0.0,
            },
            [9853] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 30,
                rank                = 6,
                lvl_req             = 58,
                lvl_max             = 0,
                lvl_outdated        = 67,
				lvl_scaling			= 0.0,
            },
            [26989] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 39,
                rank                = 7,
                lvl_req             = 68,
                lvl_max             = 77,
                lvl_outdated        = 77,
				lvl_scaling			= 0.0,
            },
            [53308] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 47,
                rank                = 8,
                lvl_req             = 78,
                lvl_max             = 87,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            -- lifebloom
            [33763] = {
                base_min            = 480.0,
                base_max            = 480.0, 
                over_time           = 32,
                over_time_tick_freq = 1.0,
                over_time_duration  = 7.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 64,
                lvl_max             = 71,
                lvl_outdated        = 71,
                cost_base_percent   = 0.14,
                flags               = spell_flags.heal,
                school              = magic_school.nature,
                coef                = 0.5160,
                over_time_coef      = 0.0952,
				lvl_scaling			= 0.0,
            },
            [48450] = {
                base_min            = 616.0,
                base_max            = 616.0, 
                over_time           = 41,
                rank                = 2,
                lvl_req             = 72,
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 0.0,
            },
            [48451] = {
                base_min            = 776.0,
                base_max            = 776.0, 
                over_time           = 53,
                rank                = 3,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            -- wild growth
            [48438] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 98,
                over_time_tick_freq = 1.0,
                over_time_duration  = 7.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 60,
                lvl_max             = 69,
                lvl_outdated        = 69,
                cost_base_percent   = 0.23,
                flags               = bit.bor(spell_flags.aoe, spell_flags.heal, spell_flags.cd),
                school              = magic_school.nature,
                coef                = 0.0,
                over_time_coef      = 0.1151,
				lvl_scaling			= 0.0,
            },
            [53248] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 123,
                rank                = 2,
                lvl_req             = 70,
                lvl_max             = 74,
                lvl_outdated        = 74,
				lvl_scaling			= 0.0,
            },
            [53249] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 177,
                rank                = 3,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 0.0,
            },
            [53251] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 206,
                rank                = 4,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            -- nourish
            [50464] = {
                base_min            = 1883,
                base_max            = 2187,
                over_time           = 0.0,
                over_time_tick_freq = 0.0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 80,
                lvl_max             = 85,
                lvl_outdated        = 80,
                cost_base_percent   = 0.18,
                flags               = spell_flags.heal,
                school              = magic_school.nature,
                coef                = 0.6731,
                over_time_coef      = 0.0,
				lvl_scaling			= 9.7,
            },
            -- force of nature
            -- TODO: what is the base tree hit? should scale with level
            [33831] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 150,
                over_time_tick_freq = 0.545,
                over_time_duration  = 30.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 50,
                lvl_max             = 0,
                lvl_outdated        = 80,
                cost_base_percent   = 0.12,
                -- TODO: crit?
                -- TODO: shouldn't scale with nature spell dmg mods
                flags               = spell_flags.cd,
                school              = magic_school.nature,
                coef                = 0.0,
                over_time_coef      = 0.065,
				lvl_scaling			= 0.0,
            },
            -- starfall?
            [48505] = {
                base_min            = 145.0,
                base_max            = 167.0, 
                over_time           = 26.0,
                over_time_tick_freq = 10.0,
                over_time_duration  = 10.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 60,
                lvl_max             = 69,
                lvl_outdated        = 69,
                cost_base_percent   = 0.35,
                flags               = bit.bor(spell_flags.cd, spell_flags.over_time_crit),
                school              = magic_school.arcane,
                coef                = 0.3,
                over_time_coef      = 0.127,
				lvl_scaling			= 0.0,
            },
            [53199] = {
                base_min            = 325.0,
                base_max            = 377.0, 
                over_time           = 58.0,
                rank                = 2,
                lvl_req             = 70,
                lvl_max             = 74,
                lvl_outdated        = 74,
				lvl_scaling			= 0.0,
            },
            [53200] = {
                base_min            = 475.0,
                base_max            = 551.0, 
                over_time           = 85.0,
                rank                = 3,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 0.0,
            },
            [53201] = {
                base_min            = 563.0,
                base_max            = 653.0, 
                over_time           = 101.0,
                rank                = 4,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            
            -- typhoon
            [50516] = {
                base_min            = 400.0,
                base_max            = 400.0, 
                over_time           = 0.0,
                over_time_tick_freq = 0.0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 50,
                lvl_max             = 59,
                lvl_outdated        = 59,
                cost_base_percent   = 0.25,
                flags               = bit.bor(spell_flags.snare, spell_flags.aoe, spell_flags.cd),
                school              = magic_school.nature,
                coef                = 0.1936,
                over_time_coef      = 0.0,
				lvl_scaling			= 0.0,
            },
            [53223] = {
                base_min            = 550.0,
                base_max            = 550.0, 
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 60,
                lvl_max             = 69,
                lvl_outdated        = 69,
				lvl_scaling			= 0.0,
            },
            [53225] = {
                base_min            = 735.0,
                base_max            = 735.0, 
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 70,
                lvl_max             = 74,
                lvl_outdated        = 74,
				lvl_scaling			= 0.0,
            },
            [53226] = {
                base_min            = 1010.0,
                base_max            = 1010.0, 
                over_time           = 0.0,
                rank                = 4,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 0.0,
            },
            [61384] = {
                base_min            = 1190.0,
                base_max            = 1190.0, 
                over_time           = 0.0,
                rank                = 5,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
        };

    elseif class == "PRIEST" then
        return {
            -- lesser heal
            [2050] = {
                base_min            = 46,
                base_max            = 56,
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 2.5,
                rank                = 1,
                lvl_req             = 1,
                lvl_max             = 3,
                lvl_outdated        = 3,
                cost_base_percent   = 0.27,
                flags               = spell_flags.heal,
                school              = magic_school.holy,
                coef                = 0.839,
                over_time_coef      = 0.0,
				lvl_scaling			= 0.9,
            },
            [2052] = {
                base_min            = 71,
                base_max            = 85,
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 4,
                lvl_max             = 9,
                lvl_outdated        = 9,
				lvl_scaling			= 1.1,
            },
            [2053] = {
                base_min            = 135,
                base_max            = 157,
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 10,
                lvl_max             = 15,
                lvl_outdated        = 15,
				lvl_scaling			= 1.6,
            },
            -- heal
            [2054] = {
                base_min            = 295,
                base_max            = 341,
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 3.0,
                rank                = 1,
                lvl_req             = 16,
                lvl_max             = 21,
                lvl_outdated        = 21,
                cost_base_percent   = 0.32,
                flags               = spell_flags.heal,
                school              = magic_school.holy,
                coef                = 1.611,
                over_time_coef      = 0.0,
				lvl_scaling			= 2.4,
            },
            [2055] = {
                base_min            = 429,
                base_max            = 491,
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 22,
                lvl_max             = 27,
                lvl_outdated        = 27,
				lvl_scaling			= 3.2,
            },
            [6063] = {
                base_min            = 566,
                base_max            = 642,
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 28,
                lvl_max             = 33,
                lvl_outdated        = 33,
				lvl_scaling			= 4,
            },
            [6064] = {
                base_min            = 712,
                base_max            = 804,
                over_time           = 0.0,
                rank                = 4,
                lvl_req             = 34,
                lvl_max             = 39,
                lvl_outdated        = 39,
				lvl_scaling			= 4.5,
            },
            -- greater heal
            [2060] = {
                base_min            = 899,
                base_max            = 1013,
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 3.0,
                rank                = 1,
                lvl_req             = 40,
                lvl_max             = 45,
                lvl_outdated        = 45,
                cost_base_percent   = 0.32,
                flags               = spell_flags.heal,
                school              = magic_school.holy,
                coef                = 1.611,
                over_time_coef      = 0.0,
				lvl_scaling			= 5.1,
            },
            [10963] = {
                base_min            = 1149,
                base_max            = 1289,
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 46,
                lvl_max             = 51,
                lvl_outdated        = 51,
				lvl_scaling			= 5.8,
            },
            [10964] = {
                base_min            = 1437,
                base_max            = 1609,
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 52,
                lvl_max             = 57,
                lvl_outdated        = 57,
				lvl_scaling			= 6.6,
            },
            [10965] = {
                base_min            = 1798,
                base_max            = 2006,
                over_time           = 0.0,
                rank                = 4,
                lvl_req             = 58,
                lvl_max             = 63,
                lvl_outdated        = 59,
				lvl_scaling			= 7.5,
            },
            [25314] = {
                base_min            = 1966,
                base_max            = 2194,
                over_time           = 0.0,
                rank                = 5,
                lvl_req             = 60,
                lvl_max             = 65,
                lvl_outdated        = 62,
				lvl_scaling			= 8.1,
            },
            [25210] = {
                base_min            = 2074,
                base_max            = 2410,
                over_time           = 0.0,
                rank                = 6,
                lvl_req             = 63,
                lvl_max             = 67,
                lvl_outdated        = 67,
				lvl_scaling			= 8.4,
            },
            [25213] = {
                base_min            = 2396,
                base_max            = 2784,
                over_time           = 0.0,
                rank                = 7,
                lvl_req             = 68,
                lvl_max             = 72,
                lvl_outdated        = 72,
				lvl_scaling			= 9.3,
            },
            [48062] = {
                base_min            = 3395,
                base_max            = 3945,
                over_time           = 0.0,
                rank                = 8,
                lvl_req             = 73,
                lvl_max             = 77,
                lvl_outdated        = 77,
				lvl_scaling			= 13.2,
            },
            [48063] = {
                base_min            = 3950,
                base_max            = 4590,
                over_time           = 0.0,
                rank                = 9,
                lvl_req             = 78,
                lvl_max             = 82,
                lvl_outdated        = 80,
				lvl_scaling			= 15.4,
            },
            -- flash heal
            [2061] = {
                base_min            = 193,
                base_max            = 237,
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 20,
                lvl_max             = 25,
                lvl_outdated        = 25,
                cost_base_percent   = 0.18,
                flags               = spell_flags.heal,
                school              = magic_school.holy,
                coef                = 0.807,
                over_time_coef      = 0.0,
				lvl_scaling			= 1.9,
            },
            [9472] = {
                base_min            = 258,
                base_max            = 314,
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 26,
                lvl_max             = 31,
                lvl_outdated        = 31,
				lvl_scaling			= 2.2,
            },
            [9473] = {
                base_min            = 327,
                base_max            = 393,
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 32,
                lvl_max             = 37,
                lvl_outdated        = 37,
				lvl_scaling			= 2.5,
            },
            [9474] = {
                base_min            = 400,
                base_max            = 478,
                over_time           = 0.0,
                rank                = 4,
                lvl_req             = 38,
                lvl_max             = 43,
                lvl_outdated        = 43,
				lvl_scaling			= 2.8,
            },
            [10915] = {
                base_min            = 518,
                base_max            = 616,
                over_time           = 0.0,
                rank                = 5,
                lvl_req             = 44,
                lvl_max             = 49,
                lvl_outdated        = 49,
				lvl_scaling			= 3.3,
            },
            [10916] = {
                base_min            = 644,
                base_max            = 764,
                over_time           = 0.0,
                rank                = 6,
                lvl_req             = 50,
                lvl_max             = 55,
                lvl_outdated        = 55,
				lvl_scaling			= 3.7,
            },
            [10917] = {
                base_min            = 812,
                base_max            = 958,
                over_time           = 0.0,
                rank                = 7,
                lvl_req             = 56,
                lvl_max             = 61,
                lvl_outdated        = 60,
				lvl_scaling			= 4.2,
            },
            [25233] = {
                base_min            = 913,
                base_max            = 1059,
                over_time           = 0.0,
                rank                = 8,
                lvl_req             = 61,
                lvl_max             = 65,
                lvl_outdated        = 66,
				lvl_scaling			= 4.7,
            },
            [25235] = {
                base_min            = 1101,
                base_max            = 1279,
                over_time           = 0.0,
                rank                = 9,
                lvl_req             = 67,
                lvl_max             = 71,
                lvl_outdated        = 72,
				lvl_scaling			= 5.2,
            },
            [48070] = {
                base_min            = 1578,
                base_max            = 1832,
                over_time           = 0.0,
                rank                = 10,
                lvl_req             = 73,
                lvl_max             = 78,
                lvl_outdated        = 78,
				lvl_scaling			= 8,
            },
            [48071] = {
                base_min            = 1887,
                base_max            = 2193,
                over_time           = 0.0,
                rank                = 11,
                lvl_req             = 79,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 9.7,
            },
            -- prayer of healing
            [596] = {
                base_min            = 301,
                base_max            = 321,
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 3.0,
                rank                = 1,
                lvl_req             = 30,
                lvl_max             = 39,
                lvl_outdated        = 39,
                cost_base_percent   = 0.48,
                flags               = bit.bor(spell_flags.heal, spell_flags.aoe),
                school              = magic_school.holy,
                coef                = 0.526,
                over_time_coef      = 0.0,
				lvl_scaling			= 1.3,
            },
            [996] = {
                base_min            = 444,
                base_max            = 472,
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 40,
                lvl_max             = 49,
                lvl_outdated        = 49,
				lvl_scaling			= 1.6,
            },
            [10960] = {
                base_min            = 657,
                base_max            = 695,
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 50,
                lvl_max             = 59,
                lvl_outdated        = 59,
				lvl_scaling			= 2,
            },
            [10961] = {
                base_min            = 939,
                base_max            = 991,
                over_time           = 0.0,
                rank                = 4,
                lvl_req             = 60,
                lvl_max             = 69,
                lvl_outdated        = 60,
				lvl_scaling			= 2.4,
            },
            [25316] = {
                base_min            = 997,
                base_max            = 1053,
                over_time           = 0.0,
                rank                = 5,
                lvl_req             = 60,
                lvl_max             = 69,
                lvl_outdated        = 67,
				lvl_scaling			= 2.5,
            },
            [25308] = {
                base_min            = 1246,
                base_max            = 1316,
                over_time           = 0.0,
                rank                = 6,
                lvl_req             = 68,
                lvl_max             = 75,
                lvl_outdated        = 75,
				lvl_scaling			= 2.8,
            },
            [48072] = {
                base_min            = 2091,
                base_max            = 2209,
                over_time           = 0.0,
                rank                = 7,
                lvl_req             = 76,
                lvl_max             = 82,
                lvl_outdated        = 80,
				lvl_scaling			= 4.7,
            },
            -- renew
            [139] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 9,
                over_time_tick_freq = 3,
                over_time_duration  = 15.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 8,
                lvl_max             = 13,
                lvl_outdated        = 13,
                cost_base_percent   = 0.17,
                flags               = spell_flags.heal,
                school              = magic_school.holy,
                coef                = 0.0,
                over_time_coef      = 0.376,
				lvl_scaling			= 0.0,
            },
            [6074] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 20,
                rank                = 2,
                lvl_req             = 14,
                lvl_max             = 19,
                lvl_outdated        = 19,
				lvl_scaling			= 0.0,
            },
            [6075] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 35,
                rank                = 3,
                lvl_req             = 20,
                lvl_max             = 25,
                lvl_outdated        = 25,
				lvl_scaling			= 0.0,
            },
            [6076] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 49,
                rank                = 4,
                lvl_req             = 26,
                lvl_max             = 31,
                lvl_outdated        = 31,
				lvl_scaling			= 0.0,
            },
            [6077] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 63,
                rank                = 5,
                lvl_req             = 32,
                lvl_max             = 37,
                lvl_outdated        = 37,
				lvl_scaling			= 0.0,
            },
            [6078] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 80,
                rank                = 6,
                lvl_req             = 38,
                lvl_max             = 43,
                lvl_outdated        = 43,
				lvl_scaling			= 0.0,
            },
            [10927] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 102,
                rank                = 7,
                lvl_req             = 44,
                lvl_max             = 49,
                lvl_outdated        = 49,
				lvl_scaling			= 0.0,
            },
            [10928] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 130,
                rank                = 8,
                lvl_req             = 50,
                lvl_max             = 55,
                lvl_outdated        = 55,
				lvl_scaling			= 0.0,
            },
            [10929] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 162,
                rank                = 9,
                lvl_req             = 56,
                lvl_max             = 61,
                lvl_outdated        = 59,
				lvl_scaling			= 0.0,
            },
            [25315] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 194,
                rank                = 10,
                lvl_req             = 60,
                lvl_max             = 65,
                lvl_outdated        = 64,
				lvl_scaling			= 0.0,
            },
            [25221] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 202,
                rank                = 11,
                lvl_req             = 65,
                lvl_max             = 69,
                lvl_outdated        = 69,
				lvl_scaling			= 0.0,
            },
            [25222] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 222,
                rank                = 11,
                lvl_req             = 70,
                lvl_max             = 74,
                lvl_outdated        = 74,
				lvl_scaling			= 0.0,
            },
            [48067] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 247,
                rank                = 12,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 0.0,
            },
            [48068] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 280,
                rank                = 13,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            -- power word: shield
            [17] = {
                base_min            = 44,
                base_max            = 44,
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 6,
                lvl_max             = 11,
                lvl_outdated        = 11,
                cost_base_percent   = 0.23,
                flags               = spell_flags.absorb,
                school              = magic_school.holy,
                coef                = 0.8057,
                over_time_coef      = 0.0,
				lvl_scaling			= 0.8,
            },
            [592] = {
                base_min            = 88,
                base_max            = 88,
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 12,
                lvl_max             = 17,
                lvl_outdated        = 17,
				lvl_scaling			= 1.2,
            },
            [600] = {
                base_min            = 158,
                base_max            = 158,
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 18,
                lvl_max             = 23,
                lvl_outdated        = 23,
				lvl_scaling			= 1.6,
            },
            [3747] = {
                base_min            = 234,
                base_max            = 234,
                over_time           = 0.0,
                rank                = 4,
                lvl_req             = 24,
                lvl_max             = 29,
                lvl_outdated        = 29,
				lvl_scaling			= 2,
            },
            [6065] = {
                base_min            = 301,
                base_max            = 301,
                over_time           = 0.0,
                rank                = 5,
                lvl_req             = 30,
                lvl_max             = 35,
                lvl_outdated        = 35,
				lvl_scaling			= 2.3,
            },
            [6066] = {
                base_min            = 381,
                base_max            = 381,
                over_time           = 0.0,
                rank                = 6,
                lvl_req             = 36,
                lvl_max             = 41,
                lvl_outdated        = 41,
				lvl_scaling			= 2.6,
            },
            [10898] = {
                base_min            = 484,
                base_max            = 484,
                over_time           = 0.0,
                rank                = 7,
                lvl_req             = 42,
                lvl_max             = 47,
                lvl_outdated        = 47,
				lvl_scaling			= 3,
            },
            [10899] = {
                base_min            = 605,
                base_max            = 605,
                over_time           = 0.0,
                rank                = 8,
                lvl_req             = 48,
                lvl_max             = 53,
                lvl_outdated        = 53,
				lvl_scaling			= 3.4,
            },
            [10900] = {
                base_min            = 763,
                base_max            = 763,
                over_time           = 0.0,
                rank                = 9,
                lvl_req             = 54,
                lvl_max             = 59,
                lvl_outdated        = 59,
				lvl_scaling			= 3.9,
            },
            [10901] = {
                base_min            = 942,
                base_max            = 942,
                over_time           = 0.0,
                rank                = 10,
                lvl_req             = 60,
                lvl_max             = 65,
                lvl_outdated        = 64,
				lvl_scaling			= 4.3,
            },
            [25217] = {
                base_min            = 1125,
                base_max            = 1125,
                over_time           = 0.0,
                rank                = 11,
                lvl_req             = 65,
                lvl_max             = 69,
                lvl_outdated        = 69,
				lvl_scaling			= 4.7,
            },
            [25218] = {
                base_min            = 1265,
                base_max            = 1265,
                over_time           = 0.0,
                rank                = 12,
                lvl_req             = 70,
                lvl_max             = 74,
                lvl_outdated        = 74,
				lvl_scaling			= 5.1,
            },
            [48065] = {
                base_min            = 1920,
                base_max            = 1920,
                over_time           = 0.0,
                rank                = 13,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 7.7,
            },
            [48066] = {
                base_min            = 2230,
                base_max            = 2230,
                over_time           = 0.0,
                rank                = 14,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 9,
            },
            -- holy nova
            [15237] = {
                base_min            = 28,
                base_max            = 32,
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 20,
                lvl_max             = 26,
                lvl_outdated        = 27,
                cost_base_percent   = 0.2,
                flags               = spell_flags.aoe,
                school              = magic_school.holy,
                coef                = 0.2143,
                over_time_coef      = 0.0,
                lvl_scaling         = 0.2,
                healing_version = {
                    base_min            = 52,
                    base_max            = 60,
                    over_time           = 0.0,
                    over_time_tick_freq = 0,
                    over_time_duration  = 0.0,
                    cast_time           = 1.5,
                    rank                = 1,
                    lvl_req             = 20,
                    lvl_max             = 26,
                    lvl_outdated        = 27,
                    cost_base_percent   = 0.2,
                    flags               = bit.bor(spell_flags.aoe, spell_flags.heal),
                    school              = magic_school.holy,
                    coef                = 0.3035,
                    over_time_coef      = 0.0,
                    lvl_scaling         = 0.4,
                }
            },
            [15430] = {
                base_min            = 50,
                base_max            = 58,
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 28,
                lvl_max             = 34,
                lvl_outdated        = 35,
                lvl_scaling         = 0.4,
                healing_version = {
                    base_min            = 86,
                    base_max            = 98,
                    over_time           = 0.0,
                    rank                = 2,
                    lvl_req             = 28,
                    lvl_max             = 34,
                    lvl_outdated        = 35,
                    lvl_scaling         = 0.5,
                }
            },
            [15431] = {
                base_min            = 76,
                base_max            = 88,
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 36,
                lvl_max             = 42,
                lvl_outdated        = 43,
                lvl_scaling         = 0.6,
                healing_version = {
                    base_min            = 121,
                    base_max            = 139,
                    over_time           = 0.0,
                    rank                = 3,
                    lvl_req             = 36,
                    lvl_max             = 42,
                    lvl_outdated        = 43,
                    lvl_scaling         = 0.6,
                }
            },
            [27799] = {
                base_min            = 106,
                base_max            = 122,
                over_time           = 0.0,
                rank                = 4,
                lvl_req             = 44,
                lvl_max             = 50,
                lvl_outdated        = 51,
                lvl_scaling         = 0.8,
                healing_version = {
                    base_min            = 161,
                    base_max            = 187,
                    over_time           = 0.0,
                    rank                = 4,
                    lvl_req             = 44,
                    lvl_max             = 50,
                    lvl_outdated        = 51,
                    lvl_scaling         = 0.7,
                }
            },
            [27800] = {
                base_min            = 140,
                base_max            = 162,
                over_time           = 0.0,
                rank                = 5,
                lvl_req             = 52,
                lvl_max             = 58,
                lvl_outdated        = 59,
                lvl_scaling         = 1,
                healing_version = {
                    base_min            = 235,
                    base_max            = 271,
                    over_time           = 0.0,
                    rank                = 5,
                    lvl_req             = 52,
                    lvl_max             = 58,
                    lvl_outdated        = 59,
                    lvl_scaling         = 0.8,
                }
            },
            [27801] = {
                base_min            = 181,
                base_max            = 209,
                over_time           = 0.0,
                rank                = 6,
                lvl_req             = 60,
                lvl_max             = 66,
                lvl_outdated        = 67,
                lvl_scaling         = 1.2,
                healing_version = {
                    base_min            = 302,
                    base_max            = 350,
                    over_time           = 0.0,
                    rank                = 6,
                    lvl_req             = 60,
                    lvl_max             = 66,
                    lvl_outdated        = 67,
                    lvl_scaling         = 0.9,
                }
            },
            [25331] = {
                base_min            = 242,
                base_max            = 280,
                over_time           = 0.0,
                rank                = 7,
                lvl_req             = 68,
                lvl_max             = 74,
                lvl_outdated        = 74,
                lvl_scaling         = 1.4,
                healing_version = {
                    base_min            = 384,
                    base_max            = 446,
                    over_time           = 0.0,
                    rank                = 7,
                    lvl_req             = 68,
                    lvl_max             = 74,
                    lvl_outdated        = 74,
                    lvl_scaling         = 1.0,
                }
            },
            [48077] = {
                base_min            = 333,
                base_max            = 387,
                over_time           = 0.0,
                rank                = 8,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 79,
                lvl_scaling         = 2,
                healing_version = {
                    base_min            = 611,
                    base_max            = 709,
                    over_time           = 0.0,
                    rank                = 8,
                    lvl_req             = 75,
                    lvl_max             = 79,
                    lvl_outdated        = 79,
                    lvl_scaling         = 1.6,
                }
            },
            [48078] = {
                base_min            = 398,
                base_max            = 462,
                over_time           = 0.0,
                rank                = 9,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
                lvl_scaling         = 3,
                healing_version = {
                    base_min            = 713,
                    base_max            = 827,
                    over_time           = 0.0,
                    rank                = 9,
                    lvl_req             = 80,
                    lvl_max             = 84,
                    lvl_outdated        = 80,
                    lvl_scaling         = 1.9,
                }
            },
            -- holy fire
            [14914] = {
                base_min            = 102,
                base_max            = 128,
                over_time           = 3,
                over_time_tick_freq = 1,
                over_time_duration  = 7.0,
                cast_time           = 2.0,
                rank                = 1,
                lvl_req             = 20,
                lvl_max             = 24,
                lvl_outdated        = 23,
                cost_base_percent   = 0.11,
                flags               = spell_flags.cd,
                school              = magic_school.holy,
                coef                = 0.575,
                over_time_coef      = 0.0529,
				lvl_scaling			= 1.5,
            },
            [15262] = {
                base_min            = 137,
                base_max            = 173,
                over_time           = 4,
                rank                = 2,
                lvl_req             = 24,
                lvl_max             = 30,
                lvl_outdated        = 29,
				lvl_scaling			= 1.7,
            },
            [15263] = {
                base_min            = 200,
                base_max            = 252,
                over_time           = 6,
                rank                = 3,
                lvl_req             = 30,
                lvl_max             = 36,
                lvl_outdated        = 25,
				lvl_scaling			= 2,
            },
            [15264] = {
                base_min            = 267,
                base_max            = 339,
                over_time           = 8,
                rank                = 4,
                lvl_req             = 36,
                lvl_max             = 42,
                lvl_outdated        = 41,
				lvl_scaling			= 2.2,
            },
            [15265] = {
                base_min            = 348,
                base_max            = 440,
                over_time           = 10,
                rank                = 5,
                lvl_req             = 42,
                lvl_max             = 48,
                lvl_outdated        = 47,
				lvl_scaling			= 2.5,
            },
            [15266] = {
                base_min            = 430,
                base_max            = 546,
                over_time           = 13,
                rank                = 6,
                lvl_req             = 48,
                lvl_max             = 54,
                lvl_outdated        = 53,
				lvl_scaling			= 2.9,
            },
            [15267] = {
                base_min            = 529,
                base_max            = 671,
                over_time           = 16,
                rank                = 7,
                lvl_req             = 54,
                lvl_max             = 60,
                lvl_outdated        = 59,
				lvl_scaling			= 3.2,
            },
            [15261] = {
                base_min            = 639,
                base_max            = 811,
                over_time           = 18,
                rank                = 8,
                lvl_req             = 60,
                lvl_max             = 66,
                lvl_outdated        = 65,
				lvl_scaling			= 3.4,
            },
            [25384] = {
                base_min            = 705,
                base_max            = 895,
                over_time           = 21,
                rank                = 9,
                lvl_req             = 66,
                lvl_max             = 70,
                lvl_outdated        = 71,
				lvl_scaling			= 3.6,
            },
            [48134] = {
                base_min            = 732,
                base_max            = 928,
                over_time           = 41,
                rank                = 10,
                lvl_req             = 72,
                lvl_max             = 76,
                lvl_outdated        = 77,
				lvl_scaling			= 4,
            },
            [48135] = {
                base_min            = 890,
                base_max            = 1130,
                over_time           = 50,
                rank                = 11,
                lvl_req             = 78,
                lvl_max             = 82,
                lvl_outdated        = 80,
				lvl_scaling			= 5,
            },
            -- smite
            [585] = {
                base_min            = 13,
                base_max            = 17,
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 1,
                lvl_max             = 6,
                lvl_outdated        = 5,
                cost_base_percent   = 0.12,
                flags               = 0,
                school              = magic_school.holy,
                coef                = 0.7143,
                over_time_coef      = 0.0,
				lvl_scaling			= 0.5,
            },
            [591] = {
                base_min            = 25,
                base_max            = 31,
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 5,
                lvl_max             = 11,
                lvl_outdated        = 13,
				lvl_scaling			= 0.6,
            },
            [598] = {
                base_min            = 54,
                base_max            = 62,
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 14,
                lvl_max             = 19,
                lvl_outdated        = 21,
				lvl_scaling			= 0.9,
            },
            [984] = {
                base_min            = 91,
                base_max            = 105,
                over_time           = 0.0,
                rank                = 4,
                lvl_req             = 22,
                lvl_max             = 27,
                lvl_outdated        = 29,
				lvl_scaling			= 1.3,
            },
            [1004] = {
                base_min            = 150,
                base_max            = 170,
                over_time           = 0.0,
                rank                = 5,
                lvl_req             = 30,
                lvl_max             = 35,
                lvl_outdated        = 37,
				lvl_scaling			= 1.6,
            },
            [6060] = {
                base_min            = 212,
                base_max            = 240,
                over_time           = 0.0,
                rank                = 6,
                lvl_req             = 38,
                lvl_max             = 43,
                lvl_outdated        = 45,
				lvl_scaling			= 2,
            },
            [10933] = {
                base_min            = 287,
                base_max            = 323,
                over_time           = 0.0,
                rank                = 7,
                lvl_req             = 46,
                lvl_max             = 51,
                lvl_outdated        = 53,
				lvl_scaling			= 2.3,
            },
            [10934] = {
                base_min            = 371,
                base_max            = 415,
                over_time           = 0.0,
                rank                = 8,
                lvl_req             = 54,
                lvl_max             = 59,
                lvl_outdated        = 60,
				lvl_scaling			= 2.7,
            },
            [25363] = {
                base_min            = 405,
                base_max            = 453,
                over_time           = 0.0,
                rank                = 9,
                lvl_req             = 61,
                lvl_max             = 66,
                lvl_outdated        = 68,
				lvl_scaling			= 3.4,
            },
            [25364] = {
                base_min            = 545,
                base_max            = 611,
                over_time           = 0.0,
                rank                = 10,
                lvl_req             = 69,
                lvl_max             = 73,
                lvl_outdated        = 73,
				lvl_scaling			= 4.1,
            },
            [48122] = {
                base_min            = 604,
                base_max            = 676,
                over_time           = 0.0,
                rank                = 11,
                lvl_req             = 74,
                lvl_max             = 78,
                lvl_outdated        = 78,
				lvl_scaling			= 5,
            },
            [48123] = {
                base_min            = 707,
                base_max            = 793,
                over_time           = 0.0,
                rank                = 12,
                lvl_req             = 79,
                lvl_max             = 83,
                lvl_outdated        = 80,
				lvl_scaling			= 6,
            },
            -- binding heal
            [32546] = {
                base_min            = 1042,
                base_max            = 1338,
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 64,
                lvl_max             = 71,
                lvl_outdated        = 71,
                cost_base_percent   = 0.27,
                flags               = bit.bor(spell_flags.aoe, spell_flags.heal),
                school              = magic_school.holy,
                coef                = 0.8057,
                over_time_coef      = 0.0,
				lvl_scaling			= 1.9,
            },
            [48119] = {
                base_min            = 1619,
                base_max            = 2081,
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 72,
                lvl_max             = 76,
                lvl_outdated        = 77,
				lvl_scaling			= 3,
            },
            [48120] = {
                base_min            = 1952,
                base_max            = 2508,
                over_time           = 0.0,
                rank                = 1,
                lvl_req             = 78,
                lvl_max             = 82,
                lvl_outdated        = 80,
				lvl_scaling			= 3.6,
            },
            -- circle of healing
            [34861] = {
                base_min            = 343,
                base_max            = 379,
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 50,
                lvl_max             = 54,
                lvl_outdated        = 55,
                cost_base_percent   = 0.21,
                flags               = bit.bor(spell_flags.aoe, spell_flags.heal),
                school              = magic_school.holy,
                coef                = 0.4029,
                over_time_coef      = 0.0,
				lvl_scaling			= 1,
            },
            [34863] = {
                base_min            = 403,
                base_max            = 445,
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 56,
                lvl_max             = 60,
                lvl_outdated        = 59,
				lvl_scaling			= 1.2,
            },
            [34864] = {
                base_min            = 458,
                base_max            = 506,
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 60,
                lvl_max             = 64,
                lvl_outdated        = 64,
				lvl_scaling			= 1.4,
            },
            [34865] = {
                base_min            = 518,
                base_max            = 572,
                over_time           = 0.0,
                rank                = 4,
                lvl_req             = 65,
                lvl_max             = 69,
                lvl_outdated        = 69,
				lvl_scaling			= 1.6,
            },
            [34866] = {
                base_min            = 572,
                base_max            = 632,
                over_time           = 0.0,
                rank                = 5,
                lvl_req             = 70,
                lvl_max             = 74,
                lvl_outdated        = 74,
				lvl_scaling			= 1.8,
            },
            [48088] = {
                base_min            = 825,
                base_max            = 911,
                over_time           = 0.0,
                rank                = 6,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 2,
            },
            [48089] = {
                base_min            = 958,
                base_max            = 1058,
                over_time           = 0.0,
                rank                = 7,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 3,
            },
            -- desperate prayer
            [19236] = {
                base_min            = 263,
                base_max            = 325,
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 20,
                lvl_max             = 26,
                lvl_outdated        = 25,
                cost_base_percent   = 0.21,
                flags               = bit.bor(spell_flags.heal, spell_flags.cd),
                school              = magic_school.holy,
                coef                = 0.8057,
                over_time_coef      = 0.0,
				lvl_scaling			= 0.0,
            },
            [19238] = {
                base_min            = 447,
                base_max            = 543,
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 26,
                lvl_max             = 32,
                lvl_outdated        = 33,
				lvl_scaling			= 0.0,
            },
            [19240] = {
                base_min            = 588,
                base_max            = 708,
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 34,
                lvl_max             = 40,
                lvl_outdated        = 41,
				lvl_scaling			= 0.0,
            },
            [19241] = {
                base_min            = 834,
                base_max            = 994,
                over_time           = 0.0,
                rank                = 4,
                lvl_req             = 42,
                lvl_max             = 48,
                lvl_outdated        = 49,
				lvl_scaling			= 6.4,
            },
            [19242] = {
                base_min            = 1101,
                base_max            = 1305,
                over_time           = 0.0,
                rank                = 5,
                lvl_req             = 50,
                lvl_max             = 0,
                lvl_outdated        = 57,
				lvl_scaling			= 0.0,
            },
            [19243] = {
                base_min            = 1324,
                base_max            = 1562,
                over_time           = 0.0,
                rank                = 6,
                lvl_req             = 58,
                lvl_max             = 0,
                lvl_outdated        = 65,
				lvl_scaling			= 0.0,
            },
            [25437] = {
                base_min            = 1601,
                base_max            = 1887,
                over_time           = 0.0,
                rank                = 7,
                lvl_req             = 66,
                lvl_max             = 0,
                lvl_outdated        = 72,
				lvl_scaling			= 0.0,
            },
            [48172] = {
                base_min            = 3111,
                base_max            = 3669,
                over_time           = 0.0,
                rank                = 8,
                lvl_req             = 73,
                lvl_max             = 0,
                lvl_outdated        = 79,
				lvl_scaling			= 0.0,
            },
            [48173] = {
                base_min            = 3716,
                base_max            = 4384,
                over_time           = 0.0,
                rank                = 9,
                lvl_req             = 80,
                lvl_max             = 0,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            -- penance
            [47540] = {
                base_min            = 240.0,
                base_max            = 240.0, 
                over_time           = 0.0,
                over_time_tick_freq = 3.0/2.0,
                over_time_duration  = 2.0,
                cast_time           = 2.0,
                rank                = 1,
                lvl_req             = 60,
                lvl_max             = 69,
                lvl_outdated        = 69,
                cost_base_percent   = 0.16,
                flags               = spell_flags.cd,
                school              = magic_school.holy,
                coef                = 0.1905,
                over_time_coef      = 0.0,
				lvl_scaling			= 0.0,
                healing_version = {
                    base_min            = 670.0,
                    base_max            = 756.0, 
                    over_time           = 0.0,
                    over_time_tick_freq = 3.0/2.0,
                    over_time_duration  = 2.0,
                    cast_time           = 2.0,
                    rank                = 1,
                    lvl_req             = 60,
                    lvl_max             = 69,
                    lvl_outdated        = 69,
                    cost_base_percent   = 0.16,
                    flags               = bit.bor(spell_flags.heal, spell_flags.cd),
                    school              = magic_school.holy,
                    coef                = 0.5362,
                    over_time_coef      = 0.0,
				    lvl_scaling			= 0.0,
                }
            },
            [53005] = {
                base_min            = 292.0,
                base_max            = 292.0, 
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 70,
                lvl_max             = 74,
                lvl_outdated        = 74,
				lvl_scaling			= 0.0,
                healing_version = {
                    base_min            = 805.0,
                    base_max            = 909.0, 
                    over_time           = 0.0,
                    rank                = 2,
                    lvl_req             = 70,
                    lvl_max             = 74,
                    lvl_outdated        = 74,
				    lvl_scaling			= 0.0,
                }
            },
            [53006] = {
                base_min            = 333.0,
                base_max            = 333.0, 
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 0.0,
                healing_version = {
                    base_min            = 1278.0,
                    base_max            = 1442.0, 
                    over_time           = 0.0,
                    rank                = 3,
                    lvl_req             = 75,
                    lvl_max             = 79,
                    lvl_outdated        = 79,
				    lvl_scaling			= 0.0,
                }
            },
            [53007] = {
                base_min            = 375.0,
                base_max            = 375.0, 
                over_time           = 0.0,
                rank                = 4,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
                healing_version = {
                    base_min            = 1484.0,
                    base_max            = 1676.0, 
                    over_time           = 0.0,
                    rank                = 4,
                    lvl_req             = 80,
                    lvl_max             = 84,
                    lvl_outdated        = 80,
				    lvl_scaling			= 0.0,
                }
            },
            -- prayer of mending
            [33076] = {
                base_min            = 800.0,
                base_max            = 800.0, 
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 68,
                lvl_max             = 72,
                lvl_outdated        = 73,
                cost_base_percent   = 0.15,
                flags               = bit.bor(spell_flags.heal, spell_flags.cd),
                school              = magic_school.holy,
                coef                = 0.8057,
                over_time_coef      = 0.0,
				lvl_scaling			= 0.0,
            },
            [48112] = {
                base_min            = 905.0,
                base_max            = 905.0, 
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 74,
                lvl_max             = 78,
                lvl_outdated        = 78,
				lvl_scaling			= 0.0,
            },
            [48113] = {
                base_min            = 1043.0,
                base_max            = 1043.0, 
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 79,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            -- shadow word: pain
            [589] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 5,
                over_time_tick_freq = 3,
                over_time_duration  = 18.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 4,
                lvl_max             = 9,
                lvl_outdated        = 9,
                cost_base_percent   = 0.22,
                flags               = 0,
                school              = magic_school.shadow,
                coef                = 0.0,
                over_time_coef      = 1.1/6,
				lvl_scaling			= 0.0,
            },
            [594] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 10,
                rank                = 2,
                lvl_req             = 10,
                lvl_max             = 17,
                lvl_outdated        = 17,
				lvl_scaling			= 0.0,
            },
            [970] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 20,
                rank                = 3,
                lvl_req             = 18,
                lvl_max             = 25,
                lvl_outdated        = 25,
				lvl_scaling			= 0.0,
            },
            [992] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 35,
                rank                = 4,
                lvl_req             = 26,
                lvl_max             = 33,
                lvl_outdated        = 33,
				lvl_scaling			= 0.0,
            },
            [2767] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 55,
                rank                = 5,
                lvl_req             = 34,
                lvl_max             = 41,
                lvl_outdated        = 41,
				lvl_scaling			= 0.0,
            },
            [10892] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 77,
                rank                = 6,
                lvl_req             = 42,
                lvl_max             = 49,
                lvl_outdated        = 49,
				lvl_scaling			= 0.0,
            },
            [10893] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 101,
                rank                = 7,
                lvl_req             = 50,
                lvl_max             = 57,
                lvl_outdated        = 57,
				lvl_scaling			= 0.0,
            },
            [10894] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 128,
                rank                = 8,
                lvl_req             = 58,
                lvl_max             = 64,
                lvl_outdated        = 64,
				lvl_scaling			= 0.0,
            },
            [25367] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 151,
                rank                = 9,
                lvl_req             = 65,
                lvl_max             = 69,
                lvl_outdated        = 69,
				lvl_scaling			= 0.0,
            },
            [25368] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 186,
                rank                = 10,
                lvl_req             = 70,
                lvl_max             = 74,
                lvl_outdated        = 74,
				lvl_scaling			= 0.0,
            },
            [48124] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 196,
                rank                = 11,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 0.0,
            },
            [48125] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 230,
                rank                = 12,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            -- mind blast
            [8092] = {
                base_min            = 39,
                base_max            = 43,
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 10,
                lvl_max             = 15,
                lvl_outdated        = 15,
                cost_base_percent   = 0.17,
                flags               = spell_flags.cd,
                school              = magic_school.shadow,
                coef                = 0.4286,
                over_time_coef      = 0.0,
				lvl_scaling			= 0.6,
            },
            [8102] = {
                base_min            = 72,
                base_max            = 78,
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 16,
                lvl_max             = 21,
                lvl_outdated        = 21,
				lvl_scaling			= 0.9,
            },
            [8103] = {
                base_min            = 112,
                base_max            = 120,
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 22,
                lvl_max             = 27,
                lvl_outdated        = 27,
				lvl_scaling			= 1.1,
            },
            [8104] = {
                base_min            = 167,
                base_max            = 177,
                over_time           = 0.0,
                rank                = 4,
                lvl_req             = 28,
                lvl_max             = 33,
                lvl_outdated        = 33,
				lvl_scaling			= 1.4,
            },
            [8105] = {
                base_min            = 217,
                base_max            = 231,
                over_time           = 0.0,
                rank                = 5,
                lvl_req             = 34,
                lvl_max             = 39,
                lvl_outdated        = 39,
				lvl_scaling			= 1.6,
            },
            [8106] = {
                base_min            = 279,
                base_max            = 297,
                over_time           = 0.0,
                rank                = 6,
                lvl_req             = 40,
                lvl_max             = 45,
                lvl_outdated        = 45,
				lvl_scaling			= 1.9,
            },
            [10945] = {
                base_min            = 346,
                base_max            = 366,
                over_time           = 0.0,
                rank                = 7,
                lvl_req             = 46,
                lvl_max             = 51,
                lvl_outdated        = 51,
				lvl_scaling			= 2.1,
            },
            [10946] = {
                base_min            = 425,
                base_max            = 449,
                over_time           = 0.0,
                rank                = 8,
                lvl_req             = 52,
                lvl_max             = 57,
                lvl_outdated        = 57,
				lvl_scaling			= 2.4,
            },
            [10947] = {
                base_min            = 503,
                base_max            = 531,
                over_time           = 0.0,
                rank                = 9,
                lvl_req             = 58,
                lvl_max             = 63,
                lvl_outdated        = 62,
				lvl_scaling			= 2.6,
            },
            [25372] = {
                base_min            = 557,
                base_max            = 587,
                over_time           = 0.0,
                rank                = 10,
                lvl_req             = 63,
                lvl_max             = 68,
                lvl_outdated        = 68,
				lvl_scaling			= 2.9,
            },
            [25375] = {
                base_min            = 708,
                base_max            = 748,
                over_time           = 0.0,
                rank                = 11,
                lvl_req             = 69,
                lvl_max             = 73,
                lvl_outdated        = 63,
				lvl_scaling			= 3.2,
            },
            [48126] = {
                base_min            = 837,
                base_max            = 883,
                over_time           = 0.0,
                rank                = 12,
                lvl_req             = 74,
                lvl_max             = 78,
                lvl_outdated        = 78,
				lvl_scaling			= 4,
            },
            [48127] = {
                base_min            = 992,
                base_max            = 1048,
                over_time           = 0.0,
                rank                = 13,
                lvl_req             = 79,
                lvl_max             = 83,
                lvl_outdated        = 80,
				lvl_scaling			= 5,
            },
            -- mind flay
            [15407] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 15,
                over_time_tick_freq = 1,
                over_time_duration  = 3.0,
                cast_time           = 3.0,
                rank                = 1,
                lvl_req             = 20,
                lvl_max             = 28,
                lvl_outdated        = 27,
                cost_base_percent   = 0.09,
                flags               = bit.bor(spell_flags.snare, spell_flags.over_time_crit, spell_flags.channel_missable),
                school              = magic_school.shadow,
                coef                = 0.0,
                over_time_coef      = 0.2570,
				lvl_scaling			= 0.0,
            },
            [17311] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 36,
                rank                = 2,
                lvl_req             = 28,
                lvl_max             = 36,
                lvl_outdated        = 35,
				lvl_scaling			= 0.0,
            },
            [17312] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 53,
                rank                = 3,
                lvl_req             = 36,
                lvl_max             = 44,
                lvl_outdated        = 43,
				lvl_scaling			= 0.0,
            },
            [17313] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 74,
                rank                = 4,
                lvl_req             = 44,
                lvl_max             = 52,
                lvl_outdated        = 51,
				lvl_scaling			= 0.0,
            },
            [17314] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 94,
                rank                = 5,
                lvl_req             = 52,
                lvl_max             = 60,
                lvl_outdated        = 59,
				lvl_scaling			= 0.0,
            },
            [18807] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 121,
                rank                = 6,
                lvl_req             = 60,
                lvl_max             = 68,
                lvl_outdated        = 67,
				lvl_scaling			= 0.0,
            },
            [25387] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 150,
                rank                = 7,
                lvl_req             = 68,
                lvl_max             = 72,
                lvl_outdated        = 73,
				lvl_scaling			= 0.0,
            },
            [48155] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 164,
                rank                = 8,
                lvl_req             = 74,
                lvl_max             = 78,
                lvl_outdated        = 79,
				lvl_scaling			= 0.0,
            },
            [48156] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 196,
                rank                = 9,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            -- devouring plague
            [2944] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 19,
                over_time_tick_freq = 3,
                over_time_duration  = 24.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 20,
                lvl_max             = 0,
                lvl_outdated        = 27,
                cost_base_percent   = 0.25,
                flags               = 0,
                school              = magic_school.shadow,
                coef                = 0.0,
                over_time_coef      = 0.185,
				lvl_scaling			= 0.0,
            },
            [19276] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 34,
                rank                = 2,
                lvl_req             = 28,
                lvl_max             = 0,
                lvl_outdated        = 35,
				lvl_scaling			= 0.0,
            },
            [19277] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 50,
                rank                = 3,
                lvl_req             = 36,
                lvl_max             = 0,
                lvl_outdated        = 43,
				lvl_scaling			= 0.0,
            },
            [19278] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 68,
                rank                = 4,
                lvl_req             = 44,
                lvl_max             = 0,
                lvl_outdated        = 51,
				lvl_scaling			= 0.0,
            },
            [19279] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 89,
                rank                = 5,
                lvl_req             = 52,
                lvl_max             = 0,
                lvl_outdated        = 59,
				lvl_scaling			= 0.0,
            },
            [19280] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 113,
                rank                = 6,
                lvl_req             = 60,
                lvl_max             = 0,
                lvl_outdated        = 67,
				lvl_scaling			= 0.0,
            },
            [25467] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 136,
                rank                = 7,
                lvl_req             = 68,
                lvl_max             = 0,
                lvl_outdated        = 72,
				lvl_scaling			= 0.0,
            },
            [48299] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 143,
                rank                = 8,
                lvl_req             = 73,
                lvl_max             = 0,
                lvl_outdated        = 78,
				lvl_scaling			= 0.0,
            },
            [48300] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 172,
                rank                = 9,
                lvl_req             = 79,
                lvl_max             = 0,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            -- vampiric touch
            [34914] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 90,
                over_time_tick_freq = 3,
                over_time_duration  = 15.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 50,
                lvl_max             = 59,
                lvl_outdated        = 59,
                cost_base_percent   = 0.16,
                flags               = 0,
                school              = magic_school.shadow,
                coef                = 0.0,
                over_time_coef      = 0.4,
				lvl_scaling			= 0.0,
            },
            [34916] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 120,
                rank                = 2,
                lvl_req             = 60,
                lvl_max             = 69,
                lvl_outdated        = 69,
				lvl_scaling			= 0.0,
            },
            [34917] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 130,
                rank                = 3,
                lvl_req             = 70,
                lvl_max             = 74,
                lvl_outdated        = 74,
				lvl_scaling			= 0.0,
            },
            [48159] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 147,
                rank                = 4,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 0.0,
            },
            [48160] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 170,
                rank                = 5,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            -- mind sear
            [48045] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 183,
                over_time_max       = 197,
                over_time_tick_freq = 1,
                over_time_duration  = 5.0,
                cast_time           = 5.0,
                rank                = 1,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 79,
                cost_base_percent   = 0.28,
                flags               = bit.bor(spell_flags.aoe, spell_flags.over_time_crit, spell_flags.over_time_range),
                school              = magic_school.shadow,
                coef                = 0.0,
                over_time_coef      = 0.266,
				lvl_scaling			= 0.0,
            },
            [53023] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 212,
                over_time_max       = 228,
                rank                = 2,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            -- shadow word: death
            [32379] = {
                base_min            = 450,
                base_max            = 522,
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 62,
                lvl_max             = 69,
                lvl_outdated        = 69,
                cost_base_percent   = 0.12,
                flags               = spell_flags.cd,
                school              = magic_school.shadow,
                coef                = 0.4286,
                over_time_coef      = 0.0,
				lvl_scaling			= 0.0,
            },
            [32996] = {
                base_min            = 572,
                base_max            = 664,
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 70,
                lvl_max             = 74,
                lvl_outdated        = 74,
				lvl_scaling			= 0.0,
            },
            [48157] = {
                base_min            = 639,
                base_max            = 741,
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 0.0,
            },
            [48158] = {
                base_min            = 750,
                base_max            = 870,
                over_time           = 0.0,
                rank                = 4,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            -- shadowfiend
             -- TODO: shadow fiend does some base damage, unclear if/how it scales with lvl
            [34433] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 170,
                over_time_max       = 230,
                over_time_tick_freq = 1.5,
                over_time_duration  = 15.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 66,
                lvl_max             = 0,
                lvl_outdated        = 80,
                cost_base_percent   = 0.0001,
                flags               = bit.bor(spell_flags.cd, spell_flags.over_time_range),
                school              = magic_school.shadow,
                coef                = 0.0,
                over_time_coef      = 0.3568,
				lvl_scaling			= 0.0,
            },
            -- lightwell
            [724] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 267,
                over_time_tick_freq = 2.0,
                over_time_duration  = 6.0,
                cast_time           = 0.5,
                rank                = 1,
                lvl_req             = 40,
                lvl_max             = 0,
                lvl_outdated        = 49,
                cost_base_percent   = 0.17,
                flags               = bit.bor(spell_flags.heal, spell_flags.aoe, spell_flags.cd),
                school              = magic_school.holy,
                coef                = 0.0,
                over_time_coef      = 0.626,
				lvl_scaling			= 0.0,
            },
            [27870] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 388,
                rank                = 2,
                lvl_req             = 50,
                lvl_max             = 0,
                lvl_outdated        = 59,
				lvl_scaling			= 0.0,
            },
            [28275] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 787,
                rank                = 3,
                lvl_req             = 60,
                lvl_max             = 0,
                lvl_outdated        = 69,
				lvl_scaling			= 0.0,
            },
            [48086] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 1305,
                rank                = 4,
                lvl_req             = 70,
                lvl_max             = 0,
                lvl_outdated        = 79,
				lvl_scaling			= 0.0,
            },
            [48087] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 1540,
                rank                = 5,
                lvl_req             = 80,
                lvl_max             = 0,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            -- divine hymn
            [64843] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 3024,
                over_time_max       = 3342,
                over_time_tick_freq = 2.0,
                over_time_duration  = 8.0,
                cast_time           = 8.0,
                rank                = 1,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
                cost_base_percent   = 0.63,
                flags               = bit.bor(spell_flags.heal, spell_flags.aoe, spell_flags.over_time_crit, spell_flags.over_time_range, spell_flags.cd),
                school              = magic_school.holy,
                coef                = 0.0,
                over_time_coef      = 0.564,
				lvl_scaling			= 0.0,
            },
        }; 
    elseif class == "SHAMAN" then
        return {
            -- healing stream totem
            [5394] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 6,
                over_time_tick_freq = 2,
                over_time_duration  = 60.0 * 5,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 20,
                lvl_max             = 0,
                lvl_outdated        = 29,
                cost_base_percent   = 0.03,
                flags               = bit.bor(spell_flags.heal, spell_flags.aoe),
                school              = magic_school.nature,
                coef                = 0.0,
                -- estimate on bad sp sample, what is the actual coef?
                over_time_coef      = 0.08275,
				lvl_scaling			= 0.0,
            },
            [6375] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 8,
                rank                = 2,
                lvl_req             = 30,
                lvl_max             = 0,
                lvl_outdated        = 39,
				lvl_scaling			= 0.0,
            },
            [6377] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 10,
                rank                = 3,
                lvl_req             = 40,
                lvl_max             = 0,
                lvl_outdated        = 49,
				lvl_scaling			= 0.0,
            },
            [10462] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 12,
                rank                = 4,
                lvl_req             = 50,
                lvl_max             = 0,
                lvl_outdated        = 59,
				lvl_scaling			= 0.0,
            },
            [10463] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 14,
                rank                = 5,
                lvl_req             = 60,
                lvl_max             = 0,
                lvl_outdated        = 68,
				lvl_scaling			= 0.0,
            },
            [25567] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 18,
                rank                = 6,
                lvl_req             = 69,
                lvl_max             = 0,
                lvl_outdated        = 70,
				lvl_scaling			= 0.0,
            },
            [58755] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 20,
                rank                = 7,
                lvl_req             = 71,
                lvl_max             = 0,
                lvl_outdated        = 75,
				lvl_scaling			= 0.0,
            },
            [58756] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 23,
                rank                = 8,
                lvl_req             = 76,
                lvl_max             = 0,
                lvl_outdated        = 79,
				lvl_scaling			= 0.0,
            },
            [58757] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 25,
                rank                = 9,
                lvl_req             = 80,
                lvl_max             = 0,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            -- lesser healing wave
            [8004] = {
                base_min            = 162,
                base_max            = 186,
                over_time           = 0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 20,
                lvl_max             = 25,
                lvl_outdated        = 27,
                cost_base_percent   = 0.15,
                flags               = spell_flags.heal,
                school              = magic_school.nature,
                coef                = 0.8057,
                over_time_coef      = 0.0,
				lvl_scaling			= 1.7,
            },
            [8008] = {
                base_min            = 247,
                base_max            = 281,
                over_time           = 0,
                rank                = 2,
                lvl_req             = 28,
                lvl_max             = 33,
                lvl_outdated        = 35,
				lvl_scaling			= 2.1,
            },
            [8010] = {
                base_min            = 337,
                base_max            = 381,
                over_time           = 0,
                rank                = 3,
                lvl_req             = 36,
                lvl_max             = 41,
                lvl_outdated        = 43,
				lvl_scaling			= 2.5,
            },
            [10466] = {
                base_min            = 458,
                base_max            = 514,
                over_time           = 0,
                rank                = 4,
                lvl_req             = 44,
                lvl_max             = 49,
                lvl_outdated        = 51,
				lvl_scaling			= 3,
            },
            [10467] = {
                base_min            = 631,
                base_max            = 705,
                over_time           = 0,
                rank                = 5,
                lvl_req             = 52,
                lvl_max             = 57,
                lvl_outdated        = 59,
				lvl_scaling			= 3.6,
            },
            [10468] = {
                base_min            = 832,
                base_max            = 928,
                over_time           = 0,
                rank                = 6,
                lvl_req             = 60,
                lvl_max             = 65,
                lvl_outdated        = 65,
				lvl_scaling			= 4.2,
            },
            [25420] = {
                base_min            = 1039,
                base_max            = 1185,
                over_time           = 0,
                rank                = 7,
                lvl_req             = 66,
                lvl_max             = 70,
                lvl_outdated        = 71,
				lvl_scaling			= 4.2,
            },
            [49275] = {
                base_min            = 1382,
                base_max            = 1578,
                over_time           = 0,
                rank                = 8,
                lvl_req             = 72,
                lvl_max             = 76,
                lvl_outdated        = 80,
				lvl_scaling			= 5,
            },
            [49276] = {
                base_min            = 1606,
                base_max            = 1834,
                over_time           = 0,
                rank                = 9,
                lvl_req             = 77,
                lvl_max             = 80,
                lvl_outdated        = 81,
				lvl_scaling			= 6,
            },
            -- healing wave
            [331] = {
                base_min            = 34,
                base_max            = 44,
                over_time           = 0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 3.0,
                rank                = 1,
                lvl_req             = 1,
                lvl_max             = 5,
                lvl_outdated        = 5,
                cost_base_percent   = 0.25,
                flags               = spell_flags.heal,
                school              = magic_school.nature,
                coef                = 1.6114,
                over_time_coef      = 0.0,
				lvl_scaling			= 0.7,
            },
            [332] = {
                base_min            = 64,
                base_max            = 78,
                over_time           = 0,
                rank                = 2,
                lvl_req             = 6,
                lvl_max             = 11,
                lvl_outdated        = 11,
				lvl_scaling			= 1,
            },
            [547] = {
                base_min            = 129,
                base_max            = 155,
                over_time           = 0,
                rank                = 3,
                lvl_req             = 12,
                lvl_max             = 17,
                lvl_outdated        = 17,
				lvl_scaling			= 1.5,
            },
            [913] = {
                base_min            = 268,
                base_max            = 316,
                over_time           = 0,
                rank                = 4,
                lvl_req             = 18,
                lvl_max             = 23,
                lvl_outdated        = 23,
				lvl_scaling			= 2.3,
            },
            [939] = {
                base_min            = 376,
                base_max            = 440,
                over_time           = 0,
                rank                = 5,
                lvl_req             = 24,
                lvl_max             = 29,
                lvl_outdated        = 31,
				lvl_scaling			= 2.7,
            },
            [959] = {
                base_min            = 536,
                base_max            = 622,
                over_time           = 0,
                rank                = 6,
                lvl_req             = 32,
                lvl_max             = 37,
                lvl_outdated        = 39,
				lvl_scaling			= 3.3,
            },
            [8005] = {
                base_min            = 740,
                base_max            = 854,
                over_time           = 0,
                rank                = 7,
                lvl_req             = 40,
                lvl_max             = 45,
                lvl_outdated        = 47,
				lvl_scaling			= 3.9,
            },
            [10395] = {
                base_min            = 1017,
                base_max            = 1167,
                over_time           = 0,
                rank                = 8,
                lvl_req             = 48,
                lvl_max             = 53,
                lvl_outdated        = 55,
				lvl_scaling			= 4.7,
            },
            [10396] = {
                base_min            = 1367,
                base_max            = 1561,
                over_time           = 0,
                rank                = 9,
                lvl_req             = 56,
                lvl_max             = 61,
                lvl_outdated        = 59,
				lvl_scaling			= 5.5,
            },
            [25357] = {
                base_min            = 1620,
                base_max            = 1850,
                over_time           = 0,
                rank                = 10,
                lvl_req             = 60,
                lvl_max             = 65,
                lvl_outdated        = 62,
				lvl_scaling			= 5.5,
            },
            [25391] = {
                base_min            = 1725,
                base_max            = 1969,
                over_time           = 0,
                rank                = 11,
                lvl_req             = 63,
                lvl_max             = 68,
                lvl_outdated        = 69,
				lvl_scaling			= 6.3,
            },
            [25396] = {
                base_min            = 2134,
                base_max            = 2436,
                over_time           = 0,
                rank                = 12,
                lvl_req             = 70,
                lvl_max             = 74,
                lvl_outdated        = 74,
				lvl_scaling			= 7.1,
            },
            [49272] = {
                base_min            = 2624,
                base_max            = 2996,
                over_time           = 0,
                rank                = 13,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 8,
            },
            [49273] = {
                base_min            = 3034,
                base_max            = 3466,
                over_time           = 0,
                rank                = 14,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 9,
            },
            -- chain heal
            [1064] = {
                base_min            = 320,
                base_max            = 368,
                over_time           = 0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 2.5,
                rank                = 1,
                lvl_req             = 40,
                lvl_max             = 45,
                lvl_outdated        = 45,
                cost_base_percent   = 0.19,
                flags               = spell_flags.heal,
                school              = magic_school.nature,
                coef                = 1.3428,
                over_time_coef      = 0.0,
				lvl_scaling			= 2.5,
            },
            [10622] = {
                base_min            = 405,
                base_max            = 465,
                over_time           = 0,
                rank                = 2,
                lvl_req             = 46,
                lvl_max             = 51,
                lvl_outdated        = 53,
				lvl_scaling			= 2.8,
            },
            [10623] = {
                base_min            = 551,
                base_max            = 629,
                over_time           = 0,
                rank                = 3,
                lvl_req             = 54,
                lvl_max             = 59,
                lvl_outdated        = 60,
				lvl_scaling			= 3.3,
            },
            [25422] = {
                base_min            = 605,
                base_max            = 691,
                over_time           = 0,
                rank                = 4,
                lvl_req             = 61,
                lvl_max             = 66,
                lvl_outdated        = 67,
				lvl_scaling			= 3.8,
            },
            [25423] = {
                base_min            = 826,
                base_max            = 942,
                over_time           = 0,
                rank                = 5,
                lvl_req             = 68,
                lvl_max             = 73,
                lvl_outdated        = 73,
				lvl_scaling			= 3.8,
            },
            [55458] = {
                base_min            = 906,
                base_max            = 1034,
                over_time           = 0,
                rank                = 6,
                lvl_req             = 74,
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 3.8,
            },
            [55459] = {
                base_min            = 1055,
                base_max            = 1205,
                over_time           = 0,
                rank                = 7,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 3.8,
            },
            -- lightning bolt
            [403] = {
                base_min            = 13,
                base_max            = 15,
                over_time           = 0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 2.5,
                rank                = 1,
                lvl_req             = 1,
                lvl_max             = 6,
                lvl_outdated        = 7,
                cost_base_percent   = 0.1,
                flags               = 0,
                school              = magic_school.nature,
                coef                = 0.714,
                over_time_coef      = 0.0,
				lvl_scaling			= 0.4,
            },
            [529] = {
                base_min            = 26,
                base_max            = 30,
                over_time           = 0,
                rank                = 2,
                lvl_req             = 8,
                lvl_max             = 13,
                lvl_outdated        = 13,
				lvl_scaling			= 0.5,
            },
            [548] = {
                base_min            = 45,
                base_max            = 53,
                over_time           = 0,
                rank                = 3,
                lvl_req             = 14,
                lvl_max             = 19,
                lvl_outdated        = 19,
				lvl_scaling			= 0.7,
            },
            [915] = {
                base_min            = 83,
                base_max            = 95,
                over_time           = 0,
                rank                = 4,
                lvl_req             = 20,
                lvl_max             = 25,
                lvl_outdated        = 25,
				lvl_scaling			= 1,
            },
            [943] = {
                base_min            = 125,
                base_max            = 143,
                over_time           = 0,
                rank                = 5,
                lvl_req             = 26,
                lvl_max             = 31,
                lvl_outdated        = 31,
				lvl_scaling			= 1.2,
            },
            [6041] = {
                base_min            = 172,
                base_max            = 194,
                over_time           = 0,
                rank                = 6,
                lvl_req             = 32,
                lvl_max             = 37,
                lvl_outdated        = 37,
				lvl_scaling			= 1.5,
            },
            [10391] = {
                base_min            = 227,
                base_max            = 255,
                over_time           = 0,
                rank                = 7,
                lvl_req             = 38,
                lvl_max             = 43,
                lvl_outdated        = 43,
				lvl_scaling			= 1.7,
            },
            [10392] = {
                base_min            = 282,
                base_max            = 316,
                over_time           = 0,
                rank                = 8,
                lvl_req             = 44,
                lvl_max             = 49,
                lvl_outdated        = 49,
				lvl_scaling			= 1.9,
            },
            [15207] = {
                base_min            = 347,
                base_max            = 389,
                over_time           = 0,
                rank                = 3,
                lvl_req             = 50,
                lvl_max             = 55,
                lvl_outdated        = 55,
				lvl_scaling			= 2.1,
            },
            [15208] = {
                base_min            = 419,
                base_max            = 467,
                over_time           = 0,
                rank                = 10,
                lvl_req             = 56,
                lvl_max             = 61,
                lvl_outdated        = 61,
				lvl_scaling			= 2.4,
            },
            [25448] = {
                base_min            = 495,
                base_max            = 565,
                over_time           = 0,
                rank                = 11,
                lvl_req             = 62,
                lvl_max             = 66,
                lvl_outdated        = 66,
				lvl_scaling			= 2.6,
            },
            [25449] = {
                base_min            = 563,
                base_max            = 643,
                over_time           = 0,
                rank                = 12,
                lvl_req             = 67,
                lvl_max             = 71,
                lvl_outdated        = 72,
				lvl_scaling			= 2.8,
            },
            [49237] = {
                base_min            = 595,
                base_max            = 679,
                over_time           = 0,
                rank                = 13,
                lvl_req             = 73,
                lvl_max             = 77,
                lvl_outdated        = 78,
				lvl_scaling			= 3,
            },
            [49238] = {
                base_min            = 715,
                base_max            = 815,
                over_time           = 0,
                rank                = 14,
                lvl_req             = 79,
                lvl_max             = 83,
                lvl_outdated        = 80,
				lvl_scaling			= 4,
            },
            -- chain lightning
            [421] = {
                base_min            = 191,
                base_max            = 217,
                over_time           = 0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 2.0,
                rank                = 1,
                lvl_req             = 32,
                lvl_max             = 37,
                lvl_outdated        = 39,
                cost_base_percent   = 0.26,
                flags               = spell_flags.cd,
                school              = magic_school.nature,
                coef                = 0.5714,
                over_time_coef      = 0.0,
				lvl_scaling			= 1.9,
            },
            [930] = {
                base_min            = 277,
                base_max            = 311,
                over_time           = 0,
                rank                = 2,
                lvl_req             = 40,
                lvl_max             = 45,
                lvl_outdated        = 47,
				lvl_scaling			= 2.3,
            },
            [2860] = {
                base_min            = 378,
                base_max            = 424,
                over_time           = 0,
                rank                = 3,
                lvl_req             = 48,
                lvl_max             = 53,
                lvl_outdated        = 55,
				lvl_scaling			= 2.7,
            },
            [10605] = {
                base_min            = 493,
                base_max            = 551,
                over_time           = 0,
                rank                = 4,
                lvl_req             = 56,
                lvl_max             = 61,
                lvl_outdated        = 62,
				lvl_scaling			= 3.1,
            },
            [25439] = {
                base_min            = 603,
                base_max            = 687,
                over_time           = 0,
                rank                = 5,
                lvl_req             = 63,
                lvl_max             = 68,
                lvl_outdated        = 69,
				lvl_scaling			= 3.5,
            },
            [25442] = {
                base_min            = 734,
                base_max            = 838,
                over_time           = 0,
                rank                = 6,
                lvl_req             = 70,
                lvl_max             = 74,
                lvl_outdated        = 73,
				lvl_scaling			= 3.9,
            },
            [49270] = {
                base_min            = 806,
                base_max            = 920,
                over_time           = 0,
                rank                = 7,
                lvl_req             = 74,
                lvl_max             = 78,
                lvl_outdated        = 79,
				lvl_scaling			= 4,
            },
            [49271] = {
                base_min            = 973,
                base_max            = 1111,
                over_time           = 0,
                rank                = 8,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 5,
            },
            -- lightning shield
            [324] = {
                base_min            = 13.0,
                base_max            = 13.0, 
                over_time           = 0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 8,
                lvl_max             = 0,
                lvl_outdated        = 15,
                cost_base_percent   = 0,
                flags               = spell_flags.cd,
                school              = magic_school.nature,
                coef                = 0.33,
                over_time_coef      = 0.0,
				lvl_scaling			= 0.0,
            },
            [325] = {
                base_min            = 29.0,
                base_max            = 29.0, 
                over_time           = 0,
                rank                = 2,
                lvl_req             = 16,
                lvl_max             = 0,
                lvl_outdated        = 23,
				lvl_scaling			= 0.0,
            },
            [905] = {
                base_min            = 51.0,
                base_max            = 51.0, 
                over_time           = 0,
                rank                = 3,
                lvl_req             = 24,
                lvl_max             = 0,
                lvl_outdated        = 31,
				lvl_scaling			= 0.0,
            },
            [945] = {
                base_min            = 80.0,
                base_max            = 80.0, 
                over_time           = 0,
                rank                = 4,
                lvl_req             = 32,
                lvl_max             = 0,
                lvl_outdated        = 39,
				lvl_scaling			= 0.0,
            },
            [8134] = {
                base_min            = 114.0,
                base_max            = 114.0, 
                over_time           = 0,
                rank                = 5,
                lvl_req             = 40,
                lvl_max             = 0,
                lvl_outdated        = 47,
				lvl_scaling			= 0.0,
            },
            [10431] = {
                base_min            = 154.0,
                base_max            = 154.0, 
                over_time           = 0,
                rank                = 6,
                lvl_req             = 48,
                lvl_max             = 0,
                lvl_outdated        = 55,
				lvl_scaling			= 0.0,
            },
            [10432] = {
                base_min            = 198.0,
                base_max            = 198.0, 
                over_time           = 0,
                rank                = 7,
                lvl_req             = 56,
                lvl_max             = 0,
                lvl_outdated        = 62,
				lvl_scaling			= 0.0,
            },
            [25469] = {
                base_min            = 232.0,
                base_max            = 232.0, 
                over_time           = 0,
                rank                = 8,
                lvl_req             = 63,
                lvl_max             = 68,
                lvl_outdated        = 69,
				lvl_scaling			= 0.0,
            },
            [25472] = {
                base_min            = 287.0,
                base_max            = 287.0, 
                over_time           = 0,
                rank                = 9,
                lvl_req             = 70,
                lvl_max             = 74,
                lvl_outdated        = 74,
				lvl_scaling			= 0.0,
            },
            [49280] = {
                base_min            = 325.0,
                base_max            = 325.0, 
                over_time           = 0,
                rank                = 10,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 0.0,
            },
            [49281] = {
                base_min            = 380.0,
                base_max            = 380.0, 
                over_time           = 0,
                rank                = 11,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            -- earth shock
            [8042] = {
                base_min            = 17,
                base_max            = 19,
                over_time           = 0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 4,
                lvl_max             = 9,
                lvl_outdated        = 7,
                cost_base_percent   = 0.18,
                flags               = 0,
                school              = magic_school.nature,
                coef                = 0.3858,
                over_time_coef      = 0.0,
				lvl_scaling			= 0.5,
            },
            [8044] = {
                base_min            = 32,
                base_max            = 34,
                over_time           = 0,
                rank                = 2,
                lvl_req             = 8,
                lvl_max             = 13,
                lvl_outdated        = 13,
				lvl_scaling			= 0.7,
            },
            [8045] = {
                base_min            = 60,
                base_max            = 64,
                over_time           = 0,
                rank                = 3,
                lvl_req             = 14,
                lvl_max             = 19,
                lvl_outdated        = 23,
				lvl_scaling			= 1,
            },
            [8046] = {
                base_min            = 119,
                base_max            = 127,
                over_time           = 0,
                rank                = 4,
                lvl_req             = 24,
                lvl_max             = 29,
                lvl_outdated        = 35,
				lvl_scaling			= 1.4,
            },
            [10412] = {
                base_min            = 225,
                base_max            = 239,
                over_time           = 0,
                rank                = 5,
                lvl_req             = 36,
                lvl_max             = 41,
                lvl_outdated        = 47,
				lvl_scaling			= 2,
            },
            [10413] = {
                base_min            = 359,
                base_max            = 381,
                over_time           = 0,
                rank                = 6,
                lvl_req             = 48,
                lvl_max             = 53,
                lvl_outdated        = 59,
				lvl_scaling			= 2.6,
            },
            [10414] = {
                base_min            = 517,
                base_max            = 545,
                over_time           = 0,
                rank                = 7,
                lvl_req             = 60,
                lvl_max             = 65,
                lvl_outdated        = 68,
				lvl_scaling			= 3.1,
            },
            [25454] = {
                base_min            = 658,
                base_max            = 692,
                over_time           = 0,
                rank                = 8,
                lvl_req             = 69,
                lvl_max             = 73,
                lvl_outdated        = 73,
				lvl_scaling			= 3.6,
            },
            [49230] = {
                base_min            = 723,
                base_max            = 761,
                over_time           = 0,
                rank                = 9,
                lvl_req             = 74,
                lvl_max             = 78,
                lvl_outdated        = 78,
				lvl_scaling			= 4,
            },
            [49231] = {
                base_min            = 849,
                base_max            = 895,
                over_time           = 0,
                rank                = 10,
                lvl_req             = 79,
                lvl_max             = 83,
                lvl_outdated        = 80,
				lvl_scaling			= 5,
            },
            -- magma totem
            [8190] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 38,
                over_time_tick_freq = 2,
                over_time_duration  = 20.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 26,
                lvl_max             = 0,
                lvl_outdated        = 35,
                cost_base_percent   = 0.27,
                flags               = bit.bor(spell_flags.aoe, spell_flags.cd),
                school              = magic_school.fire,
                coef                = 0.0,
                over_time_coef      = 0.1,
				lvl_scaling			= 0.0,
            },
            [10585] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 64,
                rank                = 2,
                lvl_req             = 36,
                lvl_max             = 0,
                lvl_outdated        = 45,
				lvl_scaling			= 0.0,
            },
            [10586] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 94,
                rank                = 3,
                lvl_req             = 46,
                lvl_max             = 0,
                lvl_outdated        = 55,
				lvl_scaling			= 0.0,
            },
            [10587] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 131,
                rank                = 4,
                lvl_req             = 56,
                lvl_max             = 0,
                lvl_outdated        = 64,
				lvl_scaling			= 0.0,
            },
            [25552] = {
                base_min            = 0.0,
                base_max            = 0.0,
                over_time           = 180,
                rank                = 5,
                lvl_req             = 65,
                lvl_max             = 0,
                lvl_outdated        = 72,
				lvl_scaling			= 0.0,
            },
            [58731] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 314,
                rank                = 6,
                lvl_req             = 73,
                lvl_max             = 0,
                lvl_outdated        = 77,
				lvl_scaling			= 0.0,
            },
            [58734] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 371,
                rank                = 7,
                lvl_req             = 78,
                lvl_max             = 0,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            -- TODO: dot haste flag?
            -- flame shock
            [8050] = {
                base_min            = 21,
                base_max            = 21,
                over_time           = 7,
                over_time_tick_freq = 3,
                over_time_duration  = 18.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 10,
                lvl_max             = 15,
                lvl_outdated        = 17,
                cost_base_percent   = 0.17,
                flags               = bit.bor(spell_flags.over_time_crit, spell_flags.cd),
                school              = magic_school.fire,
                coef                = 0.2143,
                over_time_coef      = 0.1,
				lvl_scaling			= 0.8,
            },
            [8052] = {
                base_min            = 45,
                base_max            = 45,
                over_time           = 12,
                rank                = 2,
                lvl_req             = 18,
                lvl_max             = 23,
                lvl_outdated        = 27,
				lvl_scaling			= 1.2,
            },
            [8053] = {
                base_min            = 86,
                base_max            = 86,
                over_time           = 24,
                rank                = 3,
                lvl_req             = 28,
                lvl_max             = 33,
                lvl_outdated        = 39,
				lvl_scaling			= 1.7,
            },
            [10447] = {
                base_min            = 152,
                base_max            = 152,
                over_time           = 42,
                rank                = 4,
                lvl_req             = 40,
                lvl_max             = 45,
                lvl_outdated        = 51,
				lvl_scaling			= 2.3,
            },
            [10448] = {
                base_min            = 230,
                base_max            = 230,
                over_time           = 64,
                rank                = 5,
                lvl_req             = 52,
                lvl_max             = 57,
                lvl_outdated        = 59,
				lvl_scaling			= 2.9,
            },
            [29228] = {
                base_min            = 309,
                base_max            = 309,
                over_time           = 86,
                rank                = 6,
                lvl_req             = 60,
                lvl_max             = 67,
                lvl_outdated        = 69,
				lvl_scaling			= 3.5,
            },
            [25457] = {
                base_min            = 377,
                base_max            = 377,
                over_time           = 105,
                rank                = 7,
                lvl_req             = 70,
                lvl_max             = 74,
                lvl_outdated        = 74,
				lvl_scaling			= 3.8,
            },
            [49232] = {
                base_min            = 425,
                base_max            = 425,
                over_time           = 119,
                rank                = 8,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 4,
            },
            [49233] = {
                base_min            = 500,
                base_max            = 500,
                over_time           = 139,
                rank                = 9,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 5,
            },
            -- frost shock
            [8056] = {
                base_min            = 89,
                base_max            = 95,
                over_time           = 0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 20,
                lvl_max             = 25,
                lvl_outdated        = 33,
                cost_base_percent   = 0.18,
                flags               = bit.bor(spell_flags.snare, spell_flags.cd),
                school              = magic_school.frost,
                coef                = 0.3858,
                over_time_coef      = 0.0,
				lvl_scaling			= 1.2,
            },
            [8058] = {
                base_min            = 206,
                base_max            = 220,
                over_time           = 0,
                rank                = 2,
                lvl_req             = 34,
                lvl_max             = 39,
                lvl_outdated        = 45,
				lvl_scaling			= 1.9,
            },
            [10472] = {
                base_min            = 333,
                base_max            = 353,
                over_time           = 0,
                rank                = 3,
                lvl_req             = 46,
                lvl_max             = 51,
                lvl_outdated        = 57,
				lvl_scaling			= 2.5,
            },
            [10473] = {
                base_min            = 486,
                base_max            = 514,
                over_time           = 0,
                rank                = 4,
                lvl_req             = 58,
                lvl_max             = 63,
                lvl_outdated        = 67,
				lvl_scaling			= 3,
            },
            [25464] = {
                base_min            = 640,
                base_max            = 676,
                over_time           = 0,
                rank                = 5,
                lvl_req             = 68,
                lvl_max             = 72,
                lvl_outdated        = 72,
				lvl_scaling			= 3.5,
            },
            [49235] = {
                base_min            = 681,
                base_max            = 719,
                over_time           = 0,
                rank                = 6,
                lvl_req             = 73,
                lvl_max             = 77,
                lvl_outdated        = 77,
				lvl_scaling			= 4,
            },
            [49236] = {
                base_min            = 802,
                base_max            = 848,
                over_time           = 0,
                rank                = 7,
                lvl_req             = 78,
                lvl_max             = 82,
                lvl_outdated        = 80,
				lvl_scaling			= 5,
            },
            -- TODO freq ??
            -- searing totem
            [3599] = {
                base_min            = 0.0,
                base_max            = 0.0,
                over_time           = 9,
                over_time_max       = 11,
                over_time_tick_freq = 2.5,
                over_time_duration  = 60.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 10, 
                lvl_max             = 0,
                lvl_outdated        = 19, 
                cost_base_percent   = 0.07,
                flags               = bit.bor(spell_flags.over_time_crit, spell_flags.over_time_range),
                school              = magic_school.fire,
                coef                = 0.1667,
                over_time_coef      = 0.0,
				lvl_scaling			= 0.0,
            },
            [6363] = {
                base_min            = 0.0,
                base_max            = 0.0,
                over_time           = 13,
                over_time_max       = 17,
                rank                = 2,
                lvl_req             = 20, 
                lvl_max             = 0,
                lvl_outdated        = 29, 
				lvl_scaling			= 0.0,
            },
            [6364] = {
                base_min            = 0.0,
                base_max            = 0.0,
                over_time           = 19,
                over_time_max       = 25,
                rank                = 3,
                lvl_req             = 30, 
                lvl_max             = 0,
                lvl_outdated        = 39, 
				lvl_scaling			= 0.0,
            },
            [6365] = {
                base_min            = 0.0,
                base_max            = 0.0,
                over_time           = 26,
                over_time_max       = 34,
                rank                = 4,
                lvl_req             = 40, 
                lvl_max             = 0,
                lvl_outdated        = 49, 
				lvl_scaling			= 0.0,
            },
            [10437] = {
                base_min            = 0.0,
                base_max            = 0.0,
                over_time           = 33,
                over_time_max       = 45,
                rank                = 5,
                lvl_req             = 50, 
                lvl_max             = 0,
                lvl_outdated        = 59, 
				lvl_scaling			= 0.0,
            },
            [10438] = {
                base_min            = 0.0,
                base_max            = 0.0,
                over_time           = 40,
                over_time_max       = 54,
                rank                = 6,
                lvl_req             = 60, 
                lvl_max             = 0,
                lvl_outdated        = 68, 
				lvl_scaling			= 0.0,
            },
            [25533] = {
                base_min            = 0.0,
                base_max            = 0.0,
                over_time           = 56,
                over_time_max       = 74,
                rank                = 7,
                lvl_req             = 69, 
                lvl_max             = 0,
                lvl_outdated        = 70, 
				lvl_scaling			= 0.0,
            },
            [58699] = {
                base_min            = 0.0,
                base_max            = 0.0,
                over_time           = 68,
                over_time_max       = 92,
                rank                = 8,
                lvl_req             = 71, 
                lvl_max             = 0,
                lvl_outdated        = 74, 
				lvl_scaling			= 0.0,
            },
            [58703] = {
                base_min            = 0.0,
                base_max            = 0.0,
                over_time           = 77,
                over_time_max       = 10,
                rank                = 9,
                lvl_req             = 75, 
                lvl_max             = 0,
                lvl_outdated        = 79, 
				lvl_scaling			= 0.0,
            },
            [58704] = {
                base_min            = 0.0,
                base_max            = 0.0,
                over_time           = 90,
                over_time_max       = 120,
                rank                = 10,
                lvl_req             = 80, 
                lvl_max             = 0,
                lvl_outdated        = 80, 
				lvl_scaling			= 0.0,
            },
            -- fire nova totem
            [1535] = {
                base_min            = 48,
                base_max            = 56, 
                over_time           = 0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 12,
                lvl_max             = 0,
                lvl_outdated        = 21,
                cost_base_percent   = 0.22,
                flags               = bit.bor(spell_flags.aoe, spell_flags.cd),
                school              = magic_school.fire,
                coef                = 0.2142,
                over_time_coef      = 0.0,
				lvl_scaling			= 1.1,
            },
            [8498] = {
                base_min            = 102,
                base_max            = 116, 
                over_time           = 0,
                rank                = 2,
                lvl_req             = 22,
                lvl_max             = 0,
                lvl_outdated        = 31,
				lvl_scaling			= 1.6,
            },
            [8499] = {
                base_min            = 184,
                base_max            = 208, 
                over_time           = 0,
                rank                = 3,
                lvl_req             = 32,
                lvl_max             = 0,
                lvl_outdated        = 41,
				lvl_scaling			= 2.2,
            },
            [11314] = {
                base_min            = 281,
                base_max            = 317, 
                over_time           = 0,
                rank                = 4,
                lvl_req             = 42,
                lvl_max             = 0,
                lvl_outdated        = 51,
				lvl_scaling			= 2.8,
            },
            [11315] = {
                base_min            = 396,
                base_max            = 442, 
                over_time           = 0,
                rank                = 5,
                lvl_req             = 52,
                lvl_max             = 0,
                lvl_outdated        = 60,
				lvl_scaling			= 3.4,
            },
            [25546] = {
                base_min            = 518,
                base_max            = 578, 
                over_time           = 0,
                rank                = 6,
                lvl_req             = 61,
                lvl_max             = 0,
                lvl_outdated        = 69,
				lvl_scaling			= 3.9,
            },
            [25547] = {
                base_min            = 727,
                base_max            = 813, 
                over_time           = 0,
                rank                = 7,
                lvl_req             = 70,
                lvl_max             = 0,
                lvl_outdated        = 74,
				lvl_scaling			= 4.5,
            },
            [61649] = {
                base_min            = 755,
                base_max            = 843, 
                over_time           = 0,
                rank                = 8,
                lvl_req             = 75,
                lvl_max             = 0,
                lvl_outdated        = 79,
				lvl_scaling			= 4.5,
            },
            [61657] = {
                base_min            = 893,
                base_max            = 997, 
                over_time           = 0,
                rank                = 9,
                lvl_req             = 80,
                lvl_max             = 0,
                lvl_outdated        = 80,
				lvl_scaling			= 4.5,
            },
            -- lava burst
            [51505] = {
                base_min            = 1012,
                base_max            = 1290,
                over_time           = 0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 2.0,
                rank                = 1,
                lvl_req             = 75,
                lvl_max             = 80,
                lvl_outdated        = 79,
                cost_base_percent   = 0.1,
                flags               = spell_flags.cd,
                school              = magic_school.fire,
                coef                = 0.5714,
                over_time_coef      = 0.0,
				lvl_scaling			= 5.2,
            },
            [60043] = {
                base_min            = 1192,
                base_max            = 1518,
                over_time           = 0,
                rank                = 2,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 6.5,
            },
            -- riptide
            -- TODO: chain heal does extra on riptide
            [61295] = {
                base_min            = 639,
                base_max            = 691,
                over_time           = 133,
                over_time_tick_freq = 3,
                over_time_duration  = 15.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 60,
                lvl_max             = 69,
                lvl_outdated        = 69,
                cost_base_percent   = 0.18,
                flags               = bit.bor(spell_flags.heal, spell_flags.cd),
                school              = magic_school.nature,
                coef                = 0.402,
                over_time_coef      = 0.188,
				lvl_scaling			= 0.0,
            },
            [61299] = {
                base_min            = 849,
                base_max            = 919,
                over_time           = 177,
                rank                = 2,
                lvl_req             = 70,
                lvl_max             = 74,
                lvl_outdated        = 74,
				lvl_scaling			= 0.0,
            },
            [61300] = {
                base_min            = 1378,
                base_max            = 1492,
                over_time           = 287,
                rank                = 3,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 0.0,
            },
            [61301] = {
                base_min            = 1604,
                base_max            = 1736,
                over_time           = 334,
                rank                = 4,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            -- earth shield
            [974] = {
                base_min            = 150.0,
                base_max            = 150.0, 
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 50,
                lvl_max             = 59,
                lvl_outdated        = 59,
                cost_base_percent   = 0.15,
                flags               = bit.bor(spell_flags.heal, spell_flags.cd),
                school              = magic_school.nature,
                --coef                = 0.1343,
                coef                = 0.4762,
                over_time_coef      = 0.0,
				lvl_scaling			= 0.0,
            },
            [32593] = {
                base_min            = 205.0,
                base_max            = 205.0, 
                over_time           = 0,
                rank                = 2,
                lvl_req             = 60,
                lvl_max             = 69,
                lvl_outdated        = 69,
				lvl_scaling			= 0.0,
            },
            [32594] = {
                base_min            = 270.0,
                base_max            = 270.0, 
                over_time           = 0,
                rank                = 3,
                lvl_req             = 70,
                lvl_max             = 74,
                lvl_outdated        = 74,
				lvl_scaling			= 0.0,
            },
            [49283] = {
                base_min            = 300.0,
                base_max            = 300.0, 
                over_time           = 0,
                rank                = 4,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 0.0,
            },
            [49284] = {
                base_min            = 337.0,
                base_max            = 337.0, 
                over_time           = 0,
                rank                = 5,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            -- thunderstorm
            [51490] = {
                base_min            = 551,
                base_max            = 629,
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 60,
                lvl_max             = 65,
                lvl_outdated        = 69,
                cost_base_percent   = 0.0,
                flags               = spell_flags.cd,
                school              = magic_school.nature,
                coef                = 0.172,
                over_time_coef      = 0.0,
				lvl_scaling			= 3,
            },
            [59156] = {
                base_min            = 1074,
                base_max            = 1226,
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 70,
                lvl_max             = 75,
                lvl_outdated        = 74,
				lvl_scaling			= 3,
            },
            [59158] = {
                base_min            = 1226,
                base_max            = 1400,
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 75,
                lvl_max             = 80,
                lvl_outdated        = 79,
				lvl_scaling			= 3,
            },
            [59159] = {
                base_min            = 1450,
                base_max            = 1656,
                over_time           = 0.0,
                rank                = 4,
                lvl_req             = 80,
                lvl_max             = 85,
                lvl_outdated        = 80,
				lvl_scaling			= 3,
            },

        };
    elseif class == "PALADIN" then
        return {
            -- flash of light
            [19750] = {
                base_min            = 81,
                base_max            = 93,
                over_time           = 0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 20,
                lvl_max             = 25,
                lvl_outdated        = 25,
                cost_base_percent   = 0.07,
                flags               = spell_flags.heal,
                school              = magic_school.holy,
                coef                = 1.009,
                over_time_coef      = 0,
				lvl_scaling			= 1,
            },
            [19939] = {
                base_min            = 124,
                base_max            = 144,
                over_time           = 0,
                rank                = 2,
                lvl_req             = 26,
                lvl_max             = 31,
                lvl_outdated        = 33,
				lvl_scaling			= 1.3,
            },
            [19940] = {
                base_min            = 189,
                base_max            = 211,
                over_time           = 0,
                rank                = 3,
                lvl_req             = 34,
                lvl_max             = 39,
                lvl_outdated        = 41,
				lvl_scaling			= 1.6,
            },
            [19941] = {
                base_min            = 256,
                base_max            = 288,
                over_time           = 0,
                rank                = 4,
                lvl_req             = 42,
                lvl_max             = 47,
                lvl_outdated        = 49,
				lvl_scaling			= 1.9,
            },
            [19942] = {
                base_min            = 346,
                base_max            = 390,
                over_time           = 0,
                rank                = 5,
                lvl_req             = 50,
                lvl_max             = 55,
                lvl_outdated        = 57,
				lvl_scaling			= 2.2,
            },
            [19943] = {
                base_min            = 445,
                base_max            = 499,
                over_time           = 0,
                rank                = 6,
                lvl_req             = 58,
                lvl_max             = 63,
                lvl_outdated        = 65,
				lvl_scaling			= 2.6,
            },
            [27137] = {
                base_min            = 588,
                base_max            = 658,
                over_time           = 0,
                rank                = 7,
                lvl_req             = 66,
                lvl_max             = 70,
                lvl_outdated        = 73,
				lvl_scaling			= 3,
            },
            [48784] = {
                base_min            = 682,
                base_max            = 764,
                over_time           = 0,
                rank                = 8,
                lvl_req             = 74,
                lvl_max             = 78,
                lvl_outdated        = 78,
				lvl_scaling			= 3.4,
            },
            [48785] = {
                base_min            = 785,
                base_max            = 879,
                over_time           = 0,
                rank                = 9,
                lvl_req             = 79,
                lvl_max             = 83,
                lvl_outdated        = 80,
				lvl_scaling			= 3.9,
            },
            -- holy light
            [635] = {
                base_min            = 50,
                base_max            = 60,
                over_time           = 0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 2.5,
                rank                = 1,
                lvl_req             = 1,
                lvl_max             = 5,
                lvl_outdated        = 5,
                cost_base_percent   = 0.29,
                flags               = spell_flags.heal,
                school              = magic_school.holy,
                coef                = 1.679,
                over_time_coef      = 0,
				lvl_scaling			= 0.8,
            },
            [639] = {
                base_min            = 96,
                base_max            = 116,
                over_time           = 0,
                rank                = 2,
                lvl_req             = 6,
                lvl_max             = 11,
                lvl_outdated        = 13,
				lvl_scaling			= 1.1,
            },
            [647] = {
                base_min            = 203,
                base_max            = 239,
                over_time           = 0,
                rank                = 3,
                lvl_req             = 14,
                lvl_max             = 19,
                lvl_outdated        = 21,
				lvl_scaling			= 1.7,
            },
            [1026] = {
                base_min            = 397,
                base_max            = 455,
                over_time           = 0,
                rank                = 4,
                lvl_req             = 22,
                lvl_max             = 27,
                lvl_outdated        = 29,
				lvl_scaling			= 2.4,
            },
            [1042] = {
                base_min            = 628,
                base_max            = 708,
                over_time           = 0,
                rank                = 5,
                lvl_req             = 30,
                lvl_max             = 35,
                lvl_outdated        = 37,
				lvl_scaling			= 3.1,
            },
            [3472] = {
                base_min            = 894,
                base_max            = 998,
                over_time           = 0,
                rank                = 6,
                lvl_req             = 38,
                lvl_max             = 43,
                lvl_outdated        = 45,
				lvl_scaling			= 3.8,
            },
            [10328] = {
                base_min            = 1209,
                base_max            = 1349,
                over_time           = 0,
                rank                = 7,
                lvl_req             = 46,
                lvl_max             = 51,
                lvl_outdated        = 53,
				lvl_scaling			= 4.6,
            },
            [10329] = {
                base_min            = 1595,
                base_max            = 1777,
                over_time           = 0,
                rank                = 8,
                lvl_req             = 54,
                lvl_max             = 59,
                lvl_outdated        = 59,
				lvl_scaling			= 5.2,
            },
            [25292] = {
                base_min            = 2034,
                base_max            = 2266,
                over_time           = 0,
                rank                = 9,
                lvl_req             = 60,
                lvl_max             = 65,
                lvl_outdated        = 61,
				lvl_scaling			= 5.8,
            },
            [27135] = {
                base_min            = 2232,
                base_max            = 2486,
                over_time           = 0,
                rank                = 10,
                lvl_req             = 62,
                lvl_max             = 67,
                lvl_outdated        = 69,
				lvl_scaling			= 6.4,
            },
            [27136] = {
                base_min            = 2818,
                base_max            = 3138,
                over_time           = 0,
                rank                = 10,
                lvl_req             = 70,
                lvl_max             = 74,
                lvl_outdated        = 74,
				lvl_scaling			= 7,
            },
            [48781] = {
                base_min            = 4199,
                base_max            = 4677,
                over_time           = 0,
                rank                = 11,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 10.4,
            },
            [48782] = {
                base_min            = 4888,
                base_max            = 5444,
                over_time           = 0,
                rank                = 12,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 12.1,
            },
            -- holy shock
            [20473] = {
                base_min            = 314.0,
                base_max            = 340.0, 
                over_time           = 0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 40,
                lvl_max             = 0,
                lvl_outdated        = 47,
                cost_base_percent   = 0.18,
                flags               = 0,
                school              = magic_school.holy,
                coef                = 0.4286,
                over_time_coef      = 0,
				lvl_scaling			= 0.0,
                healing_version = {
                    base_min            = 481.0,
                    base_max            = 519.0, 
                    over_time           = 0,
                    over_time_tick_freq = 0,
                    over_time_duration  = 0.0,
                    cast_time           = 1.5,
                    rank                = 1,
                    lvl_req             = 40,
                    lvl_max             = 0,
                    lvl_outdated        = 47,
                    cost_base_percent   = 0.18,
                    flags               = spell_flags.heal,
                    school              = magic_school.holy,
                    coef                = 0.807,
                    over_time_coef      = 0,
				    lvl_scaling			= 0.0,
                }
            },
            [20929] = {
                base_min            = 431.0,
                base_max            = 465.0, 
                over_time           = 0,
                rank                = 2,
                lvl_req             = 48,
                lvl_max             = 0,
                lvl_outdated        = 55,
				lvl_scaling			= 0.0,
                healing_version = {
                    base_min            = 644.0,
                    base_max            = 696.0, 
                    over_time           = 0,
                    rank                = 1,
                    lvl_req             = 48,
                    lvl_max             = 0,
                    lvl_outdated        = 55,
				    lvl_scaling			= 0.0,
                }
            },
            [20930] = {
                base_min            = 562.0,
                base_max            = 608.0, 
                over_time           = 0,
                rank                = 3,
                lvl_req             = 56,
                lvl_max             = 0,
                lvl_outdated        = 63,
				lvl_scaling			= 0.0,
                healing_version = {
                    base_min            = 845.0,
                    base_max            = 915.0, 
                    over_time           = 0,
                    rank                = 3,
                    lvl_req             = 56,
                    lvl_max             = 0,
                    lvl_outdated        = 63,
				    lvl_scaling			= 0.0,
                }
            }, 
            [27174] = {
                base_min            = 693.0,
                base_max            = 749.0, 
                over_time           = 0,
                rank                = 4,
                lvl_req             = 64,
                lvl_max             = 0,
                lvl_outdated        = 69,
				lvl_scaling			= 0.0,
                healing_version = {
                    base_min            = 1061.0,
                    base_max            = 1149.0, 
                    over_time           = 0,
                    rank                = 4,
                    lvl_req             = 64,
                    lvl_max             = 0,
                    lvl_outdated        = 69,
				    lvl_scaling			= 0.0,
                }
            }, 
            [33072] = {
                base_min            = 904.0,
                base_max            = 978.0, 
                over_time           = 0,
                rank                = 5,
                lvl_req             = 70,
                lvl_max             = 0,
                lvl_outdated        = 74,
				lvl_scaling			= 0.0,
                healing_version = {
                    base_min            = 1258.0,
                    base_max            = 1362.0, 
                    over_time           = 0,
                    rank                = 5,
                    lvl_req             = 70,
                    lvl_max             = 0,
                    lvl_outdated        = 74,
				    lvl_scaling			= 0.0,
                }
            }, 
            [48824] = {
                base_min            = 1043.0,
                base_max            = 1129.0, 
                over_time           = 0,
                rank                = 6,
                lvl_req             = 75,
                lvl_max             = 0,
                lvl_outdated        = 79,
				lvl_scaling			= 0.0,
                healing_version = {
                    base_min            = 2065.0,
                    base_max            = 2235.0, 
                    over_time           = 0,
                    rank                = 6,
                    lvl_req             = 75,
                    lvl_max             = 0,
                    lvl_outdated        = 79,
				    lvl_scaling			= 0.0,
                }
            }, 
            [48825] = {
                base_min            = 1296.0,
                base_max            = 1402.0, 
                over_time           = 0,
                rank                = 7,
                lvl_req             = 80,
                lvl_max             = 0,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
                healing_version = {
                    base_min            = 2401.0,
                    base_max            = 2599.0, 
                    over_time           = 0,
                    rank                = 7,
                    lvl_req             = 80,
                    lvl_max             = 0,
                    lvl_outdated        = 80,
				    lvl_scaling			= 0.0,
                }
            }, 
            -- sacred shield
            [53601] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 500,
                over_time_tick_freq = 7.5,
                over_time_duration  = 30.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 80,
                lvl_max             = 80,
                lvl_outdated        = 80,
                cost_base_percent   = 0.12,
                flags               = spell_flags.absorb,
                school              = magic_school.holy,
                coef                = 0.0,
                over_time_coef      = 0.429,
				lvl_scaling			= 0.0,
            },
        };

    elseif class == "WARLOCK" then
        return {
            -- curse of agony
            [980] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 7,
                over_time_tick_freq = 2,
                over_time_duration  = 24.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 8,
                lvl_max             = 17,
                lvl_outdated        = 17,
                cost_base_percent   = 0.1,
                flags               = spell_flags.curse,
                school              = magic_school.shadow,
                coef                = 0.0,
                over_time_coef      = 0.1,
				lvl_scaling			= 0.0,
            },
            [1014] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 15,
                rank                = 2,
                lvl_req             = 18,
                lvl_max             = 27,
                lvl_outdated        = 27,
				lvl_scaling			= 0.0,
            },
            [6217] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 27,
                rank                = 3,
                lvl_req             = 28,
                lvl_max             = 37,
                lvl_outdated        = 37,
				lvl_scaling			= 0.0,
            },
            [11711] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 42,
                rank                = 4,
                lvl_req             = 38,
                lvl_max             = 47,
                lvl_outdated        = 47,
				lvl_scaling			= 0.0,
            },
            [11712] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 65,
                rank                = 5,
                lvl_req             = 48,
                lvl_max             = 57,
                lvl_outdated        = 57,
				lvl_scaling			= 0.0,
            },
            [11713] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 87,
                rank                = 6,
                lvl_req             = 58,
                lvl_max             = 66,
                lvl_outdated        = 66,
				lvl_scaling			= 0.0,
            },
            [27218] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 113,
                rank                = 7,
                lvl_req             = 67,
                lvl_max             = 72,
                lvl_outdated        = 72,
				lvl_scaling			= 0.0,
            },
            [47863] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 120,
                rank                = 8,
                lvl_req             = 73,
                lvl_max             = 78,
                lvl_outdated        = 78,
				lvl_scaling			= 0.0,
            },
            [47864] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 145,
                rank                = 9,
                lvl_req             = 79,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            -- death coil
            [6789] = {
                base_min            = 244,
                base_max            = 244, 
                over_time           = 0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 42,
                lvl_max             = 48,
                lvl_outdated        = 49,
                cost_base_percent   = 0.23,
                flags               = bit.bor(spell_flags.snare, spell_flags.cd),
                school              = magic_school.shadow,
                coef                = 0.2143,
                over_time_coef      = 0.0,
				lvl_scaling			= 2.2,
            },
            [17925] = {
                base_min            = 319,
                base_max            = 319, 
                over_time           = 0,
                rank                = 2,
                lvl_req             = 50,
                lvl_max             = 56,
                lvl_outdated        = 57,
				lvl_scaling			= 2.6,
            },
            [17926] = {
                base_min            = 400,
                base_max            = 400, 
                over_time           = 0,
                rank                = 3,
                lvl_req             = 58,
                lvl_max             = 64,
                lvl_outdated        = 67,
				lvl_scaling			= 3,
            },
            [27223] = {
                base_min            = 519,
                base_max            = 519, 
                over_time           = 0,
                rank                = 4,
                lvl_req             = 68,
                lvl_max             = 74,
                lvl_outdated        = 72,
				lvl_scaling			= 3.4,
            },
            [47859] = {
                base_min            = 670,
                base_max            = 670, 
                over_time           = 0,
                rank                = 5,
                lvl_req             = 73,
                lvl_max             = 77,
                lvl_outdated        = 77,
				lvl_scaling			= 4,
            },
            [47860] = {
                base_min            = 790,
                base_max            = 790, 
                over_time           = 0,
                rank                = 6,
                lvl_req             = 78,
                lvl_max             = 82,
                lvl_outdated        = 80,
				lvl_scaling			= 5,
            },
            -- corruption
            [172] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 10,
                over_time_tick_freq = 3,
                over_time_duration  = 18.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 4,
                lvl_max             = 9,
                lvl_outdated        = 13,
                cost_base_percent   = 35,
                flags               = 0,
                school              = magic_school.shadow,
                coef                = 0.0,
                over_time_coef      = 0.2,
				lvl_scaling			= 0.0,
            },
            [6222] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 18,
                rank                = 2,
                lvl_req             = 14,
                lvl_max             = 19,
                lvl_outdated        = 23,
				lvl_scaling			= 0.0,
            },
            [6223] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 37,
                rank                = 3,
                lvl_req             = 24,
                lvl_max             = 29,
                lvl_outdated        = 33,
				lvl_scaling			= 0.0,
            },
            [7648] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 54,
                rank                = 4,
                lvl_req             = 34,
                lvl_max             = 39,
                lvl_outdated        = 43,
				lvl_scaling			= 0.0,
            },
            [11671] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 81,
                rank                = 5,
                lvl_req             = 44,
                lvl_max             = 49,
                lvl_outdated        = 53,
				lvl_scaling			= 0.0,
            },
            [11672] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 111,
                rank                = 6,
                lvl_req             = 54,
                lvl_max             = 59,
                lvl_outdated        = 59,
				lvl_scaling			= 0.0,
            },
            [25311] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 137,
                rank                = 7,
                lvl_req             = 60,
                lvl_max             = 64,
                lvl_outdated        = 64,
				lvl_scaling			= 0.0,
            },
            [27216] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 150,
                rank                = 8,
                lvl_req             = 65,
                lvl_max             = 70,
                lvl_outdated        = 70,
				lvl_scaling			= 0.0,
            },
            [47812] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 164,
                rank                = 9,
                lvl_req             = 71,
                lvl_max             = 76,
                lvl_outdated        = 76,
				lvl_scaling			= 0.0,
            },
            [47813] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 180,
                rank                = 10,
                lvl_req             = 77,
                lvl_max             = 82,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            -- drain life
            [689] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 10,
                over_time_tick_freq = 1,
                over_time_duration  = 5.0,
                cast_time           = 5.0,
                rank                = 1,
                lvl_req             = 14,
                lvl_max             = 19,
                lvl_outdated        = 21,
                cost_base_percent   = 0.17,
                flags               = 0,
                school              = magic_school.shadow,
                coef                = 0.0,
                over_time_coef      = 0.1430,
				lvl_scaling			= 0.0,
            },
            [699] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 17,
                rank                = 2,
                lvl_req             = 22,
                lvl_max             = 27,
                lvl_outdated        = 29,
				lvl_scaling			= 0.0,
            },
            [709] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 29,
                rank                = 3,
                lvl_req             = 30,
                lvl_max             = 35,
                lvl_outdated        = 37,
				lvl_scaling			= 0.0,
            },
            [7651] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 41,
                rank                = 4,
                lvl_req             = 38,
                lvl_max             = 43,
                lvl_outdated        = 45,
				lvl_scaling			= 0.0,
            },
            [11699] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 55,
                rank                = 5,
                lvl_req             = 46,
                lvl_max             = 51,
                lvl_outdated        = 53,
				lvl_scaling			= 0.0,
            },
            [11700] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 71,
                rank                = 6,
                lvl_req             = 54,
                lvl_max             = 59,
                lvl_outdated        = 61,
				lvl_scaling			= 0.0,
            },
            [27219] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 87,
                rank                = 7,
                lvl_req             = 62,
                lvl_max             = 67,
                lvl_outdated        = 68,
				lvl_scaling			= 0.0,
            },
            [27220] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 108,
                rank                = 8,
                lvl_req             = 69,
                lvl_max             = 74,
                lvl_outdated        = 77,
				lvl_scaling			= 0.0,
            },
            [47857] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 133,
                rank                = 9,
                lvl_req             = 78,
                lvl_max             = 83,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            -- drain soul
            [1120] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 11,
                over_time_tick_freq = 3,
                over_time_duration  = 15.0,
                cast_time           = 15.0,
                rank                = 1,
                lvl_req             = 10,
                lvl_max             = 23,
                lvl_outdated        = 23,
                cost_base_percent   = 0.14,
                flags               = 0,
                school              = magic_school.shadow,
                coef                = 0.0,
                over_time_coef      = 0.4290,
				lvl_scaling			= 0.0,
            },
            [8288] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 31,
                rank                = 2,
                lvl_req             = 24,
                lvl_max             = 37,
                lvl_outdated        = 37,
				lvl_scaling			= 0.0,
            },
            [8289] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 59,
                rank                = 3,
                lvl_req             = 38,
                lvl_max             = 51,
                lvl_outdated        = 51,
				lvl_scaling			= 0.0,
            },
            [11675] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 91,
                rank                = 4,
                lvl_req             = 52,
                lvl_max             = 66,
                lvl_outdated        = 66,
				lvl_scaling			= 0.0,
            },
            [27217] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 124,
                rank                = 5,
                lvl_req             = 67,
                lvl_max             = 76,
                lvl_outdated        = 76,
				lvl_scaling			= 0.0,
            },
            [47855] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 142,
                rank                = 6,
                lvl_req             = 77,
                lvl_max             = 86,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            --shadow bolt
            [686] = {
                base_min            = 12,
                base_max            = 16,
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 3.0,
                rank                = 1,
                lvl_req             = 1,
                lvl_max             = 5,
                lvl_outdated        = 5,
                cost_base_percent   = 0.17,
                flags               = 0,
                school              = magic_school.shadow,
                coef                = 0.8571,
                over_time_coef      = 0.0,
				lvl_scaling			= 0.4,
            },
            [695] = {
                base_min            = 23,
                base_max            = 29,
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 6,
                lvl_max             = 11,
                lvl_outdated        = 11,
				lvl_scaling			= 0.6,
            },
            [705] = {
                base_min            = 48,
                base_max            = 56,
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 12,
                lvl_max             = 17,
                lvl_outdated        = 19,
				lvl_scaling			= 0.9,
            },
            [1088] = {
                base_min            = 86,
                base_max            = 98,
                over_time           = 0.0,
                rank                = 4,
                lvl_req             = 20,
                lvl_max             = 25,
                lvl_outdated        = 27,
				lvl_scaling			= 1.2,
            },
            [1106] = {
                base_min            = 142,
                base_max            = 162,
                over_time           = 0.0,
                rank                = 5,
                lvl_req             = 28,
                lvl_max             = 33,
                lvl_outdated        = 35,
				lvl_scaling			= 1.6,
            },
            [7641] = {
                base_min            = 204,
                base_max            = 230,
                over_time           = 0.0,
                rank                = 6,
                lvl_req             = 36,
                lvl_max             = 41,
                lvl_outdated        = 43,
				lvl_scaling			= 1.9,
            },
            [11659] = {
                base_min            = 281,
                base_max            = 315,
                over_time           = 0.0,
                rank                = 7,
                lvl_req             = 44,
                lvl_max             = 49,
                lvl_outdated        = 51,
				lvl_scaling			= 2.3,
            },
            [11660] = {
                base_min            = 360,
                base_max            = 402,
                over_time           = 0.0,
                rank                = 8,
                lvl_req             = 52,
                lvl_max             = 57,
                lvl_outdated        = 59,
				lvl_scaling			= 2.6,
            },
            [11661] = {
                base_min            = 455,
                base_max            = 507,
                over_time           = 0.0,
                rank                = 9,
                lvl_req             = 60,
                lvl_max             = 65,
                lvl_outdated        = 60,
				lvl_scaling			= 3,
            },
            [25307] = {
                base_min            = 482,
                base_max            = 538,
                over_time           = 0.0,
                rank                = 10,
                lvl_req             = 60,
                lvl_max             = 65,
                lvl_outdated        = 68,
				lvl_scaling			= 3.1,
            },
            [27209] = {
                base_min            = 541,
                base_max            = 603,
                over_time           = 0.0,
                rank                = 11,
                lvl_req             = 69,
                lvl_max             = 74,
                lvl_outdated        = 73,
				lvl_scaling			= 3.4,
            },
            [47808] = {
                base_min            = 596,
                base_max            = 664,
                over_time           = 0.0,
                rank                = 12,
                lvl_req             = 74,
                lvl_max             = 79,
                lvl_outdated        = 78,
				lvl_scaling			= 3.8,
            },
            [47809] = {
                base_min            = 690,
                base_max            = 770,
                over_time           = 0.0,
                rank                = 13,
                lvl_req             = 79,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 4.4,
            },
            -- searing pain
            [5676] = {
                base_min            = 34,
                base_max            = 42,
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 18,
                lvl_max             = 24,
                lvl_outdated        = 25,
                cost_base_percent   = 0.08,
                flags               = 0,
                school              = magic_school.fire,
                coef                = 0.4286,
                over_time_coef      = 0.0,
				lvl_scaling			= 0.7,
            },
            [17919] = {
                base_min            = 59,
                base_max            = 71,
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 26,
                lvl_max             = 32,
                lvl_outdated        = 33,
				lvl_scaling			= 1,
            },
            [17920] = {
                base_min            = 86,
                base_max            = 104,
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 34,
                lvl_max             = 40,
                lvl_outdated        = 41,
				lvl_scaling			= 1.2,
            },
            [17921] = {
                base_min            = 122,
                base_max            = 146,
                over_time           = 0.0,
                rank                = 4,
                lvl_req             = 42,
                lvl_max             = 48,
                lvl_outdated        = 49,
				lvl_scaling			= 1.5,
            },
            [17922] = {
                base_min            = 158,
                base_max            = 188,
                over_time           = 0.0,
                rank                = 5,
                lvl_req             = 50,
                lvl_max             = 56,
                lvl_outdated        = 57,
				lvl_scaling			= 1.7,
            },
            [17923] = {
                base_min            = 204,
                base_max            = 240,
                over_time           = 0.0,
                rank                = 6,
                lvl_req             = 58,
                lvl_max             = 64,
                lvl_outdated        = 64,
				lvl_scaling			= 2,
            },
            [27210] = {
                base_min            = 243,
                base_max            = 287,
                over_time           = 0.0,
                rank                = 7,
                lvl_req             = 65,
                lvl_max             = 69,
                lvl_outdated        = 69,
				lvl_scaling			= 2.4,
            },
            [30459] = {
                base_min            = 270,
                base_max            = 320,
                over_time           = 0.0,
                rank                = 8,
                lvl_req             = 70,
                lvl_max             = 76,
                lvl_outdated        = 73,
				lvl_scaling			= 2.8,
            },
            [47814] = {
                base_min            = 295,
                base_max            = 349,
                over_time           = 0.0,
                rank                = 9,
                lvl_req             = 74,
                lvl_max             = 78,
                lvl_outdated        = 78,
				lvl_scaling			= 4.1,
            },
            [47815] = {
                base_min            = 343,
                base_max            = 405,
                over_time           = 0.0,
                rank                = 10,
                lvl_req             = 79,
                lvl_max             = 83,
                lvl_outdated        = 80,
				lvl_scaling			= 4.8,
            },
            -- soul fire
            [6353] = {
                base_min            = 623,
                base_max            = 783,
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 6.0,
                rank                = 1,
                lvl_req             = 48,
                lvl_max             = 54,
                lvl_outdated        = 55,
                cost_base_percent   = 0.09,
                flags               = 0,
                school              = magic_school.fire,
                coef                = 1.15,
                over_time_coef      = 0.0,
				lvl_scaling			= 2.9,
            },
            [17924] = {
                base_min            = 703,
                base_max            = 881,
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 56,
                lvl_max             = 62,
                lvl_outdated        = 63,
				lvl_scaling			= 3.1,
            },
            [27211] = {
                base_min            = 839,
                base_max            = 1051,
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 64,
                lvl_max             = 68,
                lvl_outdated        = 69,
				lvl_scaling			= 3.5,
            },
            [30545] = {
                base_min            = 1003,
                base_max            = 1257,
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 70,
                lvl_max             = 74,
                lvl_outdated        = 74,
				lvl_scaling			= 3.8,
            },
            [47824] = {
                base_min            = 1137,
                base_max            = 1423,
                over_time           = 0.0,
                rank                = 5,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 4,
            },
            [47825] = {
                base_min            = 1323,
                base_max            = 1657,
                over_time           = 0.0,
                rank                = 6,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 5,
            },
            -- hellfire
            [1949] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 83,
                over_time_tick_freq = 1,
                over_time_duration  = 15.0,
                cast_time           = 15.0,
                rank                = 3,
                lvl_req             = 30,
                lvl_max             = 40,
                lvl_outdated        = 41,
                cost_base_percent   = 0.64,
                flags               = spell_flags.aoe,
                school              = magic_school.fire,
                coef                = 0.0,
                over_time_coef      = 0.143,
				lvl_scaling			= 0.4,
            },
            [11683] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 139,
                rank                = 2,
                lvl_req             = 42,
                lvl_max             = 52,
                lvl_outdated        = 53,
				lvl_scaling			= 0.5,
            },
            [11684] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 208,
                rank                = 3,
                lvl_req             = 54,
                lvl_max             = 64,
                lvl_outdated        = 67,
				lvl_scaling			= 0.7,
            },
            [27213] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 306,
                rank                = 4,
                lvl_req             = 68,
                lvl_max             = 78,
                lvl_outdated        = 77,
				lvl_scaling			= 0.8,
            },
            [47823] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 451,
                rank                = 5,
                lvl_req             = 78,
                lvl_max             = 88,
                lvl_outdated        = 80,
				lvl_scaling			= 1,
            },
            -- rain of fire
            [5740] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 60,
                over_time_tick_freq = 2,
                over_time_duration  = 8.0,
                cast_time           = 8.0,
                rank                = 1,
                lvl_req             = 20,
                lvl_max             = 25,
                lvl_outdated        = 33,
                cost_base_percent   = 0.57,
                flags               = bit.bor(spell_flags.aoe, spell_flags.over_time_crit),
                school              = magic_school.fire,
                coef                = 0.0,
                over_time_coef      = 0.286,
				lvl_scaling			= 0.3,
            },
            [6219] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 136,
                rank                = 2,
                lvl_req             = 34,
                lvl_max             = 39,
                lvl_outdated        = 45,
				lvl_scaling			= 0.4,
            },
            [11677] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 220,
                rank                = 3,
                lvl_req             = 46,
                lvl_max             = 51,
                lvl_outdated        = 57,
				lvl_scaling			= 0.5,
            },
            [11678] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 321,
                rank                = 4,
                lvl_req             = 58,
                lvl_max             = 63,
                lvl_outdated        = 68,
				lvl_scaling			= 0.6,
            },
            [27212] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 452,
                rank                = 5,
                lvl_req             = 69,
                lvl_max             = 74,
                lvl_outdated        = 71,
				lvl_scaling			= 0.8,
            },
            [47819] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 538,
                rank                = 6,
                lvl_req             = 72,
                lvl_max             = 76,
                lvl_outdated        = 78,
				lvl_scaling			= 1.6,
            },
            [47820] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 675,
                rank                = 7,
                lvl_req             = 79,
                lvl_max             = 83,
                lvl_outdated        = 80,
				lvl_scaling			= 2,
            },
            -- immolate
            [348] = {
                base_min            = 8,
                base_max            = 8,
                over_time           = 4,
                over_time_tick_freq = 3,
                over_time_duration  = 15.0,
                cast_time           = 2.0,
                rank                = 1,
                lvl_req             = 1,
                lvl_max             = 5,
                lvl_outdated        = 9,
                cost_base_percent   = 0.17,
                flags               = 0,
                school              = magic_school.fire,
                coef                = 0.2,
                over_time_coef      = 0.2,
				lvl_scaling			= 0.6,
            },
            [707] = {
                base_min            = 19,
                base_max            = 19,
                over_time           = 8,
                rank                = 2,
                lvl_req             = 10,
                lvl_max             = 15,
                lvl_outdated        = 19,
				lvl_scaling			= 0.9,
            },
            [1094] = {
                base_min            = 45,
                base_max            = 45,
                over_time           = 18,
                rank                = 3,
                lvl_req             = 20,
                lvl_max             = 25,
                lvl_outdated        = 29,
				lvl_scaling			= 1.5,
            },
            [2941] = {
                base_min            = 90,
                base_max            = 90,
                over_time           = 33,
                rank                = 4,
                lvl_req             = 30,
                lvl_max             = 35,
                lvl_outdated        = 39,
				lvl_scaling			= 2.1,
            },
            [11665] = {
                base_min            = 134,
                base_max            = 134,
                over_time           = 51,
                rank                = 5,
                lvl_req             = 40,
                lvl_max             = 45,
                lvl_outdated        = 49,
				lvl_scaling			= 2.7,
            },
            [11667] = {
                base_min            = 192,
                base_max            = 192,
                over_time           = 73,
                rank                = 6,
                lvl_req             = 50,
                lvl_max             = 55,
                lvl_outdated        = 59,
				lvl_scaling			= 3.2,
            },
            [11668] = {
                base_min            = 258,
                base_max            = 258,
                over_time           = 97,
                rank                = 7,
                lvl_req             = 60,
                lvl_max             = 65,
                lvl_outdated        = 60,
				lvl_scaling			= 3.8,
            },
            [25309] = {
                base_min            = 279,
                base_max            = 279,
                over_time           = 102,
                rank                = 8,
                lvl_req             = 60,
                lvl_max             = 65,
                lvl_outdated        = 68,
				lvl_scaling			= 3.9,
            },
            [27215] = {
                base_min            = 327,
                base_max            = 327,
                over_time           = 123,
                rank                = 9,
                lvl_req             = 69,
                lvl_max             = 74,
                lvl_outdated        = 74,
				lvl_scaling			= 4.3,
            },
            [47810] = {
                base_min            = 370,
                base_max            = 370,
                over_time           = 139,
                rank                = 10,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 5,
            },
            [47811] = {
                base_min            = 460,
                base_max            = 460,
                over_time           = 157,
                rank                = 11,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 6,
            },
            -- shadowburn
            [17877] = {
                base_min            = 87,
                base_max            = 99,
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 20,
                lvl_max             = 24,
                lvl_outdated        = 23,
                cost_base_percent   = 0.2,
                flags               = spell_flags.cd,
                school              = magic_school.shadow,
                coef                = 0.4286,
                over_time_coef      = 0.0,
				lvl_scaling			= 1.2,
            },
            [18867] = {
                base_min            = 115,
                base_max            = 131,
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 24,
                lvl_max             = 30,
                lvl_outdated        = 31,
				lvl_scaling			= 1.4,
            },
            [18868] = {
                base_min            = 186,
                base_max            = 210,
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 32,
                lvl_max             = 38,
                lvl_outdated        = 39,
				lvl_scaling			= 1.8,
            },
            [18869] = {
                base_min            = 261,
                base_max            = 293,
                over_time           = 0.0,
                rank                = 4,
                lvl_req             = 40,
                lvl_max             = 46,
                lvl_outdated        = 47,
				lvl_scaling			= 2.2,
            },
            [18870] = {
                base_min            = 350,
                base_max            = 392,
                over_time           = 0.0,
                rank                = 5,
                lvl_req             = 48,
                lvl_max             = 54,
                lvl_outdated        = 55,
				lvl_scaling			= 2.6,
            },
            [18871] = {
                base_min            = 450,
                base_max            = 502,
                over_time           = 0.0,
                rank                = 6,
                lvl_req             = 56,
                lvl_max             = 62,
                lvl_outdated        = 62,
				lvl_scaling			= 3,
            },
            [27263] = {
                base_min            = 518,
                base_max            = 578,
                over_time           = 0.0,
                rank                = 7,
                lvl_req             = 63,
                lvl_max             = 69,
                lvl_outdated        = 69,
				lvl_scaling			= 3.4,
            },
            [30546] = {
                base_min            = 597,
                base_max            = 665,
                over_time           = 0.0,
                rank                = 8,
                lvl_req             = 70,
                lvl_max             = 74,
                lvl_outdated        = 74,
				lvl_scaling			= 3.8,
            },
            [47826] = {
                base_min            = 662,
                base_max            = 738,
                over_time           = 0.0,
                rank                = 9,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 5,
            },
            [47827] = {
                base_min            = 775,
                base_max            = 865,
                over_time           = 0.0,
                rank                = 10,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 6,
            },
            -- curse of doom
            [603] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 3200,
                over_time_tick_freq = 60.0,
                over_time_duration  = 60.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 60,
                lvl_max             = 69,
                lvl_outdated        = 69,
                cost_base_percent   = 0.15,
                flags               = bit.bor(spell_flags.cd, spell_flags.curse),
                school              = magic_school.shadow,
                coef                = 0.0,
                over_time_coef      = 2.0,
				lvl_scaling			= 0.0,
            },
            [30910] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 4200,
                rank                = 2,
                lvl_req             = 70,
                lvl_max             = 0,
                lvl_outdated        = 79,
				lvl_scaling			= 0.0,
            },
            [47867] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 7300,
                rank                = 3,
                lvl_req             = 80,
                lvl_max             = 0,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            -- unstable affliction
            [30108] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 110,
                over_time_tick_freq = 3,
                over_time_duration  = 15.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 50,
                lvl_max             = 59,
                lvl_outdated        = 59,
                cost_base_percent   = 0.15,
                flags               = 0,
                school              = magic_school.shadow,
                coef                = 0.0,
                over_time_coef      = 0.2,
				lvl_scaling			= 0.0,
            },
            [30404] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 140,
                rank                = 2,
                lvl_req             = 60,
                lvl_max             = 69,
                lvl_outdated        = 69,
				lvl_scaling			= 0.0,
            },
            [30405] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 175,
                rank                = 3,
                lvl_req             = 70,
                lvl_max             = 74,
                lvl_outdated        = 74,
				lvl_scaling			= 0.0,
            },
            [47841] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 197,
                rank                = 4,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 0.0,
            },
            [47843] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 230,
                rank                = 5,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            -- seed of corruption
            [27243] = {
                base_min            = 1110.0,
                base_max            = 1290.0, 
                over_time           = 174,
                over_time_tick_freq = 3,
                over_time_duration  = 18.0,
                cast_time           = 2.0,
                rank                = 1,
                lvl_req             = 70,
                lvl_max             = 0,
                lvl_outdated        = 74,
                cost_base_percent   = 0.34,
                flags               = spell_flags.cd,
                school              = magic_school.shadow,
                coef                = 0.2129,
                over_time_coef      = 0.25,
				lvl_scaling			= 0.0,
            },
            [47835] = {
                base_min            = 1383.0,
                base_max            = 1608.0, 
                over_time           = 216,
                rank                = 2,
                lvl_req             = 75,
                lvl_max             = 0,
                lvl_outdated        = 79,
				lvl_scaling			= 0.0,
            },
            [47836] = {
                base_min            = 1633.0,
                base_max            = 1897.0, 
                over_time           = 253,
                rank                = 3,
                lvl_req             = 80,
                lvl_max             = 0,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            -- shadowflame
            [47897] = {
                base_min            = 520,
                base_max            = 568,
                over_time           = 544,
                over_time_tick_freq = 2,
                over_time_duration  = 8.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 75,
                lvl_max             = 83,
                lvl_outdated        = 79,
                cost_base_percent   = 0.25,
                flags               = spell_flags.cd,
                school              = magic_school.shadow,
                coef                = 0.1064,
                over_time_coef      = 0.0667,
				lvl_scaling			= 2,
            },
            [61290] = {
                base_min            = 615,
                base_max            = 671,
                over_time           = 644,
                rank                = 1,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 2.5,
            },
            -- haunt
            [48181] = {
                base_min            = 405,
                base_max            = 473,
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 60,
                lvl_max             = 64,
                lvl_outdated        = 69,
                cost_base_percent   = 0.12,
                flags               = spell_flags.cd,
                school              = magic_school.shadow,
                coef                = 0.4286,
                over_time_coef      = 0.0,
				lvl_scaling			= 0.0,
            },
            [59161] = {
                base_min            = 487,
                base_max            = 569,
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 70,
                lvl_max             = 0,
                lvl_outdated        = 74,
				lvl_scaling			= 0.0,
            },
            [59163] = {
                base_min            = 550,
                base_max            = 642,
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 75,
                lvl_max             = 0,
                lvl_outdated        = 79,
				lvl_scaling			= 0.0,
            },
            [59164] = {
                base_min            = 645,
                base_max            = 753,
                over_time           = 0.0,
                rank                = 4,
                lvl_req             = 80,
                lvl_max             = 0,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            -- incinerate
            [29722] = {
                base_min            = 403,
                base_max            = 467,
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 2.5,
                rank                = 1,
                lvl_req             = 64,
                lvl_max             = 69,
                lvl_outdated        = 69,
                cost_base_percent   = 0.14,
                flags               = 0,
                school              = magic_school.fire,
                coef                = 0.7143,
                over_time_coef      = 0.0,
				lvl_scaling			= 2.6,
            },
            [32231] = {
                base_min            = 444,
                base_max            = 514,
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 70,
                lvl_max             = 74,
                lvl_outdated        = 73,
				lvl_scaling			= 2.8,
            },
            [47837] = {
                base_min            = 485,
                base_max            = 563,
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 74,
                lvl_max             = 78,
                lvl_outdated        = 79,
				lvl_scaling			= 3,
            },
            [47838] = {
                base_min            = 582,
                base_max            = 676,
                over_time           = 0.0,
                rank                = 4,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 4,
            },
            -- inferno
            -- TODO: how does this scale??
            [1122] = {
                base_min            = 200,
                base_max            = 200, 
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 50,
                lvl_max             = 0,
                lvl_outdated        = 80,
                cost_base_percent   = 0.80,
                flags               = bit.bor(spell_flags.aoe, spell_flags.cd),
                school              = magic_school.fire,
                coef                = 1.0,
                over_time_coef      = 0.0,
				lvl_scaling			= 1,
            },
            -- shadowfury
            [30283] = {
                base_min            = 343,
                base_max            = 407,
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 50,
                lvl_max             = 59,
                lvl_outdated        = 59,
                cost_base_percent   = 0.27,
                flags               = bit.bor(spell_flags.aoe, spell_flags.cd, spell_flags.snare),
                school              = magic_school.shadow,
                coef                = 0.1936,
                over_time_coef      = 0.0,
				lvl_scaling			= 1.6,
            },
            [30413] = {
                base_min            = 459,
                base_max            = 547,
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 60,
                lvl_max             = 69,
                lvl_outdated        = 69,
				lvl_scaling			= 1.9,
            },
            [30414] = {
                base_min            = 612,
                base_max            = 728,
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 70,
                lvl_max             = 74,
                lvl_outdated        = 74,
				lvl_scaling			= 2.2,
            },
            [47846] = {
                base_min            = 822,
                base_max            = 978,
                over_time           = 0.0,
                rank                = 4,
                lvl_req             = 75,
                lvl_max             = 79,
                lvl_outdated        = 79,
				lvl_scaling			= 3,
            },
            [47847] = {
                base_min            = 968,
                base_max            = 1152,
                over_time           = 0.0,
                rank                = 5,
                lvl_req             = 80,
                lvl_max             = 84,
                lvl_outdated        = 80,
				lvl_scaling			= 4,
            },
            -- chaos bolt
            [50796] = {
                base_min            = 837,
                base_max            = 1061,
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 2.5,
                rank                = 1,
                lvl_req             = 60,
                lvl_max             = 65,
                lvl_outdated        = 69,
                cost_base_percent   = 0.07,
                flags               = spell_flags.cd,
                school              = magic_school.fire,
                coef                = 0.7142,
                over_time_coef      = 0.0,
				lvl_scaling			= 5.5,
            },
            [59170] = {
                base_min            = 1077,
                base_max            = 1367,
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 70,
                lvl_max             = 0,
                lvl_outdated        = 74,
				lvl_scaling			= 5.5,
            },
            [59171] = {
                base_min            = 1217,
                base_max            = 1545,
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 75,
                lvl_max             = 0,
                lvl_outdated        = 79,
				lvl_scaling			= 5.5,
            },
            [59172] = {
                base_min            = 1429,
                base_max            = 1813,
                over_time           = 0.0,
                rank                = 4,
                lvl_req             = 80,
                lvl_max             = 0,
                lvl_outdated        = 80,
				lvl_scaling			= 5.5,
            },
            -- health funnel
            [755] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 12,
                over_time_tick_freq = 1,
                over_time_duration  = 10.0,
                cast_time           = 10.0,
                rank                = 1,
                lvl_req             = 12,
                lvl_max             = 0,
                lvl_outdated        = 19,
                cost_base_percent   = 0.0,
                flags               = spell_flags.heal,
                school              = magic_school.shadow,
                coef                = 0.0,
                over_time_coef      = 0.548,
				lvl_scaling			= 5,
            },
            [3698] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 24,
                rank                = 2,
                lvl_req             = 20,
                lvl_max             = 0,
                lvl_outdated        = 27,
				lvl_scaling			= 10,
            },
            [3699] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 43,
                rank                = 3,
                lvl_req             = 28,
                lvl_max             = 0,
                lvl_outdated        = 35,
				lvl_scaling			= 17,
            },
            [3700] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 64,
                rank                = 4,
                lvl_req             = 36,
                lvl_max             = 0,
                lvl_outdated        = 43,
				lvl_scaling			= 24,
            },
            [11693] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 89,
                rank                = 5,
                lvl_req             = 44,
                lvl_max             = 0,
                lvl_outdated        = 51,
				lvl_scaling			= 33,
            },
            [11694] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 119,
                rank                = 6,
                lvl_req             = 52,
                lvl_max             = 0,
                lvl_outdated        = 59,
				lvl_scaling			= 42,
            },
            [11695] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 153,
                rank                = 7,
                lvl_req             = 60,
                lvl_max             = 0,
                lvl_outdated        = 66,
				lvl_scaling			= 52,
            },
            [27259] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 188,
                rank                = 8,
                lvl_req             = 67,
                lvl_max             = 0,
                lvl_outdated        = 75,
				lvl_scaling			= 65,
            },
            [47856] = {
                base_min            = 0.0,
                base_max            = 0.0, 
                over_time           = 520,
                rank                = 9,
                lvl_req             = 76,
                lvl_max             = 0,
                lvl_outdated        = 80,
				lvl_scaling			= 182,
            },
            -- shadow ward
            [6229] = {
                base_min            = 290,
                base_max            = 290,
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 32,
                lvl_max             = 41,
                lvl_outdated        = 41,
                cost_base_percent   = 0.12,
                flags               = spell_flags.absorb,
                school              = magic_school.shadow,
                coef                = 0.807,
                over_time_coef      = 0.0,
				lvl_scaling			= 0.0,
            },
            [11739] = {
                base_min            = 470,
                base_max            = 470,
                over_time           = 0.0,
                rank                = 2,
                lvl_req             = 42,
                lvl_max             = 51,
                lvl_outdated        = 51,
				lvl_scaling			= 0.0,
            },
            [11740] = {
                base_min            = 675,
                base_max            = 675,
                over_time           = 0.0,
                rank                = 3,
                lvl_req             = 52,
                lvl_max             = 59,
                lvl_outdated        = 59,
				lvl_scaling			= 0.0,
            },
            [28610] = {
                base_min            = 875,
                base_max            = 875,
                over_time           = 0.0,
                rank                = 4,
                lvl_req             = 60,
                lvl_max             = 69,
                lvl_outdated        = 71,
				lvl_scaling			= 0.0,
            },
            [47890] = {
                base_min            = 2750,
                base_max            = 2750,
                over_time           = 0.0,
                rank                = 5,
                lvl_req             = 72,
                lvl_max             = 77,
                lvl_outdated        = 77,
				lvl_scaling			= 0.0,
            },
            [47891] = {
                base_min            = 3300,
                base_max            = 3300,
                over_time           = 0.0,
                rank                = 6,
                lvl_req             = 78,
                lvl_max             = 83,
                lvl_outdated        = 80,
				lvl_scaling			= 0.0,
            },
            -- immolation aura
            [50589] = {
                base_min            = 0,
                base_max            = 0,
                over_time           = 251,
                over_time_tick_freq = 1,
                over_time_duration  = 15.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 60,
                lvl_max             = 0,
                lvl_outdated        = 80,
                cost_base_percent   = 0.64,
                flags               = spell_flags.cd,
                school              = magic_school.fire,
                coef                = 0.0,
                over_time_coef      = 0.143,
				lvl_scaling			= 11.5,
            },
            -- shadow cleave
            [50581] = {
                base_min            = 110,
                base_max            = 110,
                over_time           = 0.0,
                over_time_tick_freq = 0,
                over_time_duration  = 0.0,
                cast_time           = 1.5,
                rank                = 1,
                lvl_req             = 60,
                lvl_max             = 0,
                lvl_outdated        = 80,
                cost_base_percent   = 0.04,
                flags               = spell_flags.cd,
                school              = magic_school.shadow,
                coef                = 0.213,
                over_time_coef      = 0.0,
				lvl_scaling			= 0.0,
            },
        };
    end
    return {};
end

local spells = create_spells();
for k, v in pairs(spells) do
    local name, _, _, _, _, _, _ ,_  = GetSpellInfo(k)
    -- rank1 contains some general fields that we write to all ranks
    local rank1_of_spell = localized_spell_names_to_id[name]
    local spell_data = spells[rank1_of_spell];

    v.over_time_tick_freq = spell_data.over_time_tick_freq;
    v.over_time_duration  = spell_data.over_time_duration;
    v.cast_time           = spell_data.cast_time;
    v.cost_base_percent   = spell_data.cost_base_percent;
    v.flags               = spell_data.flags;
    v.school              = spell_data.school;
    v.coef                = spell_data.coef;
    v.over_time_coef      = spell_data.over_time_coef;
    v.base_id             = rank1_of_spell;
    if v.healing_version then
        v.healing_version.over_time_tick_freq = spell_data.healing_version.over_time_tick_freq;
        v.healing_version.over_time_duration  = spell_data.healing_version.over_time_duration;
        v.healing_version.cast_time           = spell_data.healing_version.cast_time;
        v.healing_version.cost_base_percent   = spell_data.healing_version.cost_base_percent;
        v.healing_version.flags               = spell_data.healing_version.flags;
        v.healing_version.school              = spell_data.healing_version.school;
        v.healing_version.coef                = spell_data.healing_version.coef;
        v.healing_version.over_time_coef      = spell_data.healing_version.over_time_coef;
        v.healing_version.base_id             = rank1_of_spell;
    end
end
-- exceptions to various spell ranks
if class == "DRUID" then
    -- healing touch
    spells[5185].cost_base_perc = 0.17;
    spells[5185].cast_time = 1.5;

    spells[5186].cost_base_perc = 0.22;
    spells[5186].cast_time = 2.0;

    spells[5187].cost_base_perc = 0.28;
    spells[5187].cast_time = 2.5;

    spells[5188].cost_base_perc = 0.33;
    spells[5188].cast_time = 3.0;
    -- moonfire
    spells[8921].cost_base_perc = 0.18;

    -- wrath
    spells[5176].cost_base_perc = 0.08;
    spells[5176].cast_time = 1.5;

    spells[5177].cost_base_perc = 0.09;
    spells[5177].cast_time = 1.7;

    spells[5178].cost_base_perc = 0.11;
    spells[5178].cast_time = 2.0;

    spells[8921].over_time_duration = 9;

    -- entangling roots
    spells[339].over_time_duration = 12.0;
    spells[1062].over_time_duration = 15.0;
    spells[5195].over_time_duration = 18.0;
    spells[5196].over_time_duration = 21.0;
    spells[9852].over_time_duration = 24.0;

elseif class == "PRIEST" then
    -- lesser heal
    spells[2050].cost_base_perc = 0.16;
    spells[2050].cast_time = 1.5;

    spells[2052].cost_base_perc = 0.21;
    spells[2052].cast_time = 2.0;

    spells[2053].cost_base_perc = 0.27;
    spells[2053].cast_time = 2.5;

elseif class == "PALADIN" then

elseif class == "SHAMAN" then
    -- healing wave
    spells[331].cost_base_perc = 0.13;
    spells[331].cast_time = 1.5;

    spells[332].cost_base_perc = 0.17;
    spells[332].cast_time = 2.0;

    spells[547].cost_base_perc = 0.21;
    spells[547].cast_time = 2.5;

    -- lightning bolt
    spells[403].cost_base_perc = 0.06;
    spells[403].cast_time = 1.5;

    spells[529].cost_base_perc = 0.08;
    spells[529].cast_time = 2.0;

    -- searing totem
    spells[3599].over_time_duration = 30.0;
    spells[6363].over_time_duration = 35.0;
    spells[6364].over_time_duration = 40.0;
    spells[6365].over_time_duration = 45.0;
    spells[10437].over_time_duration = 50.0;
    spells[10438].over_time_duration = 55.0;

elseif class == "MAGE" then

    -- frostbolt
    spells[116].cost_base_perc = 0.06;
    spells[116].cast_time = 1.5;

    spells[205].cost_base_perc = 0.07;
    spells[205].cast_time = 1.8;

    spells[837].cost_base_perc = 0.08;
    spells[837].cast_time = 2.2;

    spells[7322].cost_base_perc = 0.1;
    spells[7322].cast_time = 2.6;

    -- fireball
    spells[133].cost_base_perc = 0.08;
    spells[133].cast_time = 1.5;
    spells[133].over_time_duration = 4.0;

    spells[143].cost_base_perc = 0.11;
    spells[143].cast_time = 2.0;
    spells[143].over_time_duration = 6.0;

    spells[145].cost_base_perc = 0.14;
    spells[145].cast_time = 2.5;
    spells[145].over_time_duration = 6.0;

    spells[3140].cost_base_perc = 0.16;
    spells[3140].cast_time = 3.0;
    spells[3140].over_time_duration = 8.0;

    --flamestrike
    spells[42925].cost_base_perc = 0.3;
    spells[42926].cost_base_perc = 0.3;

    -- arcane missiles
    spells[5143].cast_time = 3.0;
    spells[5143].over_time_duration = 3.0;

    spells[5143].cast_time = 4.0;
    spells[5143].over_time_duration = 4.0;

elseif class == "WARLOCK" then

    -- corruption
    spells[172].over_time_duration = 12.0;
    spells[172].cost_base_perc = 0.9;

    spells[6222].over_time_duration = 15.0;
    spells[6222].cost_base_perc = 0.12;

    --shadow bolt
    spells[686].cast_time = 1.7;
    spells[686].cost_base_perc = 0.1;

    spells[695].cast_time = 2.2;
    spells[695].cost_base_perc = 0.12;

    spells[705].cast_time = 2.8;
    spells[705].cost_base_perc = 0.16;
end

local function spell_names_to_id(english_names)
    local base_ids = {};
    for k, v in pairs(english_names) do
        base_ids[k] = spell_name_to_id[v];
    end
    return base_ids;
end

local addonName, addonTable = ...;
addonTable.spells = spells;
addonTable.spell_name_to_id = spell_name_to_id;
addonTable.spell_names_to_id = spell_names_to_id;
addonTable.magic_school = magic_school;
addonTable.spell_flags = spell_flags;

